"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runnerJobsRouter = void 0;
const tslib_1 = require("tslib");
const express_1 = tslib_1.__importDefault(require("express"));
const database_utils_1 = require("../../../helpers/database-utils");
const express_utils_1 = require("../../../helpers/express-utils");
const logger_1 = require("../../../helpers/logger");
const token_generator_1 = require("../../../helpers/token-generator");
const constants_1 = require("../../../initializers/constants");
const database_1 = require("../../../initializers/database");
const runners_1 = require("../../../lib/runners");
const middlewares_1 = require("../../../middlewares");
const runners_2 = require("../../../middlewares/validators/runners");
const runner_job_1 = require("../../../models/runner/runner-job");
const models_1 = require("../../../../shared/models");
const postRunnerJobSuccessVideoFiles = (0, express_utils_1.createReqFiles)(['payload[videoFile]', 'payload[resolutionPlaylistFile]'], Object.assign(Object.assign({}, constants_1.MIMETYPES.VIDEO.MIMETYPE_EXT), constants_1.MIMETYPES.M3U8.MIMETYPE_EXT));
const runnerJobUpdateVideoFiles = (0, express_utils_1.createReqFiles)(['payload[videoChunkFile]', 'payload[resolutionPlaylistFile]', 'payload[masterPlaylistFile]'], Object.assign(Object.assign({}, constants_1.MIMETYPES.VIDEO.MIMETYPE_EXT), constants_1.MIMETYPES.M3U8.MIMETYPE_EXT));
const lTags = (0, logger_1.loggerTagsFactory)('api', 'runner');
const runnerJobsRouter = express_1.default.Router();
exports.runnerJobsRouter = runnerJobsRouter;
runnerJobsRouter.post('/jobs/request', middlewares_1.apiRateLimiter, (0, middlewares_1.asyncMiddleware)(runners_2.getRunnerFromTokenValidator), (0, middlewares_1.asyncMiddleware)(requestRunnerJob));
runnerJobsRouter.post('/jobs/:jobUUID/accept', middlewares_1.apiRateLimiter, (0, middlewares_1.asyncMiddleware)(runners_2.runnerJobGetValidator), runners_2.acceptRunnerJobValidator, (0, middlewares_1.asyncMiddleware)(runners_2.getRunnerFromTokenValidator), (0, middlewares_1.asyncMiddleware)(acceptRunnerJob));
runnerJobsRouter.post('/jobs/:jobUUID/abort', middlewares_1.apiRateLimiter, (0, middlewares_1.asyncMiddleware)(runners_2.jobOfRunnerGetValidator), runners_2.abortRunnerJobValidator, (0, middlewares_1.asyncMiddleware)(abortRunnerJob));
runnerJobsRouter.post('/jobs/:jobUUID/update', runnerJobUpdateVideoFiles, middlewares_1.apiRateLimiter, (0, middlewares_1.asyncMiddleware)(runners_2.jobOfRunnerGetValidator), runners_2.updateRunnerJobValidator, (0, middlewares_1.asyncMiddleware)(updateRunnerJobController));
runnerJobsRouter.post('/jobs/:jobUUID/error', (0, middlewares_1.asyncMiddleware)(runners_2.jobOfRunnerGetValidator), runners_2.errorRunnerJobValidator, (0, middlewares_1.asyncMiddleware)(errorRunnerJob));
runnerJobsRouter.post('/jobs/:jobUUID/success', postRunnerJobSuccessVideoFiles, middlewares_1.apiRateLimiter, (0, middlewares_1.asyncMiddleware)(runners_2.jobOfRunnerGetValidator), runners_2.successRunnerJobValidator, (0, middlewares_1.asyncMiddleware)(postRunnerJobSuccess));
runnerJobsRouter.post('/jobs/:jobUUID/cancel', middlewares_1.authenticate, (0, middlewares_1.ensureUserHasRight)(29), (0, middlewares_1.asyncMiddleware)(runners_2.runnerJobGetValidator), runners_2.cancelRunnerJobValidator, (0, middlewares_1.asyncMiddleware)(cancelRunnerJob));
runnerJobsRouter.get('/jobs', middlewares_1.authenticate, (0, middlewares_1.ensureUserHasRight)(29), middlewares_1.paginationValidator, middlewares_1.runnerJobsSortValidator, middlewares_1.setDefaultSort, middlewares_1.setDefaultPagination, (0, middlewares_1.asyncMiddleware)(listRunnerJobs));
function requestRunnerJob(req, res) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const runner = res.locals.runner;
        const availableJobs = yield runner_job_1.RunnerJobModel.listAvailableJobs();
        logger_1.logger.debug('Runner %s requests for a job.', runner.name, Object.assign({ availableJobs }, lTags(runner.name)));
        const result = {
            availableJobs: availableJobs.map(j => ({
                uuid: j.uuid,
                type: j.type,
                payload: j.payload
            }))
        };
        (0, runners_1.updateLastRunnerContact)(req, runner);
        return res.json(result);
    });
}
function acceptRunnerJob(req, res) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const runner = res.locals.runner;
        const runnerJob = res.locals.runnerJob;
        const newRunnerJob = yield (0, database_utils_1.retryTransactionWrapper)(() => {
            return database_1.sequelizeTypescript.transaction((transaction) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield runnerJob.reload({ transaction });
                if (runnerJob.state !== models_1.RunnerJobState.PENDING) {
                    res.fail({
                        type: "runner_job_not_in_pending_state",
                        message: 'This job is not in pending state anymore',
                        status: 409
                    });
                    return undefined;
                }
                runnerJob.state = models_1.RunnerJobState.PROCESSING;
                runnerJob.processingJobToken = (0, token_generator_1.generateRunnerJobToken)();
                runnerJob.startedAt = new Date();
                runnerJob.runnerId = runner.id;
                return runnerJob.save({ transaction });
            }));
        });
        if (!newRunnerJob)
            return;
        newRunnerJob.Runner = runner;
        const result = {
            job: Object.assign(Object.assign({}, newRunnerJob.toFormattedJSON()), { jobToken: newRunnerJob.processingJobToken })
        };
        (0, runners_1.updateLastRunnerContact)(req, runner);
        logger_1.logger.info('Remote runner %s has accepted job %s (%s)', runner.name, runnerJob.uuid, runnerJob.type, lTags(runner.name, runnerJob.uuid, runnerJob.type));
        return res.json(result);
    });
}
function abortRunnerJob(req, res) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const runnerJob = res.locals.runnerJob;
        const runner = runnerJob.Runner;
        const body = req.body;
        logger_1.logger.info('Remote runner %s is aborting job %s (%s)', runner.name, runnerJob.uuid, runnerJob.type, Object.assign({ reason: body.reason }, lTags(runner.name, runnerJob.uuid, runnerJob.type)));
        const RunnerJobHandler = (0, runners_1.getRunnerJobHandlerClass)(runnerJob);
        yield new RunnerJobHandler().abort({ runnerJob });
        (0, runners_1.updateLastRunnerContact)(req, runnerJob.Runner);
        return res.sendStatus(204);
    });
}
function errorRunnerJob(req, res) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const runnerJob = res.locals.runnerJob;
        const runner = runnerJob.Runner;
        const body = req.body;
        runnerJob.failures += 1;
        logger_1.logger.error('Remote runner %s had an error with job %s (%s)', runner.name, runnerJob.uuid, runnerJob.type, Object.assign({ errorMessage: body.message, totalFailures: runnerJob.failures }, lTags(runner.name, runnerJob.uuid, runnerJob.type)));
        const RunnerJobHandler = (0, runners_1.getRunnerJobHandlerClass)(runnerJob);
        yield new RunnerJobHandler().error({ runnerJob, message: body.message });
        (0, runners_1.updateLastRunnerContact)(req, runnerJob.Runner);
        return res.sendStatus(204);
    });
}
const jobUpdateBuilders = {
    'live-rtmp-hls-transcoding': (payload, files) => {
        var _a, _b, _c;
        return Object.assign(Object.assign({}, payload), { masterPlaylistFile: (_a = files['payload[masterPlaylistFile]']) === null || _a === void 0 ? void 0 : _a[0].path, resolutionPlaylistFile: (_b = files['payload[resolutionPlaylistFile]']) === null || _b === void 0 ? void 0 : _b[0].path, videoChunkFile: (_c = files['payload[videoChunkFile]']) === null || _c === void 0 ? void 0 : _c[0].path });
    }
};
function updateRunnerJobController(req, res) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const runnerJob = res.locals.runnerJob;
        const runner = runnerJob.Runner;
        const body = req.body;
        const payloadBuilder = jobUpdateBuilders[runnerJob.type];
        const updatePayload = payloadBuilder
            ? payloadBuilder(body.payload, req.files)
            : undefined;
        logger_1.logger.debug('Remote runner %s is updating job %s (%s)', runnerJob.Runner.name, runnerJob.uuid, runnerJob.type, Object.assign({ body, updatePayload }, lTags(runner.name, runnerJob.uuid, runnerJob.type)));
        const RunnerJobHandler = (0, runners_1.getRunnerJobHandlerClass)(runnerJob);
        yield new RunnerJobHandler().update({
            runnerJob,
            progress: req.body.progress,
            updatePayload
        });
        (0, runners_1.updateLastRunnerContact)(req, runnerJob.Runner);
        return res.sendStatus(204);
    });
}
const jobSuccessPayloadBuilders = {
    'vod-web-video-transcoding': (payload, files) => {
        return Object.assign(Object.assign({}, payload), { videoFile: files['payload[videoFile]'][0].path });
    },
    'vod-hls-transcoding': (payload, files) => {
        return Object.assign(Object.assign({}, payload), { videoFile: files['payload[videoFile]'][0].path, resolutionPlaylistFile: files['payload[resolutionPlaylistFile]'][0].path });
    },
    'vod-audio-merge-transcoding': (payload, files) => {
        return Object.assign(Object.assign({}, payload), { videoFile: files['payload[videoFile]'][0].path });
    },
    'video-studio-transcoding': (payload, files) => {
        return Object.assign(Object.assign({}, payload), { videoFile: files['payload[videoFile]'][0].path });
    },
    'live-rtmp-hls-transcoding': () => ({})
};
function postRunnerJobSuccess(req, res) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const runnerJob = res.locals.runnerJob;
        const runner = runnerJob.Runner;
        const body = req.body;
        const resultPayload = jobSuccessPayloadBuilders[runnerJob.type](body.payload, req.files);
        logger_1.logger.info('Remote runner %s is sending success result for job %s (%s)', runnerJob.Runner.name, runnerJob.uuid, runnerJob.type, Object.assign({ resultPayload }, lTags(runner.name, runnerJob.uuid, runnerJob.type)));
        const RunnerJobHandler = (0, runners_1.getRunnerJobHandlerClass)(runnerJob);
        yield new RunnerJobHandler().complete({ runnerJob, resultPayload });
        (0, runners_1.updateLastRunnerContact)(req, runnerJob.Runner);
        return res.sendStatus(204);
    });
}
function cancelRunnerJob(req, res) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const runnerJob = res.locals.runnerJob;
        logger_1.logger.info('Cancelling job %s (%s)', runnerJob.uuid, runnerJob.type, lTags(runnerJob.uuid, runnerJob.type));
        const RunnerJobHandler = (0, runners_1.getRunnerJobHandlerClass)(runnerJob);
        yield new RunnerJobHandler().cancel({ runnerJob });
        return res.sendStatus(204);
    });
}
function listRunnerJobs(req, res) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const query = req.query;
        const resultList = yield runner_job_1.RunnerJobModel.listForApi({
            start: query.start,
            count: query.count,
            sort: query.sort,
            search: query.search
        });
        return res.json({
            total: resultList.total,
            data: resultList.data.map(d => d.toFormattedAdminJSON())
        });
    });
}
//# sourceMappingURL=jobs.js.map