"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.registrationsRouter = void 0;
const tslib_1 = require("tslib");
const express_1 = tslib_1.__importDefault(require("express"));
const emailer_1 = require("../../../lib/emailer");
const hooks_1 = require("../../../lib/plugins/hooks");
const user_registration_1 = require("../../../models/user/user-registration");
const core_utils_1 = require("../../../../shared/core-utils");
const audit_logger_1 = require("../../../helpers/audit-logger");
const logger_1 = require("../../../helpers/logger");
const config_1 = require("../../../initializers/config");
const notifier_1 = require("../../../lib/notifier");
const user_1 = require("../../../lib/user");
const middlewares_1 = require("../../../middlewares");
const auditLogger = (0, audit_logger_1.auditLoggerFactory)('users');
const registrationRateLimiter = (0, middlewares_1.buildRateLimiter)({
    windowMs: config_1.CONFIG.RATES_LIMIT.SIGNUP.WINDOW_MS,
    max: config_1.CONFIG.RATES_LIMIT.SIGNUP.MAX,
    skipFailedRequests: true
});
const registrationsRouter = express_1.default.Router();
exports.registrationsRouter = registrationsRouter;
registrationsRouter.post('/registrations/request', registrationRateLimiter, (0, middlewares_1.asyncMiddleware)((0, middlewares_1.ensureUserRegistrationAllowedFactory)('request-registration')), middlewares_1.ensureUserRegistrationAllowedForIP, (0, middlewares_1.asyncMiddleware)(middlewares_1.usersRequestRegistrationValidator), (0, middlewares_1.asyncRetryTransactionMiddleware)(requestRegistration));
registrationsRouter.post('/registrations/:registrationId/accept', middlewares_1.authenticate, (0, middlewares_1.ensureUserHasRight)(28), (0, middlewares_1.asyncMiddleware)(middlewares_1.acceptOrRejectRegistrationValidator), (0, middlewares_1.asyncRetryTransactionMiddleware)(acceptRegistration));
registrationsRouter.post('/registrations/:registrationId/reject', middlewares_1.authenticate, (0, middlewares_1.ensureUserHasRight)(28), (0, middlewares_1.asyncMiddleware)(middlewares_1.acceptOrRejectRegistrationValidator), (0, middlewares_1.asyncRetryTransactionMiddleware)(rejectRegistration));
registrationsRouter.delete('/registrations/:registrationId', middlewares_1.authenticate, (0, middlewares_1.ensureUserHasRight)(28), (0, middlewares_1.asyncMiddleware)(middlewares_1.getRegistrationValidator), (0, middlewares_1.asyncRetryTransactionMiddleware)(deleteRegistration));
registrationsRouter.get('/registrations', middlewares_1.authenticate, (0, middlewares_1.ensureUserHasRight)(28), middlewares_1.paginationValidator, middlewares_1.userRegistrationsSortValidator, middlewares_1.setDefaultSort, middlewares_1.setDefaultPagination, middlewares_1.listRegistrationsValidator, (0, middlewares_1.asyncMiddleware)(listRegistrations));
registrationsRouter.post('/register', registrationRateLimiter, (0, middlewares_1.asyncMiddleware)((0, middlewares_1.ensureUserRegistrationAllowedFactory)('direct-registration')), middlewares_1.ensureUserRegistrationAllowedForIP, (0, middlewares_1.asyncMiddleware)(middlewares_1.usersDirectRegistrationValidator), (0, middlewares_1.asyncRetryTransactionMiddleware)(registerUser));
function requestRegistration(req, res) {
    var _a, _b;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const body = req.body;
        const registration = new user_registration_1.UserRegistrationModel(Object.assign(Object.assign({}, (0, core_utils_1.pick)(body, ['username', 'password', 'email', 'registrationReason'])), { accountDisplayName: body.displayName, channelDisplayName: (_a = body.channel) === null || _a === void 0 ? void 0 : _a.displayName, channelHandle: (_b = body.channel) === null || _b === void 0 ? void 0 : _b.name, state: 1, emailVerified: config_1.CONFIG.SIGNUP.REQUIRES_EMAIL_VERIFICATION ? false : null }));
        yield registration.save();
        if (config_1.CONFIG.SIGNUP.REQUIRES_EMAIL_VERIFICATION) {
            yield (0, user_1.sendVerifyRegistrationEmail)(registration);
        }
        notifier_1.Notifier.Instance.notifyOnNewRegistrationRequest(registration);
        hooks_1.Hooks.runAction('action:api.user.requested-registration', { body, registration, req, res });
        return res.json(registration.toFormattedJSON());
    });
}
function acceptRegistration(req, res) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const registration = res.locals.userRegistration;
        const body = req.body;
        const userToCreate = (0, user_1.buildUser)({
            username: registration.username,
            password: registration.password,
            email: registration.email,
            emailVerified: registration.emailVerified
        });
        userToCreate.skipPasswordEncryption = true;
        const { user } = yield (0, user_1.createUserAccountAndChannelAndPlaylist)({
            userToCreate,
            userDisplayName: registration.accountDisplayName,
            channelNames: registration.channelHandle && registration.channelDisplayName
                ? {
                    name: registration.channelHandle,
                    displayName: registration.channelDisplayName
                }
                : undefined
        });
        registration.userId = user.id;
        registration.state = 3;
        registration.moderationResponse = body.moderationResponse;
        yield registration.save();
        logger_1.logger.info('Registration of %s accepted', registration.username);
        if (body.preventEmailDelivery !== true) {
            emailer_1.Emailer.Instance.addUserRegistrationRequestProcessedJob(registration);
        }
        return res.sendStatus(204);
    });
}
function rejectRegistration(req, res) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const registration = res.locals.userRegistration;
        const body = req.body;
        registration.state = 2;
        registration.moderationResponse = body.moderationResponse;
        yield registration.save();
        if (body.preventEmailDelivery !== true) {
            emailer_1.Emailer.Instance.addUserRegistrationRequestProcessedJob(registration);
        }
        logger_1.logger.info('Registration of %s rejected', registration.username);
        return res.sendStatus(204);
    });
}
function deleteRegistration(req, res) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const registration = res.locals.userRegistration;
        yield registration.destroy();
        logger_1.logger.info('Registration of %s deleted', registration.username);
        return res.sendStatus(204);
    });
}
function listRegistrations(req, res) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const resultList = yield user_registration_1.UserRegistrationModel.listForApi({
            start: req.query.start,
            count: req.query.count,
            sort: req.query.sort,
            search: req.query.search
        });
        return res.json({
            total: resultList.total,
            data: resultList.data.map(d => d.toFormattedJSON())
        });
    });
}
function registerUser(req, res) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const body = req.body;
        const userToCreate = (0, user_1.buildUser)(Object.assign(Object.assign({}, (0, core_utils_1.pick)(body, ['username', 'password', 'email'])), { emailVerified: config_1.CONFIG.SIGNUP.REQUIRES_EMAIL_VERIFICATION ? false : null }));
        const { user, account, videoChannel } = yield (0, user_1.createUserAccountAndChannelAndPlaylist)({
            userToCreate,
            userDisplayName: body.displayName || undefined,
            channelNames: body.channel
        });
        auditLogger.create(body.username, new audit_logger_1.UserAuditView(user.toFormattedJSON()));
        logger_1.logger.info('User %s with its channel and account registered.', body.username);
        if (config_1.CONFIG.SIGNUP.REQUIRES_EMAIL_VERIFICATION) {
            yield (0, user_1.sendVerifyUserEmail)(user);
        }
        notifier_1.Notifier.Instance.notifyOnNewDirectRegistration(user);
        hooks_1.Hooks.runAction('action:api.user.registered', { body, user, account, videoChannel, req, res });
        return res.sendStatus(204);
    });
}
//# sourceMappingURL=registrations.js.map