"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.videoPodcastFeedsRouter = void 0;
const tslib_1 = require("tslib");
const express_1 = tslib_1.__importDefault(require("express"));
const path_1 = require("path");
const typings_1 = require("@peertube/feed/lib/typings");
const actor_image_1 = require("../../lib/actor-image");
const internal_event_emitter_1 = require("../../lib/internal-event-emitter");
const hooks_1 = require("../../lib/plugins/hooks");
const middlewares_1 = require("../../middlewares");
const core_utils_1 = require("../../../shared/core-utils");
const express_utils_1 = require("../../helpers/express-utils");
const constants_1 = require("../../initializers/constants");
const middlewares_2 = require("../../middlewares");
const video_caption_1 = require("../../models/video/video-caption");
const shared_1 = require("./shared");
const videoPodcastFeedsRouter = express_1.default.Router();
exports.videoPodcastFeedsRouter = videoPodcastFeedsRouter;
const { middleware: podcastCacheRouteMiddleware, instance: podcastApiCache } = (0, middlewares_1.cacheRouteFactory)({
    headerBlacklist: ['Content-Type']
});
for (const event of ['video-created', 'video-updated', 'video-deleted']) {
    internal_event_emitter_1.InternalEventEmitter.Instance.on(event, ({ video }) => {
        if (video.remote)
            return;
        podcastApiCache.clearGroupSafe((0, middlewares_1.buildPodcastGroupsCache)({ channelId: video.channelId }));
    });
}
for (const event of ['channel-updated', 'channel-deleted']) {
    internal_event_emitter_1.InternalEventEmitter.Instance.on(event, ({ channel }) => {
        podcastApiCache.clearGroupSafe((0, middlewares_1.buildPodcastGroupsCache)({ channelId: channel.id }));
    });
}
videoPodcastFeedsRouter.get('/feeds/podcast/videos.xml', middlewares_2.setFeedPodcastContentType, middlewares_1.videoFeedsPodcastSetCacheKey, podcastCacheRouteMiddleware(constants_1.ROUTE_CACHE_LIFETIME.FEEDS), (0, middlewares_2.asyncMiddleware)(middlewares_2.videoFeedsPodcastValidator), (0, middlewares_2.asyncMiddleware)(generateVideoPodcastFeed));
function generateVideoPodcastFeed(req, res) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const videoChannel = res.locals.videoChannel;
        const { name, userName, description, imageUrl, accountImageUrl, email, link, accountLink } = yield (0, shared_1.buildFeedMetadata)({ videoChannel });
        const data = yield (0, shared_1.getVideosForFeeds)({
            sort: '-publishedAt',
            nsfw: (0, express_utils_1.buildNSFWFilter)(),
            isLocal: true,
            include: 8,
            videoChannelId: videoChannel === null || videoChannel === void 0 ? void 0 : videoChannel.id
        });
        const customTags = yield hooks_1.Hooks.wrapObject([], 'filter:feed.podcast.channel.create-custom-tags.result', { videoChannel });
        const customXMLNS = yield hooks_1.Hooks.wrapObject([], 'filter:feed.podcast.rss.create-custom-xmlns.result');
        const feed = (0, shared_1.initFeed)({
            name,
            description,
            link,
            isPodcast: true,
            imageUrl,
            locked: email
                ? { isLocked: true, email }
                : undefined,
            person: [{ name: userName, href: accountLink, img: accountImageUrl }],
            resourceType: 'videos',
            queryString: new URL(constants_1.WEBSERVER.URL + req.url).search,
            medium: 'video',
            customXMLNS,
            customTags
        });
        yield addVideosToPodcastFeed(feed, data);
        return res.send(feed.podcast()).end();
    });
}
function generatePodcastItem(options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { video, liveItem, media } = options;
        const customTags = yield hooks_1.Hooks.wrapObject([], 'filter:feed.podcast.video.create-custom-tags.result', { video, liveItem });
        const account = video.VideoChannel.Account;
        const author = {
            name: account.getDisplayName(),
            href: account.getClientUrl()
        };
        const commonAttributes = (0, shared_1.getCommonVideoFeedAttributes)(video);
        const guid = liveItem
            ? `${video.uuid}_${video.publishedAt.toISOString()}`
            : commonAttributes.link;
        let personImage;
        if (account.Actor.hasImage(1)) {
            const avatar = (0, actor_image_1.getBiggestActorImage)(account.Actor.Avatars);
            personImage = constants_1.WEBSERVER.URL + avatar.getStaticPath();
        }
        return Object.assign(Object.assign({ guid }, commonAttributes), { trackers: video.getTrackerUrls(), author: [author], person: [
                Object.assign(Object.assign({}, author), { img: personImage })
            ], media, socialInteract: [
                {
                    uri: video.url,
                    protocol: 'activitypub',
                    accountUrl: account.getClientUrl()
                }
            ], customTags });
    });
}
function addVideosToPodcastFeed(feed, videos) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const captionsGroup = yield video_caption_1.VideoCaptionModel.listCaptionsOfMultipleVideos(videos.map(v => v.id));
        for (const video of videos) {
            if (!video.isLive) {
                yield addVODPodcastItem({ feed, video, captionsGroup });
            }
            else if (video.isLive && video.state !== 5) {
                yield addLivePodcastItem({ feed, video });
            }
        }
    });
}
function addVODPodcastItem(options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { feed, video, captionsGroup } = options;
        const webVideos = video.getFormattedWebVideoFilesJSON(true)
            .map(f => buildVODWebVideoFile(video, f))
            .sort((0, core_utils_1.sortObjectComparator)('bitrate', 'desc'));
        const streamingPlaylistFiles = buildVODStreamingPlaylists(video);
        const media = [...webVideos, ...streamingPlaylistFiles];
        const videoCaptions = buildVODCaptions(video, captionsGroup[video.id]);
        const item = yield generatePodcastItem({ video, liveItem: false, media });
        feed.addPodcastItem(Object.assign(Object.assign({}, item), { subTitle: videoCaptions }));
    });
}
function addLivePodcastItem(options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { feed, video } = options;
        let status;
        switch (video.state) {
            case 4:
                status = typings_1.LiveItemStatus.pending;
                break;
            case 1:
                status = typings_1.LiveItemStatus.live;
                break;
        }
        const item = yield generatePodcastItem({ video, liveItem: true, media: buildLiveStreamingPlaylists(video) });
        feed.addPodcastLiveItem(Object.assign(Object.assign({}, item), { status, start: video.updatedAt.toISOString() }));
    });
}
function buildVODWebVideoFile(video, videoFile) {
    const isAudio = videoFile.resolution.id === 0;
    const type = isAudio
        ? constants_1.MIMETYPES.AUDIO.EXT_MIMETYPE[(0, path_1.extname)(videoFile.fileUrl)]
        : constants_1.MIMETYPES.VIDEO.EXT_MIMETYPE[(0, path_1.extname)(videoFile.fileUrl)];
    const sources = [
        { uri: videoFile.fileUrl },
        { uri: videoFile.torrentUrl, contentType: 'application/x-bittorrent' }
    ];
    if (videoFile.magnetUri) {
        sources.push({ uri: videoFile.magnetUri });
    }
    return {
        type,
        title: videoFile.resolution.label,
        length: videoFile.size,
        bitrate: videoFile.size / video.duration * 8,
        language: video.language,
        sources
    };
}
function buildVODStreamingPlaylists(video) {
    const hls = video.getHLSPlaylist();
    if (!hls)
        return [];
    return [
        {
            type: 'application/x-mpegURL',
            title: 'HLS',
            sources: [
                { uri: hls.getMasterPlaylistUrl(video) }
            ],
            language: video.language
        }
    ];
}
function buildLiveStreamingPlaylists(video) {
    const hls = video.getHLSPlaylist();
    return [
        {
            type: 'application/x-mpegURL',
            title: `HLS live stream`,
            sources: [
                { uri: hls.getMasterPlaylistUrl(video) }
            ],
            language: video.language
        }
    ];
}
function buildVODCaptions(video, videoCaptions) {
    return videoCaptions.map(caption => {
        const type = constants_1.MIMETYPES.VIDEO_CAPTIONS.EXT_MIMETYPE[(0, path_1.extname)(caption.filename)];
        if (!type)
            return null;
        return {
            url: caption.getFileUrl(video),
            language: caption.language,
            type,
            rel: 'captions'
        };
    }).filter(c => c);
}
//# sourceMappingURL=video-podcast-feeds.js.map