"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.processVideoStudioEdition = void 0;
const tslib_1 = require("tslib");
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
const ffmpeg_1 = require("../../../helpers/ffmpeg");
const config_1 = require("../../../initializers/config");
const default_transcoding_profiles_1 = require("../../transcoding/default-transcoding-profiles");
const user_1 = require("../../user");
const video_path_manager_1 = require("../../video-path-manager");
const video_studio_1 = require("../../video-studio");
const user_2 = require("../../../models/user/user");
const video_1 = require("../../../models/video/video");
const core_utils_1 = require("../../../../shared/core-utils");
const extra_utils_1 = require("../../../../shared/extra-utils");
const ffmpeg_2 = require("../../../../shared/ffmpeg");
const logger_1 = require("../../../helpers/logger");
const lTagsBase = (0, logger_1.loggerTagsFactory)('video-studio');
function processVideoStudioEdition(job) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const payload = job.data;
        const lTags = lTagsBase(payload.videoUUID);
        logger_1.logger.info('Process video studio edition of %s in job %s.', payload.videoUUID, job.id, lTags);
        try {
            const video = yield video_1.VideoModel.loadFull(payload.videoUUID);
            if (!video) {
                logger_1.logger.info('Can\'t process job %d, video does not exist.', job.id, lTags);
                yield (0, video_studio_1.safeCleanupStudioTMPFiles)(payload.tasks);
                return undefined;
            }
            yield checkUserQuotaOrThrow(video, payload);
            const inputFile = video.getMaxQualityFile();
            const editionResultPath = yield video_path_manager_1.VideoPathManager.Instance.makeAvailableVideoFile(inputFile, (originalFilePath) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                let tmpInputFilePath;
                let outputPath;
                for (const task of payload.tasks) {
                    const outputFilename = (0, extra_utils_1.buildUUID)() + inputFile.extname;
                    outputPath = (0, path_1.join)(config_1.CONFIG.STORAGE.TMP_DIR, outputFilename);
                    yield processTask({
                        inputPath: tmpInputFilePath !== null && tmpInputFilePath !== void 0 ? tmpInputFilePath : originalFilePath,
                        video,
                        outputPath,
                        task,
                        lTags
                    });
                    if (tmpInputFilePath)
                        yield (0, fs_extra_1.remove)(tmpInputFilePath);
                    tmpInputFilePath = outputPath;
                }
                return outputPath;
            }));
            logger_1.logger.info('Video edition ended for video %s.', video.uuid, lTags);
            yield (0, video_studio_1.onVideoStudioEnded)({ video, editionResultPath, tasks: payload.tasks });
        }
        catch (err) {
            yield (0, video_studio_1.safeCleanupStudioTMPFiles)(payload.tasks);
            throw err;
        }
    });
}
exports.processVideoStudioEdition = processVideoStudioEdition;
const taskProcessors = {
    'add-intro': processAddIntroOutro,
    'add-outro': processAddIntroOutro,
    'cut': processCut,
    'add-watermark': processAddWatermark
};
function processTask(options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { video, task, lTags } = options;
        logger_1.logger.info('Processing %s task for video %s.', task.name, video.uuid, Object.assign({ task }, lTags));
        const processor = taskProcessors[options.task.name];
        if (!process)
            throw new Error('Unknown task ' + task.name);
        return processor(options);
    });
}
function processAddIntroOutro(options) {
    const { task, lTags } = options;
    logger_1.logger.debug('Will add intro/outro to the video.', Object.assign({ options }, lTags));
    return buildFFmpegEdition().addIntroOutro(Object.assign(Object.assign({}, (0, core_utils_1.pick)(options, ['inputPath', 'outputPath'])), { introOutroPath: task.options.file, type: task.name === 'add-intro'
            ? 'intro'
            : 'outro' }));
}
function processCut(options) {
    const { task, lTags } = options;
    logger_1.logger.debug('Will cut the video.', Object.assign({ options }, lTags));
    return buildFFmpegEdition().cutVideo(Object.assign(Object.assign({}, (0, core_utils_1.pick)(options, ['inputPath', 'outputPath'])), { start: task.options.start, end: task.options.end }));
}
function processAddWatermark(options) {
    const { task, lTags } = options;
    logger_1.logger.debug('Will add watermark to the video.', Object.assign({ options }, lTags));
    return buildFFmpegEdition().addWatermark(Object.assign(Object.assign({}, (0, core_utils_1.pick)(options, ['inputPath', 'outputPath'])), { watermarkPath: task.options.file, videoFilters: {
            watermarkSizeRatio: task.options.watermarkSizeRatio,
            horitonzalMarginRatio: task.options.horitonzalMarginRatio,
            verticalMarginRatio: task.options.verticalMarginRatio
        } }));
}
function checkUserQuotaOrThrow(video, payload) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const user = yield user_2.UserModel.loadByVideoId(video.id);
        const filePathFinder = (i) => payload.tasks[i].options.file;
        const additionalBytes = yield (0, video_studio_1.approximateIntroOutroAdditionalSize)(video, payload.tasks, filePathFinder);
        if ((yield (0, user_1.isAbleToUploadVideo)(user.id, additionalBytes)) === false) {
            throw new Error('Quota exceeded for this user to edit the video');
        }
    });
}
function buildFFmpegEdition() {
    return new ffmpeg_2.FFmpegEdition((0, ffmpeg_1.getFFmpegCommandWrapperOptions)('vod', default_transcoding_profiles_1.VideoTranscodingProfilesManager.Instance.getAvailableEncoders()));
}
//# sourceMappingURL=video-studio-edition.js.map