"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.processVideoTranscoding = void 0;
const tslib_1 = require("tslib");
const ended_transcoding_1 = require("../../transcoding/ended-transcoding");
const hls_transcoding_1 = require("../../transcoding/hls-transcoding");
const web_transcoding_1 = require("../../transcoding/web-transcoding");
const video_file_1 = require("../../video-file");
const video_path_manager_1 = require("../../video-path-manager");
const video_state_1 = require("../../video-state");
const user_1 = require("../../../models/user/user");
const video_job_info_1 = require("../../../models/video/video-job-info");
const logger_1 = require("../../../helpers/logger");
const video_1 = require("../../../models/video/video");
const handlers = {
    'new-resolution-to-hls': handleHLSJob,
    'new-resolution-to-webtorrent': handleNewWebTorrentResolutionJob,
    'merge-audio-to-webtorrent': handleWebTorrentMergeAudioJob,
    'optimize-to-webtorrent': handleWebTorrentOptimizeJob
};
const lTags = (0, logger_1.loggerTagsFactory)('transcoding');
function processVideoTranscoding(job) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const payload = job.data;
        logger_1.logger.info('Processing transcoding job %s.', job.id, lTags(payload.videoUUID));
        const video = yield video_1.VideoModel.loadFull(payload.videoUUID);
        if (!video) {
            logger_1.logger.info('Do not process job %d, video does not exist.', job.id, lTags(payload.videoUUID));
            return undefined;
        }
        const user = yield user_1.UserModel.loadByChannelActorId(video.VideoChannel.actorId);
        const handler = handlers[payload.type];
        if (!handler) {
            yield (0, video_state_1.moveToFailedTranscodingState)(video);
            yield video_job_info_1.VideoJobInfoModel.decrease(video.uuid, 'pendingTranscode');
            throw new Error('Cannot find transcoding handler for ' + payload.type);
        }
        try {
            yield handler(job, payload, video, user);
        }
        catch (error) {
            yield (0, video_state_1.moveToFailedTranscodingState)(video);
            yield video_job_info_1.VideoJobInfoModel.decrease(video.uuid, 'pendingTranscode');
            throw error;
        }
        return video;
    });
}
exports.processVideoTranscoding = processVideoTranscoding;
function handleWebTorrentMergeAudioJob(job, payload, video, user) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        logger_1.logger.info('Handling merge audio transcoding job for %s.', video.uuid, lTags(video.uuid), { payload });
        yield (0, web_transcoding_1.mergeAudioVideofile)({ video, resolution: payload.resolution, fps: payload.fps, job });
        logger_1.logger.info('Merge audio transcoding job for %s ended.', video.uuid, lTags(video.uuid), { payload });
        yield (0, ended_transcoding_1.onTranscodingEnded)({ isNewVideo: payload.isNewVideo, moveVideoToNextState: !payload.hasChildren, video });
    });
}
function handleWebTorrentOptimizeJob(job, payload, video, user) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        logger_1.logger.info('Handling optimize transcoding job for %s.', video.uuid, lTags(video.uuid), { payload });
        yield (0, web_transcoding_1.optimizeOriginalVideofile)({ video, inputVideoFile: video.getMaxQualityFile(), quickTranscode: payload.quickTranscode, job });
        logger_1.logger.info('Optimize transcoding job for %s ended.', video.uuid, lTags(video.uuid), { payload });
        yield (0, ended_transcoding_1.onTranscodingEnded)({ isNewVideo: payload.isNewVideo, moveVideoToNextState: !payload.hasChildren, video });
    });
}
function handleNewWebTorrentResolutionJob(job, payload, video) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        logger_1.logger.info('Handling WebTorrent transcoding job for %s.', video.uuid, lTags(video.uuid), { payload });
        yield (0, web_transcoding_1.transcodeNewWebTorrentResolution)({ video, resolution: payload.resolution, fps: payload.fps, job });
        logger_1.logger.info('WebTorrent transcoding job for %s ended.', video.uuid, lTags(video.uuid), { payload });
        yield (0, ended_transcoding_1.onTranscodingEnded)({ isNewVideo: payload.isNewVideo, moveVideoToNextState: true, video });
    });
}
function handleHLSJob(job, payload, videoArg) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        logger_1.logger.info('Handling HLS transcoding job for %s.', videoArg.uuid, lTags(videoArg.uuid), { payload });
        const inputFileMutexReleaser = yield video_path_manager_1.VideoPathManager.Instance.lockFiles(videoArg.uuid);
        let video;
        try {
            video = yield video_1.VideoModel.loadFull(videoArg.uuid);
            const videoFileInput = payload.copyCodecs
                ? video.getWebTorrentFile(payload.resolution)
                : video.getMaxQualityFile();
            const videoOrStreamingPlaylist = videoFileInput.getVideoOrStreamingPlaylist();
            yield video_path_manager_1.VideoPathManager.Instance.makeAvailableVideoFile(videoFileInput.withVideoOrPlaylist(videoOrStreamingPlaylist), videoInputPath => {
                return (0, hls_transcoding_1.generateHlsPlaylistResolution)({
                    video,
                    videoInputPath,
                    inputFileMutexReleaser,
                    resolution: payload.resolution,
                    fps: payload.fps,
                    copyCodecs: payload.copyCodecs,
                    job
                });
            });
        }
        finally {
            inputFileMutexReleaser();
        }
        logger_1.logger.info('HLS transcoding job for %s ended.', video.uuid, lTags(video.uuid), { payload });
        if (payload.deleteWebTorrentFiles === true) {
            logger_1.logger.info('Removing WebTorrent files of %s now we have a HLS version of it.', video.uuid, lTags(video.uuid));
            yield (0, video_file_1.removeAllWebTorrentFiles)(video);
        }
        yield (0, ended_transcoding_1.onTranscodingEnded)({ isNewVideo: payload.isNewVideo, moveVideoToNextState: true, video });
    });
}
//# sourceMappingURL=video-transcoding.js.map