"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MuxingSession = void 0;
const tslib_1 = require("tslib");
const bluebird_1 = require("bluebird");
const chokidar_1 = require("chokidar");
const events_1 = require("events");
const fs_extra_1 = require("fs-extra");
const p_queue_1 = tslib_1.__importDefault(require("p-queue"));
const path_1 = require("path");
const ffmpeg_1 = require("../../../helpers/ffmpeg");
const logger_1 = require("../../../helpers/logger");
const config_1 = require("../../../initializers/config");
const constants_1 = require("../../../initializers/constants");
const object_storage_1 = require("../../object-storage");
const video_file_1 = require("../../../models/video/video-file");
const video_streaming_playlist_1 = require("../../../models/video/video-streaming-playlist");
const paths_1 = require("../../paths");
const user_1 = require("../../user");
const live_quota_store_1 = require("../live-quota-store");
const live_segment_sha_store_1 = require("../live-segment-sha-store");
const live_utils_1 = require("../live-utils");
const transcoding_wrapper_1 = require("./transcoding-wrapper");
const memoizee = require("memoizee");
class MuxingSession extends events_1.EventEmitter {
    constructor(options) {
        super();
        this.objectStorageSendQueues = new Map();
        this.segmentsToProcessPerPlaylist = {};
        this.masterPlaylistCreated = false;
        this.liveReady = false;
        this.aborted = false;
        this.isAbleToUploadVideoWithCache = memoizee((userId) => {
            return (0, user_1.isAbleToUploadVideo)(userId, 1000);
        }, { maxAge: constants_1.MEMOIZE_TTL.LIVE_ABLE_TO_UPLOAD });
        this.hasClientSocketInBadHealthWithCache = memoizee((sessionId) => {
            return this.hasClientSocketInBadHealth(sessionId);
        }, { maxAge: constants_1.MEMOIZE_TTL.LIVE_CHECK_SOCKET_HEALTH });
        this.context = options.context;
        this.user = options.user;
        this.sessionId = options.sessionId;
        this.videoLive = options.videoLive;
        this.inputLocalUrl = options.inputLocalUrl;
        this.inputPublicUrl = options.inputPublicUrl;
        this.fps = options.fps;
        this.bitrate = options.bitrate;
        this.ratio = options.ratio;
        this.hasAudio = options.hasAudio;
        this.allResolutions = options.allResolutions;
        this.videoUUID = this.videoLive.Video.uuid;
        this.saveReplay = this.videoLive.saveReplay;
        this.outDirectory = (0, paths_1.getLiveDirectory)(this.videoLive.Video);
        this.replayDirectory = (0, path_1.join)((0, paths_1.getLiveReplayBaseDirectory)(this.videoLive.Video), new Date().toISOString());
        this.lTags = (0, logger_1.loggerTagsFactory)('live', this.sessionId, this.videoUUID);
    }
    runMuxing() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.streamingPlaylist = yield this.createLivePlaylist();
            this.createLiveShaStore();
            this.createFiles();
            yield this.prepareDirectories();
            this.transcodingWrapper = this.buildTranscodingWrapper();
            this.transcodingWrapper.on('end', () => this.onTranscodedEnded());
            this.transcodingWrapper.on('error', () => this.onTranscodingError());
            yield this.transcodingWrapper.run();
            this.filesWatcher = (0, chokidar_1.watch)(this.outDirectory, { depth: 0 });
            this.watchMasterFile();
            this.watchTSFiles();
        });
    }
    abort() {
        if (!this.transcodingWrapper)
            return;
        this.aborted = true;
        this.transcodingWrapper.abort();
    }
    destroy() {
        this.removeAllListeners();
        this.isAbleToUploadVideoWithCache.clear();
        this.hasClientSocketInBadHealthWithCache.clear();
    }
    watchMasterFile() {
        this.filesWatcher.on('add', (path) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (path !== (0, path_1.join)(this.outDirectory, this.streamingPlaylist.playlistFilename))
                return;
            if (this.masterPlaylistCreated === true)
                return;
            try {
                if (this.streamingPlaylist.storage === 1) {
                    const masterContent = yield (0, fs_extra_1.readFile)(path, 'utf-8');
                    logger_1.logger.debug('Uploading live master playlist on object storage for %s', this.videoUUID, Object.assign({ masterContent }, this.lTags()));
                    const url = yield (0, object_storage_1.storeHLSFileFromContent)(this.streamingPlaylist, this.streamingPlaylist.playlistFilename, masterContent);
                    this.streamingPlaylist.playlistUrl = url;
                }
                this.streamingPlaylist.assignP2PMediaLoaderInfoHashes(this.videoLive.Video, this.allResolutions);
                yield this.streamingPlaylist.save();
            }
            catch (err) {
                logger_1.logger.error('Cannot update streaming playlist.', Object.assign({ err }, this.lTags()));
            }
            this.masterPlaylistCreated = true;
            logger_1.logger.info('Master playlist file for %s has been created', this.videoUUID, this.lTags());
        }));
    }
    watchTSFiles() {
        const startStreamDateTime = new Date().getTime();
        const addHandler = (segmentPath) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (segmentPath.endsWith('.ts') !== true)
                return;
            logger_1.logger.debug('Live add handler of TS file %s.', segmentPath, this.lTags());
            const playlistId = this.getPlaylistIdFromTS(segmentPath);
            const segmentsToProcess = this.segmentsToProcessPerPlaylist[playlistId] || [];
            this.processSegments(segmentsToProcess);
            this.segmentsToProcessPerPlaylist[playlistId] = [segmentPath];
            if (this.hasClientSocketInBadHealthWithCache(this.sessionId)) {
                this.emit('bad-socket-health', { videoUUID: this.videoUUID });
                return;
            }
            if (this.isDurationConstraintValid(startStreamDateTime) !== true) {
                this.emit('duration-exceeded', { videoUUID: this.videoUUID });
                return;
            }
            if ((yield this.isQuotaExceeded(segmentPath)) === true) {
                this.emit('quota-exceeded', { videoUUID: this.videoUUID });
            }
        });
        const deleteHandler = (segmentPath) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (segmentPath.endsWith('.ts') !== true)
                return;
            logger_1.logger.debug('Live delete handler of TS file %s.', segmentPath, this.lTags());
            try {
                yield this.liveSegmentShaStore.removeSegmentSha(segmentPath);
            }
            catch (err) {
                logger_1.logger.warn('Cannot remove segment sha %s from sha store', segmentPath, Object.assign({ err }, this.lTags()));
            }
            if (this.streamingPlaylist.storage === 1) {
                try {
                    yield (0, object_storage_1.removeHLSFileObjectStorageByPath)(this.streamingPlaylist, segmentPath);
                }
                catch (err) {
                    logger_1.logger.error('Cannot remove segment %s from object storage', segmentPath, Object.assign({ err }, this.lTags()));
                }
            }
        });
        this.filesWatcher.on('add', p => addHandler(p));
        this.filesWatcher.on('unlink', p => deleteHandler(p));
    }
    isQuotaExceeded(segmentPath) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this.saveReplay !== true)
                return false;
            if (this.aborted)
                return false;
            try {
                const segmentStat = yield (0, fs_extra_1.stat)(segmentPath);
                live_quota_store_1.LiveQuotaStore.Instance.addQuotaTo(this.user.id, this.sessionId, segmentStat.size);
                const canUpload = yield this.isAbleToUploadVideoWithCache(this.user.id);
                return canUpload !== true;
            }
            catch (err) {
                logger_1.logger.error('Cannot stat %s or check quota of %d.', segmentPath, this.user.id, Object.assign({ err }, this.lTags()));
            }
        });
    }
    createFiles() {
        for (let i = 0; i < this.allResolutions.length; i++) {
            const resolution = this.allResolutions[i];
            const file = new video_file_1.VideoFileModel({
                resolution,
                size: -1,
                extname: '.ts',
                infoHash: null,
                fps: this.fps,
                storage: this.streamingPlaylist.storage,
                videoStreamingPlaylistId: this.streamingPlaylist.id
            });
            video_file_1.VideoFileModel.customUpsert(file, 'streaming-playlist', null)
                .catch(err => logger_1.logger.error('Cannot create file for live streaming.', Object.assign({ err }, this.lTags())));
        }
    }
    prepareDirectories() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield (0, fs_extra_1.ensureDir)(this.outDirectory);
            if (this.videoLive.saveReplay === true) {
                yield (0, fs_extra_1.ensureDir)(this.replayDirectory);
            }
        });
    }
    isDurationConstraintValid(streamingStartTime) {
        const maxDuration = config_1.CONFIG.LIVE.MAX_DURATION;
        if (maxDuration < 0)
            return true;
        const now = new Date().getTime();
        const max = streamingStartTime + maxDuration;
        return now <= max;
    }
    processSegments(segmentPaths) {
        (0, bluebird_1.mapSeries)(segmentPaths, previousSegment => this.processSegment(previousSegment))
            .catch(err => {
            if (this.aborted)
                return;
            logger_1.logger.error('Cannot process segments', Object.assign({ err }, this.lTags()));
        });
    }
    processSegment(segmentPath) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.liveSegmentShaStore.addSegmentSha(segmentPath);
            if (this.saveReplay) {
                yield this.addSegmentToReplay(segmentPath);
            }
            if (this.streamingPlaylist.storage === 1) {
                try {
                    yield (0, object_storage_1.storeHLSFileFromPath)(this.streamingPlaylist, segmentPath);
                    yield this.processM3U8ToObjectStorage(segmentPath);
                }
                catch (err) {
                    logger_1.logger.error('Cannot store TS segment %s in object storage', segmentPath, Object.assign({ err }, this.lTags()));
                }
            }
            if (this.masterPlaylistCreated && !this.liveReady) {
                this.liveReady = true;
                this.emit('live-ready', { videoUUID: this.videoUUID });
            }
        });
    }
    processM3U8ToObjectStorage(segmentPath) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const m3u8Path = (0, path_1.join)(this.outDirectory, this.getPlaylistNameFromTS(segmentPath));
            logger_1.logger.debug('Process M3U8 file %s.', m3u8Path, this.lTags());
            const segmentName = (0, path_1.basename)(segmentPath);
            const playlistContent = yield (0, fs_extra_1.readFile)(m3u8Path, 'utf-8');
            const filteredPlaylistContent = playlistContent.substring(0, playlistContent.lastIndexOf(segmentName) + segmentName.length) + '\n';
            try {
                if (!this.objectStorageSendQueues.has(m3u8Path)) {
                    this.objectStorageSendQueues.set(m3u8Path, new p_queue_1.default({ concurrency: 1 }));
                }
                const queue = this.objectStorageSendQueues.get(m3u8Path);
                yield queue.add(() => (0, object_storage_1.storeHLSFileFromContent)(this.streamingPlaylist, m3u8Path, filteredPlaylistContent));
            }
            catch (err) {
                logger_1.logger.error('Cannot store in object storage m3u8 file %s', m3u8Path, Object.assign({ err }, this.lTags()));
            }
        });
    }
    onTranscodingError() {
        this.emit('transcoding-error', ({ videoUUID: this.videoUUID }));
    }
    onTranscodedEnded() {
        this.emit('transcoding-end', ({ videoUUID: this.videoUUID }));
        logger_1.logger.info('RTMP transmuxing for video %s ended. Scheduling cleanup', this.inputLocalUrl, this.lTags());
        setTimeout(() => {
            var _a;
            const promise = ((_a = this.filesWatcher) === null || _a === void 0 ? void 0 : _a.close()) || Promise.resolve();
            promise
                .then(() => {
                for (const key of Object.keys(this.segmentsToProcessPerPlaylist)) {
                    this.processSegments(this.segmentsToProcessPerPlaylist[key]);
                }
            })
                .catch(err => {
                logger_1.logger.error('Cannot close watchers of %s or process remaining hash segments.', this.outDirectory, Object.assign({ err }, this.lTags()));
            });
            this.emit('after-cleanup', { videoUUID: this.videoUUID });
        }, 1000);
    }
    hasClientSocketInBadHealth(sessionId) {
        const rtmpSession = this.context.sessions.get(sessionId);
        if (!rtmpSession) {
            logger_1.logger.warn('Cannot get session %s to check players socket health.', sessionId, this.lTags());
            return;
        }
        for (const playerSessionId of rtmpSession.players) {
            const playerSession = this.context.sessions.get(playerSessionId);
            if (!playerSession) {
                logger_1.logger.error('Cannot get player session %s to check socket health.', playerSession, this.lTags());
                continue;
            }
            if (playerSession.socket.writableLength > constants_1.VIDEO_LIVE.MAX_SOCKET_WAITING_DATA) {
                return true;
            }
        }
        return false;
    }
    addSegmentToReplay(segmentPath) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const segmentName = (0, path_1.basename)(segmentPath);
            const dest = (0, path_1.join)(this.replayDirectory, (0, live_utils_1.buildConcatenatedName)(segmentName));
            try {
                const data = yield (0, fs_extra_1.readFile)(segmentPath);
                yield (0, fs_extra_1.appendFile)(dest, data);
            }
            catch (err) {
                logger_1.logger.error('Cannot copy segment %s to replay directory.', segmentPath, Object.assign({ err }, this.lTags()));
            }
        });
    }
    createLivePlaylist() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const playlist = yield video_streaming_playlist_1.VideoStreamingPlaylistModel.loadOrGenerate(this.videoLive.Video);
            playlist.playlistFilename = (0, paths_1.generateHLSMasterPlaylistFilename)(true);
            playlist.segmentsSha256Filename = (0, paths_1.generateHlsSha256SegmentsFilename)(true);
            playlist.p2pMediaLoaderPeerVersion = constants_1.P2P_MEDIA_LOADER_PEER_VERSION;
            playlist.type = 1;
            playlist.storage = config_1.CONFIG.OBJECT_STORAGE.ENABLED
                ? 1
                : 0;
            return playlist.save();
        });
    }
    createLiveShaStore() {
        this.liveSegmentShaStore = new live_segment_sha_store_1.LiveSegmentShaStore({
            videoUUID: this.videoLive.Video.uuid,
            sha256Path: (0, path_1.join)(this.outDirectory, this.streamingPlaylist.segmentsSha256Filename),
            streamingPlaylist: this.streamingPlaylist,
            sendToObjectStorage: config_1.CONFIG.OBJECT_STORAGE.ENABLED
        });
    }
    buildTranscodingWrapper() {
        const options = {
            streamingPlaylist: this.streamingPlaylist,
            videoLive: this.videoLive,
            lTags: this.lTags,
            sessionId: this.sessionId,
            inputLocalUrl: this.inputLocalUrl,
            inputPublicUrl: this.inputPublicUrl,
            toTranscode: this.allResolutions.map(resolution => ({
                resolution,
                fps: (0, ffmpeg_1.computeOutputFPS)({ inputFPS: this.fps, resolution })
            })),
            fps: this.fps,
            bitrate: this.bitrate,
            ratio: this.ratio,
            hasAudio: this.hasAudio,
            segmentListSize: constants_1.VIDEO_LIVE.SEGMENTS_LIST_SIZE,
            segmentDuration: (0, live_utils_1.getLiveSegmentTime)(this.videoLive.latencyMode),
            outDirectory: this.outDirectory
        };
        return config_1.CONFIG.LIVE.TRANSCODING.ENABLED && config_1.CONFIG.LIVE.TRANSCODING.REMOTE_RUNNERS.ENABLED
            ? new transcoding_wrapper_1.RemoteTranscodingWrapper(options)
            : new transcoding_wrapper_1.FFmpegTranscodingWrapper(options);
    }
    getPlaylistIdFromTS(segmentPath) {
        const playlistIdMatcher = /^([\d+])-/;
        return (0, path_1.basename)(segmentPath).match(playlistIdMatcher)[1];
    }
    getPlaylistNameFromTS(segmentPath) {
        return `${this.getPlaylistIdFromTS(segmentPath)}.m3u8`;
    }
}
exports.MuxingSession = MuxingSession;
//# sourceMappingURL=muxing-session.js.map