"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.proxifyHLS = exports.proxifyWebTorrentFile = void 0;
const tslib_1 = require("tslib");
const stream_1 = require("stream");
const m3u8_playlist_1 = require("../../controllers/shared/m3u8-playlist");
const logger_1 = require("../../helpers/logger");
const stream_replacer_1 = require("../../helpers/stream-replacer");
const hls_1 = require("../hls");
const videos_1 = require("./videos");
function proxifyWebTorrentFile(options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { req, res, filename } = options;
        logger_1.logger.debug('Proxifying WebTorrent file %s from object storage.', filename);
        try {
            const { response: s3Response, stream } = yield (0, videos_1.getWebTorrentFileReadStream)({
                filename,
                rangeHeader: req.header('range')
            });
            setS3Headers(res, s3Response);
            return stream.pipe(res);
        }
        catch (err) {
            return handleObjectStorageFailure(res, err);
        }
    });
}
exports.proxifyWebTorrentFile = proxifyWebTorrentFile;
function proxifyHLS(options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { req, res, playlist, video, filename, reinjectVideoFileToken } = options;
        logger_1.logger.debug('Proxifying HLS file %s from object storage.', filename);
        try {
            const { response: s3Response, stream } = yield (0, videos_1.getHLSFileReadStream)({
                playlist: playlist.withVideo(video),
                filename,
                rangeHeader: req.header('range')
            });
            setS3Headers(res, s3Response);
            const streamReplacer = reinjectVideoFileToken
                ? new stream_replacer_1.StreamReplacer(line => (0, hls_1.injectQueryToPlaylistUrls)(line, (0, m3u8_playlist_1.buildReinjectVideoFileTokenQuery)(req, filename.endsWith('master.m3u8'))))
                : new stream_1.PassThrough();
            return (0, stream_1.pipeline)(stream, streamReplacer, res, err => {
                if (!err)
                    return;
                handleObjectStorageFailure(res, err);
            });
        }
        catch (err) {
            return handleObjectStorageFailure(res, err);
        }
    });
}
exports.proxifyHLS = proxifyHLS;
function handleObjectStorageFailure(res, err) {
    if (err.name === 'NoSuchKey') {
        logger_1.logger.debug('Could not find key in object storage to proxify private HLS video file.', { err });
        return res.sendStatus(404);
    }
    return res.fail({
        status: 500,
        message: err.message,
        type: err.name
    });
}
function setS3Headers(res, s3Response) {
    if (s3Response.$metadata.httpStatusCode === 206) {
        res.setHeader('Content-Range', s3Response.ContentRange);
        res.status(206);
    }
}
//# sourceMappingURL=proxy.js.map