"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PeerTubeSocket = void 0;
const tslib_1 = require("tslib");
const socket_io_1 = require("socket.io");
const misc_1 = require("../helpers/custom-validators/misc");
const logger_1 = require("../helpers/logger");
const middlewares_1 = require("../middlewares");
const debounce_1 = require("../helpers/debounce");
class PeerTubeSocket {
    constructor() {
        this.userNotificationSockets = {};
        this.runnerSockets = new Set();
    }
    init(server) {
        const io = new socket_io_1.Server(server);
        io.of('/user-notifications')
            .use(middlewares_1.authenticateSocket)
            .on('connection', socket => {
            const userId = socket.handshake.auth.user.id;
            logger_1.logger.debug('User %d connected to the notification system.', userId);
            if (!this.userNotificationSockets[userId])
                this.userNotificationSockets[userId] = [];
            this.userNotificationSockets[userId].push(socket);
            socket.on('disconnect', () => {
                logger_1.logger.debug('User %d disconnected from SocketIO notifications.', userId);
                this.userNotificationSockets[userId] = this.userNotificationSockets[userId].filter(s => s !== socket);
            });
        });
        this.liveVideosNamespace = io.of('/live-videos')
            .on('connection', socket => {
            socket.on('subscribe', ({ videoId }) => {
                if (!(0, misc_1.isIdValid)(videoId))
                    return;
                socket.join(videoId);
            });
            socket.on('unsubscribe', ({ videoId }) => {
                if (!(0, misc_1.isIdValid)(videoId))
                    return;
                socket.leave(videoId);
            });
        });
        io.of('/runners')
            .use(middlewares_1.authenticateRunnerSocket)
            .on('connection', socket => {
            const runner = socket.handshake.auth.runner;
            logger_1.logger.debug(`New runner "${runner.name}" connected to the notification system.`);
            this.runnerSockets.add(socket);
            socket.on('disconnect', () => {
                logger_1.logger.debug(`Runner "${runner.name}" disconnected from the notification system.`);
                this.runnerSockets.delete(socket);
            });
        });
    }
    sendNotification(userId, notification) {
        const sockets = this.userNotificationSockets[userId];
        if (!sockets)
            return;
        logger_1.logger.debug('Sending user notification to user %d.', userId);
        const notificationMessage = notification.toFormattedJSON();
        for (const socket of sockets) {
            socket.emit('new-notification', notificationMessage);
        }
    }
    sendVideoLiveNewState(video) {
        const data = { state: video.state };
        const type = 'state-change';
        logger_1.logger.debug('Sending video live new state notification of %s.', video.url, { state: video.state });
        this.liveVideosNamespace
            .in(video.id)
            .emit(type, data);
    }
    sendVideoViewsUpdate(video, numViewers) {
        const data = { viewers: numViewers, views: numViewers };
        const type = 'views-change';
        logger_1.logger.debug('Sending video live views update notification of %s.', video.url, { viewers: numViewers });
        this.liveVideosNamespace
            .in(video.id)
            .emit(type, data);
    }
    sendAvailableJobsPingToRunners() {
        logger_1.logger.debug(`Sending available-jobs notification to ${this.runnerSockets.size} runner sockets`);
        for (const runners of this.runnerSockets) {
            runners.emit('available-jobs');
        }
    }
    static get Instance() {
        return this.instance || (this.instance = new this());
    }
}
tslib_1.__decorate([
    (0, debounce_1.Debounce)({ timeoutMS: 1000 }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", []),
    tslib_1.__metadata("design:returntype", void 0)
], PeerTubeSocket.prototype, "sendAvailableJobsPingToRunners", null);
exports.PeerTubeSocket = PeerTubeSocket;
//# sourceMappingURL=peertube-socket.js.map