"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractJobHandler = void 0;
const tslib_1 = require("tslib");
const lodash_1 = require("lodash");
const database_utils_1 = require("../../../helpers/database-utils");
const logger_1 = require("../../../helpers/logger");
const constants_1 = require("../../../initializers/constants");
const database_1 = require("../../../initializers/database");
const peertube_socket_1 = require("../../peertube-socket");
const runner_job_1 = require("../../../models/runner/runner-job");
const shared_1 = require("../../../models/shared");
const core_utils_1 = require("../../../../shared/core-utils");
const models_1 = require("../../../../shared/models");
class AbstractJobHandler {
    constructor() {
        this.lTags = (0, logger_1.loggerTagsFactory)('runner');
    }
    createRunnerJob(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { priority, dependsOnRunnerJob } = options;
            logger_1.logger.debug('Creating runner job', Object.assign({ options }, this.lTags(options.type)));
            const runnerJob = new runner_job_1.RunnerJobModel(Object.assign(Object.assign({}, (0, core_utils_1.pick)(options, ['type', 'payload', 'privatePayload'])), { uuid: options.jobUUID, state: dependsOnRunnerJob
                    ? models_1.RunnerJobState.WAITING_FOR_PARENT_JOB
                    : models_1.RunnerJobState.PENDING, dependsOnRunnerJobId: dependsOnRunnerJob === null || dependsOnRunnerJob === void 0 ? void 0 : dependsOnRunnerJob.id, priority }));
            const job = yield database_1.sequelizeTypescript.transaction((transaction) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                return runnerJob.save({ transaction });
            }));
            if (runnerJob.state === models_1.RunnerJobState.PENDING) {
                peertube_socket_1.PeerTubeSocket.Instance.sendAvailableJobsPingToRunners();
            }
            return job;
        });
    }
    update(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { runnerJob, progress } = options;
            yield this.specificUpdate(options);
            if (progress)
                runnerJob.progress = progress;
            if (!runnerJob.changed()) {
                try {
                    yield AbstractJobHandler.setJobAsUpdatedThrottled({ sequelize: database_1.sequelizeTypescript, table: 'runnerJob', id: runnerJob.id });
                }
                catch (err) {
                    logger_1.logger.warn('Cannot set remote job as updated', Object.assign({ err }, this.lTags(runnerJob.id, runnerJob.type)));
                }
                return;
            }
            yield (0, database_utils_1.saveInTransactionWithRetries)(runnerJob);
        });
    }
    complete(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { runnerJob } = options;
            runnerJob.state = models_1.RunnerJobState.COMPLETING;
            yield (0, database_utils_1.saveInTransactionWithRetries)(runnerJob);
            try {
                yield this.specificComplete(options);
                runnerJob.state = models_1.RunnerJobState.COMPLETED;
            }
            catch (err) {
                logger_1.logger.error('Cannot complete runner job', Object.assign({ err }, this.lTags(runnerJob.id, runnerJob.type)));
                runnerJob.state = models_1.RunnerJobState.ERRORED;
                runnerJob.error = err.message;
            }
            runnerJob.progress = null;
            runnerJob.finishedAt = new Date();
            yield (0, database_utils_1.saveInTransactionWithRetries)(runnerJob);
            const [affectedCount] = yield runner_job_1.RunnerJobModel.updateDependantJobsOf(runnerJob);
            if (affectedCount !== 0)
                peertube_socket_1.PeerTubeSocket.Instance.sendAvailableJobsPingToRunners();
        });
    }
    cancel(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { runnerJob, fromParent } = options;
            yield this.specificCancel(options);
            const cancelState = fromParent
                ? models_1.RunnerJobState.PARENT_CANCELLED
                : models_1.RunnerJobState.CANCELLED;
            runnerJob.setToErrorOrCancel(cancelState);
            yield (0, database_utils_1.saveInTransactionWithRetries)(runnerJob);
            const children = yield runner_job_1.RunnerJobModel.listChildrenOf(runnerJob);
            for (const child of children) {
                logger_1.logger.info(`Cancelling child job ${child.uuid} of ${runnerJob.uuid} because of parent cancel`, this.lTags(child.uuid));
                yield this.cancel({ runnerJob: child, fromParent: true });
            }
        });
    }
    abort(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { runnerJob } = options;
            if (this.isAbortSupported() !== true) {
                return this.error({ runnerJob, message: 'Job has been aborted but it is not supported by this job type' });
            }
            yield this.specificAbort(options);
            runnerJob.resetToPending();
            yield (0, database_utils_1.saveInTransactionWithRetries)(runnerJob);
        });
    }
    setAbortState(runnerJob) {
        runnerJob.resetToPending();
    }
    error(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { runnerJob, message, fromParent } = options;
            const errorState = fromParent
                ? models_1.RunnerJobState.PARENT_ERRORED
                : models_1.RunnerJobState.ERRORED;
            const nextState = errorState === models_1.RunnerJobState.ERRORED && this.isAbortSupported() && runnerJob.failures < constants_1.RUNNER_JOBS.MAX_FAILURES
                ? models_1.RunnerJobState.PENDING
                : errorState;
            yield this.specificError(Object.assign(Object.assign({}, options), { nextState }));
            if (nextState === errorState) {
                runnerJob.setToErrorOrCancel(nextState);
                runnerJob.error = message;
            }
            else {
                runnerJob.resetToPending();
            }
            yield (0, database_utils_1.saveInTransactionWithRetries)(runnerJob);
            if (runnerJob.state === errorState) {
                const children = yield runner_job_1.RunnerJobModel.listChildrenOf(runnerJob);
                for (const child of children) {
                    logger_1.logger.info(`Erroring child job ${child.uuid} of ${runnerJob.uuid} because of parent error`, this.lTags(child.uuid));
                    yield this.error({ runnerJob: child, message: 'Parent error', fromParent: true });
                }
            }
        });
    }
}
AbstractJobHandler.setJobAsUpdatedThrottled = (0, lodash_1.throttle)(shared_1.setAsUpdated, 2000);
exports.AbstractJobHandler = AbstractJobHandler;
//# sourceMappingURL=abstract-job-handler.js.map