"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LiveRTMPHLSTranscodingJobHandler = void 0;
const tslib_1 = require("tslib");
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
const logger_1 = require("../../../helpers/logger");
const constants_1 = require("../../../initializers/constants");
const live_1 = require("../../live");
const extra_utils_1 = require("../../../../shared/extra-utils");
const abstract_job_handler_1 = require("./abstract-job-handler");
class LiveRTMPHLSTranscodingJobHandler extends abstract_job_handler_1.AbstractJobHandler {
    create(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { video, rtmpUrl, toTranscode, playlist, segmentDuration, segmentListSize, outputDirectory, sessionId } = options;
            const jobUUID = (0, extra_utils_1.buildUUID)();
            const payload = {
                input: {
                    rtmpUrl
                },
                output: {
                    toTranscode,
                    segmentListSize,
                    segmentDuration
                }
            };
            const privatePayload = {
                videoUUID: video.uuid,
                masterPlaylistName: playlist.playlistFilename,
                sessionId,
                outputDirectory
            };
            const job = yield this.createRunnerJob({
                type: 'live-rtmp-hls-transcoding',
                jobUUID,
                payload,
                privatePayload,
                priority: constants_1.JOB_PRIORITY.TRANSCODING
            });
            return job;
        });
    }
    specificUpdate(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { runnerJob, updatePayload } = options;
            const privatePayload = runnerJob.privatePayload;
            const outputDirectory = privatePayload.outputDirectory;
            const videoUUID = privatePayload.videoUUID;
            if (updatePayload.type === 'add-chunk') {
                yield (0, fs_extra_1.move)(updatePayload.videoChunkFile, (0, path_1.join)(outputDirectory, updatePayload.videoChunkFilename), { overwrite: true });
            }
            else if (updatePayload.type === 'remove-chunk') {
                yield (0, fs_extra_1.remove)((0, path_1.join)(outputDirectory, updatePayload.videoChunkFilename));
            }
            if (updatePayload.resolutionPlaylistFile && updatePayload.resolutionPlaylistFilename) {
                yield (0, fs_extra_1.move)(updatePayload.resolutionPlaylistFile, (0, path_1.join)(outputDirectory, updatePayload.resolutionPlaylistFilename), { overwrite: true });
            }
            if (updatePayload.masterPlaylistFile) {
                yield (0, fs_extra_1.move)(updatePayload.masterPlaylistFile, (0, path_1.join)(outputDirectory, privatePayload.masterPlaylistName), { overwrite: true });
            }
            logger_1.logger.debug('Runner live RTMP to HLS job %s for %s updated.', runnerJob.uuid, videoUUID, Object.assign({ updatePayload }, this.lTags(videoUUID, runnerJob.uuid)));
        });
    }
    specificComplete(options) {
        return this.stopLive({
            runnerJob: options.runnerJob,
            type: 'ended'
        });
    }
    isAbortSupported() {
        return false;
    }
    specificAbort() {
        throw new Error('Not implemented');
    }
    specificError(options) {
        return this.stopLive({
            runnerJob: options.runnerJob,
            type: 'errored'
        });
    }
    specificCancel(options) {
        return this.stopLive({
            runnerJob: options.runnerJob,
            type: 'cancelled'
        });
    }
    stopLive(options) {
        const { runnerJob, type } = options;
        const privatePayload = runnerJob.privatePayload;
        const videoUUID = privatePayload.videoUUID;
        const errorType = {
            ended: null,
            errored: 6,
            cancelled: 7
        };
        live_1.LiveManager.Instance.stopSessionOf(privatePayload.videoUUID, errorType[type]);
        logger_1.logger.info('Runner live RTMP to HLS job %s for video %s %s.', runnerJob.uuid, videoUUID, type, this.lTags(runnerJob.uuid, videoUUID));
    }
}
exports.LiveRTMPHLSTranscodingJobHandler = LiveRTMPHLSTranscodingJobHandler;
//# sourceMappingURL=live-rtmp-hls-transcoding-job-handler.js.map