"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.onHLSVideoFileTranscoding = exports.generateHlsPlaylistResolution = exports.generateHlsPlaylistResolutionFromTS = void 0;
const tslib_1 = require("tslib");
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
const database_utils_1 = require("../../helpers/database-utils");
const webtorrent_1 = require("../../helpers/webtorrent");
const database_1 = require("../../initializers/database");
const core_utils_1 = require("../../../shared/core-utils");
const ffmpeg_1 = require("../../../shared/ffmpeg");
const config_1 = require("../../initializers/config");
const video_file_1 = require("../../models/video/video-file");
const video_streaming_playlist_1 = require("../../models/video/video-streaming-playlist");
const hls_1 = require("../hls");
const paths_1 = require("../paths");
const video_file_2 = require("../video-file");
const video_path_manager_1 = require("../video-path-manager");
const shared_1 = require("./shared");
function generateHlsPlaylistResolutionFromTS(options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return generateHlsPlaylistCommon(Object.assign({ type: 'hls-from-ts', inputPath: options.concatenatedTsFilePath }, (0, core_utils_1.pick)(options, ['video', 'resolution', 'fps', 'inputFileMutexReleaser', 'isAAC'])));
    });
}
exports.generateHlsPlaylistResolutionFromTS = generateHlsPlaylistResolutionFromTS;
function generateHlsPlaylistResolution(options) {
    return generateHlsPlaylistCommon(Object.assign({ type: 'hls', inputPath: options.videoInputPath }, (0, core_utils_1.pick)(options, ['video', 'resolution', 'fps', 'copyCodecs', 'inputFileMutexReleaser', 'job'])));
}
exports.generateHlsPlaylistResolution = generateHlsPlaylistResolution;
function onHLSVideoFileTranscoding(options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { video, videoFile, videoOutputPath, m3u8OutputPath } = options;
        const playlist = yield (0, database_utils_1.retryTransactionWrapper)(() => {
            return database_1.sequelizeTypescript.transaction((transaction) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                return video_streaming_playlist_1.VideoStreamingPlaylistModel.loadOrGenerate(video, transaction);
            }));
        });
        videoFile.videoStreamingPlaylistId = playlist.id;
        const mutexReleaser = yield video_path_manager_1.VideoPathManager.Instance.lockFiles(video.uuid);
        try {
            yield video.reload();
            const videoFilePath = video_path_manager_1.VideoPathManager.Instance.getFSVideoFileOutputPath(playlist, videoFile);
            yield (0, fs_extra_1.ensureDir)(video_path_manager_1.VideoPathManager.Instance.getFSHLSOutputPath(video));
            const resolutionPlaylistPath = video_path_manager_1.VideoPathManager.Instance.getFSHLSOutputPath(video, (0, path_1.basename)(m3u8OutputPath));
            yield (0, fs_extra_1.move)(m3u8OutputPath, resolutionPlaylistPath, { overwrite: true });
            yield (0, fs_extra_1.move)(videoOutputPath, videoFilePath, { overwrite: true });
            if (!video.duration) {
                video.duration = yield (0, ffmpeg_1.getVideoStreamDuration)(videoFilePath);
                yield video.save();
            }
            const stats = yield (0, fs_extra_1.stat)(videoFilePath);
            videoFile.size = stats.size;
            videoFile.fps = yield (0, ffmpeg_1.getVideoStreamFPS)(videoFilePath);
            videoFile.metadata = yield (0, video_file_2.buildFileMetadata)(videoFilePath);
            yield (0, webtorrent_1.createTorrentAndSetInfoHash)(playlist, videoFile);
            const oldFile = yield video_file_1.VideoFileModel.loadHLSFile({
                playlistId: playlist.id,
                fps: videoFile.fps,
                resolution: videoFile.resolution
            });
            if (oldFile) {
                yield video.removeStreamingPlaylistVideoFile(playlist, oldFile);
                yield oldFile.destroy();
            }
            const savedVideoFile = yield video_file_1.VideoFileModel.customUpsert(videoFile, 'streaming-playlist', undefined);
            yield (0, hls_1.updatePlaylistAfterFileChange)(video, playlist);
            return { resolutionPlaylistPath, videoFile: savedVideoFile };
        }
        finally {
            mutexReleaser();
        }
    });
}
exports.onHLSVideoFileTranscoding = onHLSVideoFileTranscoding;
function generateHlsPlaylistCommon(options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { type, video, inputPath, resolution, fps, copyCodecs, isAAC, job, inputFileMutexReleaser } = options;
        const transcodeDirectory = config_1.CONFIG.STORAGE.TMP_DIR;
        const videoTranscodedBasePath = (0, path_1.join)(transcodeDirectory, type);
        yield (0, fs_extra_1.ensureDir)(videoTranscodedBasePath);
        const videoFilename = (0, paths_1.generateHLSVideoFilename)(resolution);
        const videoOutputPath = (0, path_1.join)(videoTranscodedBasePath, videoFilename);
        const resolutionPlaylistFilename = (0, paths_1.getHlsResolutionPlaylistFilename)(videoFilename);
        const m3u8OutputPath = (0, path_1.join)(videoTranscodedBasePath, resolutionPlaylistFilename);
        const transcodeOptions = {
            type,
            inputPath,
            outputPath: m3u8OutputPath,
            resolution,
            fps,
            copyCodecs,
            isAAC,
            inputFileMutexReleaser,
            hlsPlaylist: {
                videoFilename
            }
        };
        yield (0, shared_1.buildFFmpegVOD)(job).transcode(transcodeOptions);
        const newVideoFile = new video_file_1.VideoFileModel({
            resolution,
            extname: (0, path_1.extname)(videoFilename),
            size: 0,
            filename: videoFilename,
            fps: -1
        });
        yield onHLSVideoFileTranscoding({ video, videoFile: newVideoFile, videoOutputPath, m3u8OutputPath });
    });
}
//# sourceMappingURL=hls-transcoding.js.map