"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TranscodingJobQueueBuilder = void 0;
const tslib_1 = require("tslib");
const bluebird_1 = tslib_1.__importDefault(require("bluebird"));
const ffmpeg_1 = require("../../../../helpers/ffmpeg");
const logger_1 = require("../../../../helpers/logger");
const config_1 = require("../../../../initializers/config");
const constants_1 = require("../../../../initializers/constants");
const job_queue_1 = require("../../../job-queue");
const hooks_1 = require("../../../plugins/hooks");
const video_path_manager_1 = require("../../../video-path-manager");
const video_job_info_1 = require("../../../../models/video/video-job-info");
const ffmpeg_2 = require("../../../../../shared/ffmpeg");
const transcoding_priority_1 = require("../../transcoding-priority");
const transcoding_quick_transcode_1 = require("../../transcoding-quick-transcode");
const transcoding_resolutions_1 = require("../../transcoding-resolutions");
const abstract_job_builder_1 = require("./abstract-job-builder");
class TranscodingJobQueueBuilder extends abstract_job_builder_1.AbstractJobBuilder {
    createOptimizeOrMergeAudioJobs(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { video, videoFile, isNewVideo, user, videoFileAlreadyLocked } = options;
            let mergeOrOptimizePayload;
            let nextTranscodingSequentialJobPayloads = [];
            const mutexReleaser = videoFileAlreadyLocked
                ? () => { }
                : yield video_path_manager_1.VideoPathManager.Instance.lockFiles(video.uuid);
            try {
                yield video.reload();
                yield videoFile.reload();
                yield video_path_manager_1.VideoPathManager.Instance.makeAvailableVideoFile(videoFile.withVideoOrPlaylist(video), (videoFilePath) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const probe = yield (0, ffmpeg_2.ffprobePromise)(videoFilePath);
                    const { resolution } = yield (0, ffmpeg_2.getVideoStreamDimensionsInfo)(videoFilePath, probe);
                    const hasAudio = yield (0, ffmpeg_2.hasAudioStream)(videoFilePath, probe);
                    const quickTranscode = yield (0, transcoding_quick_transcode_1.canDoQuickTranscode)(videoFilePath, probe);
                    const inputFPS = videoFile.isAudio()
                        ? constants_1.VIDEO_TRANSCODING_FPS.AUDIO_MERGE
                        : yield (0, ffmpeg_2.getVideoStreamFPS)(videoFilePath, probe);
                    const maxResolution = (yield (0, ffmpeg_2.isAudioFile)(videoFilePath, probe))
                        ? constants_1.DEFAULT_AUDIO_RESOLUTION
                        : (0, transcoding_resolutions_1.buildOriginalFileResolution)(resolution);
                    if (config_1.CONFIG.TRANSCODING.HLS.ENABLED === true) {
                        nextTranscodingSequentialJobPayloads.push([
                            this.buildHLSJobPayload({
                                deleteWebTorrentFiles: config_1.CONFIG.TRANSCODING.WEBTORRENT.ENABLED === false,
                                copyCodecs: !quickTranscode,
                                resolution: maxResolution,
                                fps: (0, ffmpeg_1.computeOutputFPS)({ inputFPS, resolution: maxResolution }),
                                videoUUID: video.uuid,
                                isNewVideo
                            })
                        ]);
                    }
                    const lowerResolutionJobPayloads = yield this.buildLowerResolutionJobPayloads({
                        video,
                        inputVideoResolution: maxResolution,
                        inputVideoFPS: inputFPS,
                        hasAudio,
                        isNewVideo
                    });
                    nextTranscodingSequentialJobPayloads = [...nextTranscodingSequentialJobPayloads, ...lowerResolutionJobPayloads];
                    const hasChildren = nextTranscodingSequentialJobPayloads.length !== 0;
                    mergeOrOptimizePayload = videoFile.isAudio()
                        ? this.buildMergeAudioPayload({ videoUUID: video.uuid, isNewVideo, hasChildren })
                        : this.buildOptimizePayload({ videoUUID: video.uuid, isNewVideo, quickTranscode, hasChildren });
                }));
            }
            finally {
                mutexReleaser();
            }
            const nextTranscodingSequentialJobs = yield bluebird_1.default.mapSeries(nextTranscodingSequentialJobPayloads, payloads => {
                return bluebird_1.default.mapSeries(payloads, payload => {
                    return this.buildTranscodingJob({ payload, user });
                });
            });
            const transcodingJobBuilderJob = {
                type: 'transcoding-job-builder',
                payload: {
                    videoUUID: video.uuid,
                    sequentialJobs: nextTranscodingSequentialJobs
                }
            };
            const mergeOrOptimizeJob = yield this.buildTranscodingJob({ payload: mergeOrOptimizePayload, user });
            yield job_queue_1.JobQueue.Instance.createSequentialJobFlow(...[mergeOrOptimizeJob, transcodingJobBuilderJob]);
            yield video_job_info_1.VideoJobInfoModel.increaseOrCreate(video.uuid, 'pendingTranscode');
        });
    }
    createTranscodingJobs(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { video, transcodingType, resolutions, isNewVideo } = options;
            const maxResolution = Math.max(...resolutions);
            const childrenResolutions = resolutions.filter(r => r !== maxResolution);
            logger_1.logger.info('Manually creating transcoding jobs for %s.', transcodingType, { childrenResolutions, maxResolution });
            const { fps: inputFPS } = yield video.probeMaxQualityFile();
            const children = childrenResolutions.map(resolution => {
                const fps = (0, ffmpeg_1.computeOutputFPS)({ inputFPS, resolution });
                if (transcodingType === 'hls') {
                    return this.buildHLSJobPayload({ videoUUID: video.uuid, resolution, fps, isNewVideo });
                }
                if (transcodingType === 'webtorrent') {
                    return this.buildWebTorrentJobPayload({ videoUUID: video.uuid, resolution, fps, isNewVideo });
                }
                throw new Error('Unknown transcoding type');
            });
            const fps = (0, ffmpeg_1.computeOutputFPS)({ inputFPS, resolution: maxResolution });
            const parent = transcodingType === 'hls'
                ? this.buildHLSJobPayload({ videoUUID: video.uuid, resolution: maxResolution, fps, isNewVideo })
                : this.buildWebTorrentJobPayload({ videoUUID: video.uuid, resolution: maxResolution, fps, isNewVideo });
            yield this.createTranscodingJobsWithChildren({ videoUUID: video.uuid, parent, children, user: null });
        });
    }
    createTranscodingJobsWithChildren(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { videoUUID, parent, children, user } = options;
            const parentJob = yield this.buildTranscodingJob({ payload: parent, user });
            const childrenJobs = yield bluebird_1.default.mapSeries(children, c => this.buildTranscodingJob({ payload: c, user }));
            yield job_queue_1.JobQueue.Instance.createJobWithChildren(parentJob, childrenJobs);
            yield video_job_info_1.VideoJobInfoModel.increaseOrCreate(videoUUID, 'pendingTranscode', 1 + children.length);
        });
    }
    buildTranscodingJob(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { user, payload } = options;
            return {
                type: 'video-transcoding',
                priority: yield (0, transcoding_priority_1.getTranscodingJobPriority)({ user, type: 'vod', fallback: undefined }),
                payload
            };
        });
    }
    buildLowerResolutionJobPayloads(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { video, inputVideoResolution, inputVideoFPS, isNewVideo, hasAudio } = options;
            const resolutionsEnabled = yield hooks_1.Hooks.wrapObject((0, transcoding_resolutions_1.computeResolutionsToTranscode)({ input: inputVideoResolution, type: 'vod', includeInput: false, strictLower: true, hasAudio }), 'filter:transcoding.auto.resolutions-to-transcode.result', options);
            const sequentialPayloads = [];
            for (const resolution of resolutionsEnabled) {
                const fps = (0, ffmpeg_1.computeOutputFPS)({ inputFPS: inputVideoFPS, resolution });
                if (config_1.CONFIG.TRANSCODING.WEBTORRENT.ENABLED) {
                    const payloads = [
                        this.buildWebTorrentJobPayload({
                            videoUUID: video.uuid,
                            resolution,
                            fps,
                            isNewVideo
                        })
                    ];
                    if (config_1.CONFIG.TRANSCODING.HLS.ENABLED) {
                        payloads.push(this.buildHLSJobPayload({
                            videoUUID: video.uuid,
                            resolution,
                            fps,
                            isNewVideo,
                            copyCodecs: true
                        }));
                    }
                    sequentialPayloads.push(payloads);
                }
                else if (config_1.CONFIG.TRANSCODING.HLS.ENABLED) {
                    sequentialPayloads.push([
                        this.buildHLSJobPayload({
                            videoUUID: video.uuid,
                            resolution,
                            fps,
                            copyCodecs: false,
                            isNewVideo
                        })
                    ]);
                }
            }
            return sequentialPayloads;
        });
    }
    buildHLSJobPayload(options) {
        const { videoUUID, resolution, fps, isNewVideo, deleteWebTorrentFiles = false, copyCodecs = false } = options;
        return {
            type: 'new-resolution-to-hls',
            videoUUID,
            resolution,
            fps,
            copyCodecs,
            isNewVideo,
            deleteWebTorrentFiles
        };
    }
    buildWebTorrentJobPayload(options) {
        const { videoUUID, resolution, fps, isNewVideo } = options;
        return {
            type: 'new-resolution-to-webtorrent',
            videoUUID,
            isNewVideo,
            resolution,
            fps
        };
    }
    buildMergeAudioPayload(options) {
        const { videoUUID, isNewVideo, hasChildren } = options;
        return {
            type: 'merge-audio-to-webtorrent',
            resolution: constants_1.DEFAULT_AUDIO_RESOLUTION,
            fps: constants_1.VIDEO_TRANSCODING_FPS.AUDIO_MERGE,
            videoUUID,
            isNewVideo,
            hasChildren
        };
    }
    buildOptimizePayload(options) {
        const { videoUUID, quickTranscode, isNewVideo, hasChildren } = options;
        return {
            type: 'optimize-to-webtorrent',
            videoUUID,
            isNewVideo,
            hasChildren,
            quickTranscode
        };
    }
}
exports.TranscodingJobQueueBuilder = TranscodingJobQueueBuilder;
//# sourceMappingURL=transcoding-job-queue-builder.js.map