"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TranscodingRunnerJobBuilder = void 0;
const tslib_1 = require("tslib");
const ffmpeg_1 = require("../../../../helpers/ffmpeg");
const logger_1 = require("../../../../helpers/logger");
const config_1 = require("../../../../initializers/config");
const constants_1 = require("../../../../initializers/constants");
const hooks_1 = require("../../../plugins/hooks");
const runners_1 = require("../../../runners");
const video_path_manager_1 = require("../../../video-path-manager");
const ffmpeg_2 = require("../../../../../shared/ffmpeg");
const transcoding_priority_1 = require("../../transcoding-priority");
const transcoding_resolutions_1 = require("../../transcoding-resolutions");
const abstract_job_builder_1 = require("./abstract-job-builder");
const lTags = (0, logger_1.loggerTagsFactory)('transcoding');
class TranscodingRunnerJobBuilder extends abstract_job_builder_1.AbstractJobBuilder {
    createOptimizeOrMergeAudioJobs(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { video, videoFile, isNewVideo, user, videoFileAlreadyLocked } = options;
            const mutexReleaser = videoFileAlreadyLocked
                ? () => { }
                : yield video_path_manager_1.VideoPathManager.Instance.lockFiles(video.uuid);
            try {
                yield video.reload();
                yield videoFile.reload();
                yield video_path_manager_1.VideoPathManager.Instance.makeAvailableVideoFile(videoFile.withVideoOrPlaylist(video), (videoFilePath) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const probe = yield (0, ffmpeg_2.ffprobePromise)(videoFilePath);
                    const { resolution } = yield (0, ffmpeg_2.getVideoStreamDimensionsInfo)(videoFilePath, probe);
                    const hasAudio = yield (0, ffmpeg_2.hasAudioStream)(videoFilePath, probe);
                    const inputFPS = videoFile.isAudio()
                        ? constants_1.VIDEO_TRANSCODING_FPS.AUDIO_MERGE
                        : yield (0, ffmpeg_2.getVideoStreamFPS)(videoFilePath, probe);
                    const maxResolution = (yield (0, ffmpeg_2.isAudioFile)(videoFilePath, probe))
                        ? constants_1.DEFAULT_AUDIO_RESOLUTION
                        : resolution;
                    const fps = (0, ffmpeg_1.computeOutputFPS)({ inputFPS, resolution: maxResolution });
                    const priority = yield (0, transcoding_priority_1.getTranscodingJobPriority)({ user, type: 'vod', fallback: 0 });
                    const mainRunnerJob = videoFile.isAudio()
                        ? yield new runners_1.VODAudioMergeTranscodingJobHandler().create({ video, resolution: maxResolution, fps, isNewVideo, priority })
                        : yield new runners_1.VODWebVideoTranscodingJobHandler().create({ video, resolution: maxResolution, fps, isNewVideo, priority });
                    if (config_1.CONFIG.TRANSCODING.HLS.ENABLED === true) {
                        yield new runners_1.VODHLSTranscodingJobHandler().create({
                            video,
                            deleteWebVideoFiles: config_1.CONFIG.TRANSCODING.WEBTORRENT.ENABLED === false,
                            resolution: maxResolution,
                            fps,
                            isNewVideo,
                            dependsOnRunnerJob: mainRunnerJob,
                            priority: yield (0, transcoding_priority_1.getTranscodingJobPriority)({ user, type: 'vod', fallback: 0 })
                        });
                    }
                    yield this.buildLowerResolutionJobPayloads({
                        video,
                        inputVideoResolution: maxResolution,
                        inputVideoFPS: inputFPS,
                        hasAudio,
                        isNewVideo,
                        mainRunnerJob,
                        user
                    });
                }));
            }
            finally {
                mutexReleaser();
            }
        });
    }
    createTranscodingJobs(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { video, transcodingType, resolutions, isNewVideo, user } = options;
            const maxResolution = Math.max(...resolutions);
            const { fps: inputFPS } = yield video.probeMaxQualityFile();
            const maxFPS = (0, ffmpeg_1.computeOutputFPS)({ inputFPS, resolution: maxResolution });
            const priority = yield (0, transcoding_priority_1.getTranscodingJobPriority)({ user, type: 'vod', fallback: 0 });
            const childrenResolutions = resolutions.filter(r => r !== maxResolution);
            logger_1.logger.info('Manually creating transcoding jobs for %s.', transcodingType, { childrenResolutions, maxResolution });
            const mainJob = transcodingType === 'hls'
                ? yield new runners_1.VODHLSTranscodingJobHandler().create({ video, resolution: maxResolution, fps: maxFPS, isNewVideo, deleteWebVideoFiles: false, priority })
                : yield new runners_1.VODWebVideoTranscodingJobHandler().create({ video, resolution: maxResolution, fps: maxFPS, isNewVideo, priority });
            for (const resolution of childrenResolutions) {
                const dependsOnRunnerJob = mainJob;
                const fps = (0, ffmpeg_1.computeOutputFPS)({ inputFPS, resolution });
                if (transcodingType === 'hls') {
                    yield new runners_1.VODHLSTranscodingJobHandler().create({
                        video,
                        resolution,
                        fps,
                        isNewVideo,
                        deleteWebVideoFiles: false,
                        dependsOnRunnerJob,
                        priority: yield (0, transcoding_priority_1.getTranscodingJobPriority)({ user, type: 'vod', fallback: 0 })
                    });
                    continue;
                }
                if (transcodingType === 'webtorrent') {
                    yield new runners_1.VODWebVideoTranscodingJobHandler().create({
                        video,
                        resolution,
                        fps,
                        isNewVideo,
                        dependsOnRunnerJob,
                        priority: yield (0, transcoding_priority_1.getTranscodingJobPriority)({ user, type: 'vod', fallback: 0 })
                    });
                    continue;
                }
                throw new Error('Unknown transcoding type');
            }
        });
    }
    buildLowerResolutionJobPayloads(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { video, inputVideoResolution, inputVideoFPS, isNewVideo, hasAudio, mainRunnerJob, user } = options;
            const resolutionsEnabled = yield hooks_1.Hooks.wrapObject((0, transcoding_resolutions_1.computeResolutionsToTranscode)({ input: inputVideoResolution, type: 'vod', includeInput: false, strictLower: true, hasAudio }), 'filter:transcoding.auto.resolutions-to-transcode.result', options);
            logger_1.logger.debug('Lower resolutions build for %s.', video.uuid, Object.assign({ resolutionsEnabled }, lTags(video.uuid)));
            for (const resolution of resolutionsEnabled) {
                const fps = (0, ffmpeg_1.computeOutputFPS)({ inputFPS: inputVideoFPS, resolution });
                if (config_1.CONFIG.TRANSCODING.WEBTORRENT.ENABLED) {
                    yield new runners_1.VODWebVideoTranscodingJobHandler().create({
                        video,
                        resolution,
                        fps,
                        isNewVideo,
                        dependsOnRunnerJob: mainRunnerJob,
                        priority: yield (0, transcoding_priority_1.getTranscodingJobPriority)({ user, type: 'vod', fallback: 0 })
                    });
                }
                if (config_1.CONFIG.TRANSCODING.HLS.ENABLED) {
                    yield new runners_1.VODHLSTranscodingJobHandler().create({
                        video,
                        resolution,
                        fps,
                        isNewVideo,
                        deleteWebVideoFiles: false,
                        dependsOnRunnerJob: mainRunnerJob,
                        priority: yield (0, transcoding_priority_1.getTranscodingJobPriority)({ user, type: 'vod', fallback: 0 })
                    });
                }
            }
        });
    }
}
exports.TranscodingRunnerJobBuilder = TranscodingRunnerJobBuilder;
//# sourceMappingURL=transcoding-runner-job-builder.js.map