"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildFileMetadata = exports.removeWebTorrentFile = exports.removeAllWebTorrentFiles = exports.removeHLSFile = exports.removeHLSPlaylist = exports.buildNewFile = void 0;
const tslib_1 = require("tslib");
const logger_1 = require("../helpers/logger");
const video_file_1 = require("../models/video/video-file");
const core_utils_1 = require("../../shared/core-utils");
const extra_utils_1 = require("../../shared/extra-utils");
const ffmpeg_1 = require("../../shared/ffmpeg");
const models_1 = require("../../shared/models");
const shared_1 = require("./object-storage/shared");
const paths_1 = require("./paths");
const video_path_manager_1 = require("./video-path-manager");
function buildNewFile(options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { path, mode } = options;
        const probe = yield (0, ffmpeg_1.ffprobePromise)(path);
        const size = yield (0, extra_utils_1.getFileSize)(path);
        const videoFile = new video_file_1.VideoFileModel({
            extname: (0, core_utils_1.getLowercaseExtension)(path),
            size,
            metadata: yield buildFileMetadata(path, probe)
        });
        if (yield (0, ffmpeg_1.isAudioFile)(path, probe)) {
            videoFile.resolution = 0;
        }
        else {
            videoFile.fps = yield (0, ffmpeg_1.getVideoStreamFPS)(path, probe);
            videoFile.resolution = (yield (0, ffmpeg_1.getVideoStreamDimensionsInfo)(path, probe)).resolution;
        }
        videoFile.filename = mode === 'web-video'
            ? (0, paths_1.generateWebTorrentVideoFilename)(videoFile.resolution, videoFile.extname)
            : (0, paths_1.generateHLSVideoFilename)(videoFile.resolution);
        return videoFile;
    });
}
exports.buildNewFile = buildNewFile;
function removeHLSPlaylist(video) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const hls = video.getHLSPlaylist();
        if (!hls)
            return;
        const videoFileMutexReleaser = yield video_path_manager_1.VideoPathManager.Instance.lockFiles(video.uuid);
        try {
            yield video.removeStreamingPlaylistFiles(hls);
            yield hls.destroy();
            video.VideoStreamingPlaylists = video.VideoStreamingPlaylists.filter(p => p.id !== hls.id);
        }
        finally {
            videoFileMutexReleaser();
        }
    });
}
exports.removeHLSPlaylist = removeHLSPlaylist;
function removeHLSFile(video, fileToDeleteId) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        logger_1.logger.info('Deleting HLS file %d of %s.', fileToDeleteId, video.url, (0, shared_1.lTags)(video.uuid));
        const hls = video.getHLSPlaylist();
        const files = hls.VideoFiles;
        if (files.length === 1) {
            yield removeHLSPlaylist(video);
            return undefined;
        }
        const videoFileMutexReleaser = yield video_path_manager_1.VideoPathManager.Instance.lockFiles(video.uuid);
        try {
            const toDelete = files.find(f => f.id === fileToDeleteId);
            yield video.removeStreamingPlaylistVideoFile(video.getHLSPlaylist(), toDelete);
            yield toDelete.destroy();
            hls.VideoFiles = hls.VideoFiles.filter(f => f.id !== toDelete.id);
        }
        finally {
            videoFileMutexReleaser();
        }
        return hls;
    });
}
exports.removeHLSFile = removeHLSFile;
function removeAllWebTorrentFiles(video) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const videoFileMutexReleaser = yield video_path_manager_1.VideoPathManager.Instance.lockFiles(video.uuid);
        try {
            for (const file of video.VideoFiles) {
                yield video.removeWebTorrentFile(file);
                yield file.destroy();
            }
            video.VideoFiles = [];
        }
        finally {
            videoFileMutexReleaser();
        }
        return video;
    });
}
exports.removeAllWebTorrentFiles = removeAllWebTorrentFiles;
function removeWebTorrentFile(video, fileToDeleteId) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const files = video.VideoFiles;
        if (files.length === 1) {
            return removeAllWebTorrentFiles(video);
        }
        const videoFileMutexReleaser = yield video_path_manager_1.VideoPathManager.Instance.lockFiles(video.uuid);
        try {
            const toDelete = files.find(f => f.id === fileToDeleteId);
            yield video.removeWebTorrentFile(toDelete);
            yield toDelete.destroy();
            video.VideoFiles = files.filter(f => f.id !== toDelete.id);
        }
        finally {
            videoFileMutexReleaser();
        }
        return video;
    });
}
exports.removeWebTorrentFile = removeWebTorrentFile;
function buildFileMetadata(path, existingProbe) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const metadata = existingProbe || (yield (0, ffmpeg_1.ffprobePromise)(path));
        return new models_1.VideoFileMetadata(metadata);
    });
}
exports.buildFileMetadata = buildFileMetadata;
//# sourceMappingURL=video-file.js.map