"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.onVideoStudioEnded = exports.createVideoStudioJob = exports.approximateIntroOutroAdditionalSize = exports.safeCleanupStudioTMPFiles = exports.getStudioTaskFilePath = exports.getTaskFileFromReq = exports.buildTaskFileFieldname = void 0;
const tslib_1 = require("tslib");
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
const logger_1 = require("../helpers/logger");
const webtorrent_1 = require("../helpers/webtorrent");
const config_1 = require("../initializers/config");
const user_1 = require("../models/user/user");
const ffmpeg_1 = require("../../shared/ffmpeg");
const videos_1 = require("./activitypub/videos");
const job_queue_1 = require("./job-queue");
const runners_1 = require("./runners");
const create_transcoding_job_1 = require("./transcoding/create-transcoding-job");
const transcoding_priority_1 = require("./transcoding/transcoding-priority");
const video_file_1 = require("./video-file");
const video_path_manager_1 = require("./video-path-manager");
const lTags = (0, logger_1.loggerTagsFactory)('video-studio');
function buildTaskFileFieldname(indice, fieldName = 'file') {
    return `tasks[${indice}][options][${fieldName}]`;
}
exports.buildTaskFileFieldname = buildTaskFileFieldname;
function getTaskFileFromReq(files, indice, fieldName = 'file') {
    return files.find(f => f.fieldname === buildTaskFileFieldname(indice, fieldName));
}
exports.getTaskFileFromReq = getTaskFileFromReq;
function getStudioTaskFilePath(filename) {
    return (0, path_1.join)(config_1.CONFIG.STORAGE.TMP_PERSISTENT_DIR, filename);
}
exports.getStudioTaskFilePath = getStudioTaskFilePath;
function safeCleanupStudioTMPFiles(tasks) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        logger_1.logger.info('Removing studio task files', Object.assign({ tasks }, lTags()));
        for (const task of tasks) {
            try {
                if (task.name === 'add-intro' || task.name === 'add-outro') {
                    yield (0, fs_extra_1.remove)(task.options.file);
                }
                else if (task.name === 'add-watermark') {
                    yield (0, fs_extra_1.remove)(task.options.file);
                }
            }
            catch (err) {
                logger_1.logger.error('Cannot remove studio file', { err });
            }
        }
    });
}
exports.safeCleanupStudioTMPFiles = safeCleanupStudioTMPFiles;
function approximateIntroOutroAdditionalSize(video, tasks, fileFinder) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        let additionalDuration = 0;
        for (let i = 0; i < tasks.length; i++) {
            const task = tasks[i];
            if (task.name !== 'add-intro' && task.name !== 'add-outro')
                continue;
            const filePath = fileFinder(i);
            additionalDuration += yield (0, ffmpeg_1.getVideoStreamDuration)(filePath);
        }
        return (video.getMaxQualityFile().size / video.duration) * additionalDuration;
    });
}
exports.approximateIntroOutroAdditionalSize = approximateIntroOutroAdditionalSize;
function createVideoStudioJob(options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { video, user, payload } = options;
        const priority = yield (0, transcoding_priority_1.getTranscodingJobPriority)({ user, type: 'studio', fallback: 0 });
        if (config_1.CONFIG.VIDEO_STUDIO.REMOTE_RUNNERS.ENABLED) {
            yield new runners_1.VideoStudioTranscodingJobHandler().create({ video, tasks: payload.tasks, priority });
            return;
        }
        yield job_queue_1.JobQueue.Instance.createJob({ type: 'video-studio-edition', payload, priority });
    });
}
exports.createVideoStudioJob = createVideoStudioJob;
function onVideoStudioEnded(options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { video, tasks, editionResultPath } = options;
        const newFile = yield (0, video_file_1.buildNewFile)({ path: editionResultPath, mode: 'web-video' });
        newFile.videoId = video.id;
        const outputPath = video_path_manager_1.VideoPathManager.Instance.getFSVideoFileOutputPath(video, newFile);
        yield (0, fs_extra_1.move)(editionResultPath, outputPath);
        yield safeCleanupStudioTMPFiles(tasks);
        yield (0, webtorrent_1.createTorrentAndSetInfoHashFromPath)(video, newFile, outputPath);
        yield removeAllFiles(video, newFile);
        yield newFile.save();
        video.duration = yield (0, ffmpeg_1.getVideoStreamDuration)(outputPath);
        yield video.save();
        yield (0, videos_1.federateVideoIfNeeded)(video, false, undefined);
        const user = yield user_1.UserModel.loadByVideoId(video.id);
        yield (0, create_transcoding_job_1.createOptimizeOrMergeAudioJobs)({ video, videoFile: newFile, isNewVideo: false, user, videoFileAlreadyLocked: false });
    });
}
exports.onVideoStudioEnded = onVideoStudioEnded;
function removeAllFiles(video, webTorrentFileException) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        yield (0, video_file_1.removeHLSPlaylist)(video);
        for (const file of video.VideoFiles) {
            if (file.id === webTorrentFileException.id)
                continue;
            yield (0, video_file_1.removeWebTorrentFile)(video, file.id);
        }
    });
}
//# sourceMappingURL=video-studio.js.map