"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.jobOfRunnerGetValidator = exports.runnerJobGetValidator = exports.cancelRunnerJobValidator = exports.successRunnerJobValidator = exports.errorRunnerJobValidator = exports.updateRunnerJobValidator = exports.abortRunnerJobValidator = exports.acceptRunnerJobValidator = void 0;
const tslib_1 = require("tslib");
const express_validator_1 = require("express-validator");
const misc_1 = require("../../../helpers/custom-validators/misc");
const jobs_1 = require("../../../helpers/custom-validators/runners/jobs");
const runners_1 = require("../../../helpers/custom-validators/runners/runners");
const express_utils_1 = require("../../../helpers/express-utils");
const live_1 = require("../../../lib/live");
const runner_job_1 = require("../../../models/runner/runner-job");
const models_1 = require("../../../../shared/models");
const shared_1 = require("../shared");
const tags = ['runner'];
exports.acceptRunnerJobValidator = [
    (req, res, next) => {
        if (res.locals.runnerJob.state !== models_1.RunnerJobState.PENDING) {
            return res.fail({
                status: 400,
                message: 'This runner job is not in pending state',
                tags
            });
        }
        return next();
    }
];
exports.abortRunnerJobValidator = [
    (0, express_validator_1.body)('reason').custom(jobs_1.isRunnerJobAbortReasonValid),
    (req, res, next) => {
        if ((0, shared_1.areValidationErrors)(req, res, { tags }))
            return;
        return next();
    }
];
exports.updateRunnerJobValidator = [
    (0, express_validator_1.body)('progress').optional().custom(jobs_1.isRunnerJobProgressValid),
    (req, res, next) => {
        if ((0, shared_1.areValidationErrors)(req, res, { tags }))
            return (0, express_utils_1.cleanUpReqFiles)(req);
        const body = req.body;
        const job = res.locals.runnerJob;
        if ((0, jobs_1.isRunnerJobUpdatePayloadValid)(body.payload, job.type, req.files) !== true) {
            (0, express_utils_1.cleanUpReqFiles)(req);
            return res.fail({
                status: 400,
                message: 'Payload is invalid',
                tags
            });
        }
        if (res.locals.runnerJob.type === 'live-rtmp-hls-transcoding') {
            const privatePayload = job.privatePayload;
            if (!live_1.LiveManager.Instance.hasSession(privatePayload.sessionId)) {
                (0, express_utils_1.cleanUpReqFiles)(req);
                return res.fail({
                    status: 400,
                    type: "runner_job_not_in_processing_state",
                    message: 'Session of this live ended',
                    tags
                });
            }
        }
        return next();
    }
];
exports.errorRunnerJobValidator = [
    (0, express_validator_1.body)('message').custom(jobs_1.isRunnerJobErrorMessageValid),
    (req, res, next) => {
        if ((0, shared_1.areValidationErrors)(req, res, { tags }))
            return;
        return next();
    }
];
exports.successRunnerJobValidator = [
    (req, res, next) => {
        const body = req.body;
        if ((0, jobs_1.isRunnerJobSuccessPayloadValid)(body.payload, res.locals.runnerJob.type, req.files) !== true) {
            (0, express_utils_1.cleanUpReqFiles)(req);
            return res.fail({
                status: 400,
                message: 'Payload is invalid',
                tags
            });
        }
        return next();
    }
];
exports.cancelRunnerJobValidator = [
    (req, res, next) => {
        const runnerJob = res.locals.runnerJob;
        const allowedStates = new Set([
            models_1.RunnerJobState.PENDING,
            models_1.RunnerJobState.PROCESSING,
            models_1.RunnerJobState.WAITING_FOR_PARENT_JOB
        ]);
        if (allowedStates.has(runnerJob.state) !== true) {
            return res.fail({
                status: 400,
                message: 'Cannot cancel this job that is not in "pending", "processing" or "waiting for parent job" state',
                tags
            });
        }
        return next();
    }
];
exports.runnerJobGetValidator = [
    (0, express_validator_1.param)('jobUUID').custom(misc_1.isUUIDValid),
    (req, res, next) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        if ((0, shared_1.areValidationErrors)(req, res, { tags }))
            return;
        const runnerJob = yield runner_job_1.RunnerJobModel.loadWithRunner(req.params.jobUUID);
        if (!runnerJob) {
            return res.fail({
                status: 404,
                message: 'Unknown runner job',
                tags
            });
        }
        res.locals.runnerJob = runnerJob;
        return next();
    })
];
exports.jobOfRunnerGetValidator = [
    (0, express_validator_1.param)('jobUUID').custom(misc_1.isUUIDValid),
    (0, express_validator_1.body)('runnerToken').custom(runners_1.isRunnerTokenValid),
    (0, express_validator_1.body)('jobToken').custom(jobs_1.isRunnerJobTokenValid),
    (req, res, next) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        if ((0, shared_1.areValidationErrors)(req, res, { tags }))
            return (0, express_utils_1.cleanUpReqFiles)(req);
        const runnerJob = yield runner_job_1.RunnerJobModel.loadByRunnerAndJobTokensWithRunner({
            uuid: req.params.jobUUID,
            runnerToken: req.body.runnerToken,
            jobToken: req.body.jobToken
        });
        if (!runnerJob) {
            (0, express_utils_1.cleanUpReqFiles)(req);
            return res.fail({
                status: 404,
                message: 'Unknown runner job',
                tags
            });
        }
        if (runnerJob.state !== models_1.RunnerJobState.PROCESSING) {
            (0, express_utils_1.cleanUpReqFiles)(req);
            return res.fail({
                status: 400,
                type: "runner_job_not_in_processing_state",
                message: 'Job is not in "processing" state',
                tags
            });
        }
        res.locals.runnerJob = runnerJob;
        return next();
    })
];
//# sourceMappingURL=jobs.js.map