"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.acceptOrRejectRegistrationValidator = exports.listRegistrationsValidator = exports.getRegistrationValidator = exports.ensureUserRegistrationAllowedForIP = exports.ensureUserRegistrationAllowedFactory = exports.usersRequestRegistrationValidator = exports.usersDirectRegistrationValidator = void 0;
const tslib_1 = require("tslib");
const express_validator_1 = require("express-validator");
const misc_1 = require("../../helpers/custom-validators/misc");
const user_registration_1 = require("../../helpers/custom-validators/user-registration");
const config_1 = require("../../initializers/config");
const hooks_1 = require("../../lib/plugins/hooks");
const users_1 = require("../../helpers/custom-validators/users");
const video_channels_1 = require("../../helpers/custom-validators/video-channels");
const signup_1 = require("../../lib/signup");
const actor_1 = require("../../models/actor/actor");
const shared_1 = require("./shared");
const user_registrations_1 = require("./shared/user-registrations");
const usersDirectRegistrationValidator = usersCommonRegistrationValidatorFactory();
exports.usersDirectRegistrationValidator = usersDirectRegistrationValidator;
const usersRequestRegistrationValidator = [
    ...usersCommonRegistrationValidatorFactory([
        (0, express_validator_1.body)('registrationReason')
            .custom(user_registration_1.isRegistrationReasonValid)
    ]),
    (req, res, next) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        var _a;
        const body = req.body;
        if (config_1.CONFIG.SIGNUP.REQUIRES_APPROVAL !== true) {
            return res.fail({
                status: 400,
                message: 'Signup approval is not enabled on this instance'
            });
        }
        const options = { username: body.username, email: body.email, channelHandle: (_a = body.channel) === null || _a === void 0 ? void 0 : _a.name, res };
        if (!(yield (0, user_registrations_1.checkRegistrationHandlesDoNotAlreadyExist)(options)))
            return;
        return next();
    })
];
exports.usersRequestRegistrationValidator = usersRequestRegistrationValidator;
function ensureUserRegistrationAllowedFactory(signupMode) {
    return (req, res, next) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        const allowedParams = {
            body: req.body,
            ip: req.ip,
            signupMode
        };
        const allowedResult = yield hooks_1.Hooks.wrapPromiseFun(signup_1.isSignupAllowed, allowedParams, signupMode === 'direct-registration'
            ? 'filter:api.user.signup.allowed.result'
            : 'filter:api.user.request-signup.allowed.result');
        if (allowedResult.allowed === false) {
            return res.fail({
                status: 403,
                message: allowedResult.errorMessage || 'User registration is not allowed'
            });
        }
        return next();
    });
}
exports.ensureUserRegistrationAllowedFactory = ensureUserRegistrationAllowedFactory;
const ensureUserRegistrationAllowedForIP = [
    (req, res, next) => {
        const allowed = (0, signup_1.isSignupAllowedForCurrentIP)(req.ip);
        if (allowed === false) {
            return res.fail({
                status: 403,
                message: 'You are not on a network authorized for registration.'
            });
        }
        return next();
    }
];
exports.ensureUserRegistrationAllowedForIP = ensureUserRegistrationAllowedForIP;
const acceptOrRejectRegistrationValidator = [
    (0, express_validator_1.param)('registrationId')
        .custom(misc_1.isIdValid),
    (0, express_validator_1.body)('moderationResponse')
        .custom(user_registration_1.isRegistrationModerationResponseValid),
    (0, express_validator_1.body)('preventEmailDelivery')
        .optional()
        .customSanitizer(misc_1.toBooleanOrNull)
        .custom(misc_1.isBooleanValid).withMessage('Should have preventEmailDelivery boolean'),
    (req, res, next) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        if ((0, shared_1.areValidationErrors)(req, res))
            return;
        if (!(yield (0, user_registrations_1.checkRegistrationIdExist)(req.params.registrationId, res)))
            return;
        if (res.locals.userRegistration.state !== 1) {
            return res.fail({
                status: 409,
                message: 'This registration is already accepted or rejected.'
            });
        }
        return next();
    })
];
exports.acceptOrRejectRegistrationValidator = acceptOrRejectRegistrationValidator;
const getRegistrationValidator = [
    (0, express_validator_1.param)('registrationId')
        .custom(misc_1.isIdValid),
    (req, res, next) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        if ((0, shared_1.areValidationErrors)(req, res))
            return;
        if (!(yield (0, user_registrations_1.checkRegistrationIdExist)(req.params.registrationId, res)))
            return;
        return next();
    })
];
exports.getRegistrationValidator = getRegistrationValidator;
const listRegistrationsValidator = [
    (0, express_validator_1.query)('search')
        .optional()
        .custom(misc_1.exists),
    (req, res, next) => {
        if ((0, shared_1.areValidationErrors)(req, res))
            return;
        return next();
    }
];
exports.listRegistrationsValidator = listRegistrationsValidator;
function usersCommonRegistrationValidatorFactory(additionalValidationChain = []) {
    return [
        (0, express_validator_1.body)('username')
            .custom(users_1.isUserUsernameValid),
        (0, express_validator_1.body)('password')
            .custom(users_1.isUserPasswordValid),
        (0, express_validator_1.body)('email')
            .isEmail(),
        (0, express_validator_1.body)('displayName')
            .optional()
            .custom(users_1.isUserDisplayNameValid),
        (0, express_validator_1.body)('channel.name')
            .optional()
            .custom(video_channels_1.isVideoChannelUsernameValid),
        (0, express_validator_1.body)('channel.displayName')
            .optional()
            .custom(video_channels_1.isVideoChannelDisplayNameValid),
        ...additionalValidationChain,
        (req, res, next) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            if ((0, shared_1.areValidationErrors)(req, res, { omitBodyLog: true }))
                return;
            const body = req.body;
            if (!(yield (0, shared_1.checkUserNameOrEmailDoNotAlreadyExist)(body.username, body.email, res)))
                return;
            if (body.channel) {
                if (!body.channel.name || !body.channel.displayName) {
                    return res.fail({ message: 'Channel is optional but if you specify it, channel.name and channel.displayName are required.' });
                }
                if (body.channel.name === body.username) {
                    return res.fail({ message: 'Channel name cannot be the same as user username.' });
                }
                const existing = yield actor_1.ActorModel.loadLocalByName(body.channel.name);
                if (existing) {
                    return res.fail({
                        status: 409,
                        message: `Channel with name ${body.channel.name} already exists.`
                    });
                }
            }
            return next();
        })
    ];
}
//# sourceMappingURL=user-registrations.js.map