"use strict";
var RunnerJobModel_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.RunnerJobModel = void 0;
const tslib_1 = require("tslib");
const sequelize_1 = require("sequelize");
const sequelize_typescript_1 = require("sequelize-typescript");
const misc_1 = require("../../helpers/custom-validators/misc");
const constants_1 = require("../../initializers/constants");
const models_1 = require("../../../shared/models");
const shared_1 = require("../shared");
const runner_1 = require("./runner");
var ScopeNames;
(function (ScopeNames) {
    ScopeNames["WITH_RUNNER"] = "WITH_RUNNER";
    ScopeNames["WITH_PARENT"] = "WITH_PARENT";
})(ScopeNames || (ScopeNames = {}));
let RunnerJobModel = RunnerJobModel_1 = class RunnerJobModel extends sequelize_typescript_1.Model {
    static loadWithRunner(uuid) {
        const query = {
            where: { uuid }
        };
        return RunnerJobModel_1.scope(ScopeNames.WITH_RUNNER).findOne(query);
    }
    static loadByRunnerAndJobTokensWithRunner(options) {
        const { uuid, runnerToken, jobToken } = options;
        const query = {
            where: {
                uuid,
                processingJobToken: jobToken
            },
            include: {
                model: runner_1.RunnerModel.unscoped(),
                required: true,
                where: {
                    runnerToken
                }
            }
        };
        return RunnerJobModel_1.findOne(query);
    }
    static listAvailableJobs() {
        const query = {
            limit: 10,
            order: (0, shared_1.getSort)('priority'),
            where: {
                state: models_1.RunnerJobState.PENDING
            }
        };
        return RunnerJobModel_1.findAll(query);
    }
    static listStalledJobs(options) {
        const before = new Date(Date.now() - options.staleTimeMS);
        return RunnerJobModel_1.findAll({
            where: {
                type: {
                    [sequelize_1.Op.in]: options.types
                },
                state: models_1.RunnerJobState.PROCESSING,
                updatedAt: {
                    [sequelize_1.Op.lt]: before
                }
            }
        });
    }
    static listChildrenOf(job, transaction) {
        const query = {
            where: {
                dependsOnRunnerJobId: job.id
            },
            transaction
        };
        return RunnerJobModel_1.findAll(query);
    }
    static listForApi(options) {
        const { start, count, sort, search } = options;
        const query = {
            offset: start,
            limit: count,
            order: (0, shared_1.getSort)(sort)
        };
        if (search) {
            if ((0, misc_1.isUUIDValid)(search)) {
                query.where = { uuid: search };
            }
            else {
                query.where = {
                    [sequelize_1.Op.or]: [
                        (0, shared_1.searchAttribute)(search, 'type'),
                        (0, shared_1.searchAttribute)(search, '$Runner.name$')
                    ]
                };
            }
        }
        return Promise.all([
            RunnerJobModel_1.scope([ScopeNames.WITH_RUNNER]).count(query),
            RunnerJobModel_1.scope([ScopeNames.WITH_RUNNER, ScopeNames.WITH_PARENT]).findAll(query)
        ]).then(([total, data]) => ({ total, data }));
    }
    static updateDependantJobsOf(runnerJob) {
        const where = {
            dependsOnRunnerJobId: runnerJob.id
        };
        return RunnerJobModel_1.update({ state: models_1.RunnerJobState.PENDING }, { where });
    }
    static cancelAllJobs(options) {
        const where = {
            type: options.type
        };
        return RunnerJobModel_1.update({ state: models_1.RunnerJobState.CANCELLED }, { where });
    }
    resetToPending() {
        this.state = models_1.RunnerJobState.PENDING;
        this.processingJobToken = null;
        this.progress = null;
        this.startedAt = null;
        this.runnerId = null;
    }
    setToErrorOrCancel(state) {
        this.state = state;
        this.processingJobToken = null;
        this.finishedAt = new Date();
    }
    toFormattedJSON() {
        var _a, _b;
        const runner = this.Runner
            ? {
                id: this.Runner.id,
                name: this.Runner.name,
                description: this.Runner.description
            }
            : null;
        const parent = this.DependsOnRunnerJob
            ? {
                id: this.DependsOnRunnerJob.id,
                uuid: this.DependsOnRunnerJob.uuid,
                type: this.DependsOnRunnerJob.type,
                state: {
                    id: this.DependsOnRunnerJob.state,
                    label: constants_1.RUNNER_JOB_STATES[this.DependsOnRunnerJob.state]
                }
            }
            : undefined;
        return {
            uuid: this.uuid,
            type: this.type,
            state: {
                id: this.state,
                label: constants_1.RUNNER_JOB_STATES[this.state]
            },
            progress: this.progress,
            priority: this.priority,
            failures: this.failures,
            error: this.error,
            payload: this.payload,
            startedAt: (_a = this.startedAt) === null || _a === void 0 ? void 0 : _a.toISOString(),
            finishedAt: (_b = this.finishedAt) === null || _b === void 0 ? void 0 : _b.toISOString(),
            createdAt: this.createdAt.toISOString(),
            updatedAt: this.updatedAt.toISOString(),
            parent,
            runner
        };
    }
    toFormattedAdminJSON() {
        return Object.assign(Object.assign({}, this.toFormattedJSON()), { privatePayload: this.privatePayload });
    }
};
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    (0, sequelize_typescript_1.IsUUID)(4),
    (0, sequelize_typescript_1.Column)(sequelize_typescript_1.DataType.UUID),
    tslib_1.__metadata("design:type", String)
], RunnerJobModel.prototype, "uuid", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", String)
], RunnerJobModel.prototype, "type", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    (0, sequelize_typescript_1.Column)(sequelize_typescript_1.DataType.JSONB),
    tslib_1.__metadata("design:type", Object)
], RunnerJobModel.prototype, "payload", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    (0, sequelize_typescript_1.Column)(sequelize_typescript_1.DataType.JSONB),
    tslib_1.__metadata("design:type", Object)
], RunnerJobModel.prototype, "privatePayload", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", Number)
], RunnerJobModel.prototype, "state", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    (0, sequelize_typescript_1.Default)(0),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", Number)
], RunnerJobModel.prototype, "failures", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(true),
    (0, sequelize_typescript_1.Column)(sequelize_typescript_1.DataType.STRING(constants_1.CONSTRAINTS_FIELDS.RUNNER_JOBS.ERROR_MESSAGE.max)),
    tslib_1.__metadata("design:type", String)
], RunnerJobModel.prototype, "error", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", Number)
], RunnerJobModel.prototype, "priority", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(true),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", String)
], RunnerJobModel.prototype, "processingJobToken", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(true),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", Number)
], RunnerJobModel.prototype, "progress", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(true),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", Date)
], RunnerJobModel.prototype, "startedAt", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(true),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", Date)
], RunnerJobModel.prototype, "finishedAt", void 0);
tslib_1.__decorate([
    sequelize_typescript_1.CreatedAt,
    tslib_1.__metadata("design:type", Date)
], RunnerJobModel.prototype, "createdAt", void 0);
tslib_1.__decorate([
    sequelize_typescript_1.UpdatedAt,
    tslib_1.__metadata("design:type", Date)
], RunnerJobModel.prototype, "updatedAt", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.ForeignKey)(() => RunnerJobModel_1),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", Number)
], RunnerJobModel.prototype, "dependsOnRunnerJobId", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.BelongsTo)(() => RunnerJobModel_1, {
        foreignKey: {
            name: 'dependsOnRunnerJobId',
            allowNull: true
        },
        onDelete: 'cascade'
    }),
    tslib_1.__metadata("design:type", RunnerJobModel)
], RunnerJobModel.prototype, "DependsOnRunnerJob", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.ForeignKey)(() => runner_1.RunnerModel),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", Number)
], RunnerJobModel.prototype, "runnerId", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.BelongsTo)(() => runner_1.RunnerModel, {
        foreignKey: {
            name: 'runnerId',
            allowNull: true
        },
        onDelete: 'SET NULL'
    }),
    tslib_1.__metadata("design:type", runner_1.RunnerModel)
], RunnerJobModel.prototype, "Runner", void 0);
RunnerJobModel = RunnerJobModel_1 = tslib_1.__decorate([
    (0, sequelize_typescript_1.Scopes)(() => ({
        [ScopeNames.WITH_RUNNER]: {
            include: [
                {
                    model: runner_1.RunnerModel.unscoped(),
                    required: false
                }
            ]
        },
        [ScopeNames.WITH_PARENT]: {
            include: [
                {
                    model: RunnerJobModel_1.unscoped(),
                    required: false
                }
            ]
        }
    })),
    (0, sequelize_typescript_1.Table)({
        tableName: 'runnerJob',
        indexes: [
            {
                fields: ['uuid'],
                unique: true
            },
            {
                fields: ['processingJobToken'],
                unique: true
            },
            {
                fields: ['runnerId']
            }
        ]
    })
], RunnerJobModel);
exports.RunnerJobModel = RunnerJobModel;
//# sourceMappingURL=runner-job.js.map