"use strict";
var UserRegistrationModel_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserRegistrationModel = void 0;
const tslib_1 = require("tslib");
const sequelize_1 = require("sequelize");
const sequelize_typescript_1 = require("sequelize-typescript");
const user_registration_1 = require("../../helpers/custom-validators/user-registration");
const video_channels_1 = require("../../helpers/custom-validators/video-channels");
const peertube_crypto_1 = require("../../helpers/peertube-crypto");
const constants_1 = require("../../initializers/constants");
const users_1 = require("../../helpers/custom-validators/users");
const shared_1 = require("../shared");
const user_1 = require("./user");
let UserRegistrationModel = UserRegistrationModel_1 = class UserRegistrationModel extends sequelize_typescript_1.Model {
    static cryptPasswordIfNeeded(instance) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            instance.password = yield (0, peertube_crypto_1.cryptPassword)(instance.password);
        });
    }
    static load(id) {
        return UserRegistrationModel_1.findByPk(id);
    }
    static loadByEmail(email) {
        const query = {
            where: { email }
        };
        return UserRegistrationModel_1.findOne(query);
    }
    static loadByEmailOrUsername(emailOrUsername) {
        const query = {
            where: {
                [sequelize_1.Op.or]: [
                    { email: emailOrUsername },
                    { username: emailOrUsername }
                ]
            }
        };
        return UserRegistrationModel_1.findOne(query);
    }
    static loadByEmailOrHandle(options) {
        const { email, username, channelHandle } = options;
        let or = [
            { email },
            { channelHandle: username },
            { username }
        ];
        if (channelHandle) {
            or = or.concat([
                { username: channelHandle },
                { channelHandle }
            ]);
        }
        const query = {
            where: {
                [sequelize_1.Op.or]: or
            }
        };
        return UserRegistrationModel_1.findOne(query);
    }
    static listForApi(options) {
        const { start, count, sort, search } = options;
        const where = {};
        if (search) {
            Object.assign(where, {
                [sequelize_1.Op.or]: [
                    {
                        email: {
                            [sequelize_1.Op.iLike]: '%' + search + '%'
                        }
                    },
                    {
                        username: {
                            [sequelize_1.Op.iLike]: '%' + search + '%'
                        }
                    }
                ]
            });
        }
        const query = {
            offset: start,
            limit: count,
            order: (0, shared_1.getSort)(sort),
            where,
            include: [
                {
                    model: user_1.UserModel.unscoped(),
                    required: false
                }
            ]
        };
        return Promise.all([
            UserRegistrationModel_1.count(query),
            UserRegistrationModel_1.findAll(query)
        ]).then(([total, data]) => ({ total, data }));
    }
    toFormattedJSON() {
        return {
            id: this.id,
            state: {
                id: this.state,
                label: constants_1.USER_REGISTRATION_STATES[this.state]
            },
            registrationReason: this.registrationReason,
            moderationResponse: this.moderationResponse,
            username: this.username,
            email: this.email,
            emailVerified: this.emailVerified,
            accountDisplayName: this.accountDisplayName,
            channelHandle: this.channelHandle,
            channelDisplayName: this.channelDisplayName,
            createdAt: this.createdAt,
            updatedAt: this.updatedAt,
            user: this.User
                ? { id: this.User.id }
                : null
        };
    }
};
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    (0, sequelize_typescript_1.Is)('RegistrationState', value => (0, shared_1.throwIfNotValid)(value, user_registration_1.isRegistrationStateValid, 'state')),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", Number)
], UserRegistrationModel.prototype, "state", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    (0, sequelize_typescript_1.Is)('RegistrationReason', value => (0, shared_1.throwIfNotValid)(value, user_registration_1.isRegistrationReasonValid, 'registration reason')),
    (0, sequelize_typescript_1.Column)(sequelize_typescript_1.DataType.TEXT),
    tslib_1.__metadata("design:type", String)
], UserRegistrationModel.prototype, "registrationReason", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(true),
    (0, sequelize_typescript_1.Is)('RegistrationModerationResponse', value => (0, shared_1.throwIfNotValid)(value, user_registration_1.isRegistrationModerationResponseValid, 'moderation response', true)),
    (0, sequelize_typescript_1.Column)(sequelize_typescript_1.DataType.TEXT),
    tslib_1.__metadata("design:type", String)
], UserRegistrationModel.prototype, "moderationResponse", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(true),
    (0, sequelize_typescript_1.Is)('RegistrationPassword', value => (0, shared_1.throwIfNotValid)(value, users_1.isUserPasswordValid, 'registration password', true)),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", String)
], UserRegistrationModel.prototype, "password", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", String)
], UserRegistrationModel.prototype, "username", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    sequelize_typescript_1.IsEmail,
    (0, sequelize_typescript_1.Column)(sequelize_typescript_1.DataType.STRING(400)),
    tslib_1.__metadata("design:type", String)
], UserRegistrationModel.prototype, "email", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(true),
    (0, sequelize_typescript_1.Is)('RegistrationEmailVerified', value => (0, shared_1.throwIfNotValid)(value, users_1.isUserEmailVerifiedValid, 'email verified boolean', true)),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", Boolean)
], UserRegistrationModel.prototype, "emailVerified", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(true),
    (0, sequelize_typescript_1.Is)('RegistrationAccountDisplayName', value => (0, shared_1.throwIfNotValid)(value, users_1.isUserDisplayNameValid, 'account display name', true)),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", String)
], UserRegistrationModel.prototype, "accountDisplayName", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(true),
    (0, sequelize_typescript_1.Is)('ChannelHandle', value => (0, shared_1.throwIfNotValid)(value, video_channels_1.isVideoChannelDisplayNameValid, 'channel handle', true)),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", String)
], UserRegistrationModel.prototype, "channelHandle", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.AllowNull)(true),
    (0, sequelize_typescript_1.Is)('ChannelDisplayName', value => (0, shared_1.throwIfNotValid)(value, video_channels_1.isVideoChannelDisplayNameValid, 'channel display name', true)),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", String)
], UserRegistrationModel.prototype, "channelDisplayName", void 0);
tslib_1.__decorate([
    sequelize_typescript_1.CreatedAt,
    tslib_1.__metadata("design:type", Date)
], UserRegistrationModel.prototype, "createdAt", void 0);
tslib_1.__decorate([
    sequelize_typescript_1.UpdatedAt,
    tslib_1.__metadata("design:type", Date)
], UserRegistrationModel.prototype, "updatedAt", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.ForeignKey)(() => user_1.UserModel),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", Number)
], UserRegistrationModel.prototype, "userId", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.BelongsTo)(() => user_1.UserModel, {
        foreignKey: {
            allowNull: true
        },
        onDelete: 'SET NULL'
    }),
    tslib_1.__metadata("design:type", user_1.UserModel)
], UserRegistrationModel.prototype, "User", void 0);
tslib_1.__decorate([
    sequelize_typescript_1.BeforeCreate,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [UserRegistrationModel]),
    tslib_1.__metadata("design:returntype", Promise)
], UserRegistrationModel, "cryptPasswordIfNeeded", null);
UserRegistrationModel = UserRegistrationModel_1 = tslib_1.__decorate([
    (0, sequelize_typescript_1.Table)({
        tableName: 'userRegistration',
        indexes: [
            {
                fields: ['username'],
                unique: true
            },
            {
                fields: ['email'],
                unique: true
            },
            {
                fields: ['channelHandle'],
                unique: true
            },
            {
                fields: ['userId'],
                unique: true
            }
        ]
    })
], UserRegistrationModel);
exports.UserRegistrationModel = UserRegistrationModel;
//# sourceMappingURL=user-registration.js.map