"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chai_1 = require("chai");
const peertube_crypto_1 = require("../../../helpers/peertube-crypto");
const constants_1 = require("../../../initializers/constants");
const context_1 = require("../../../lib/activitypub/context");
const send_1 = require("../../../lib/activitypub/send");
const shared_1 = require("../../shared");
const core_utils_1 = require("../../../../shared/core-utils");
const server_commands_1 = require("../../../../shared/server-commands");
function setKeysOfServer(onServer, ofServerUrl, publicKey, privateKey) {
    const url = ofServerUrl + '/accounts/peertube';
    return Promise.all([
        onServer.setActorField(url, 'publicKey', publicKey),
        onServer.setActorField(url, 'privateKey', privateKey)
    ]);
}
function setUpdatedAtOfServer(onServer, ofServerUrl, updatedAt) {
    const url = ofServerUrl + '/accounts/peertube';
    return Promise.all([
        onServer.setActorField(url, 'createdAt', updatedAt),
        onServer.setActorField(url, 'updatedAt', updatedAt)
    ]);
}
function getAnnounceWithoutContext(server) {
    const json = require((0, core_utils_1.buildAbsoluteFixturePath)('./ap-json/peertube/announce-without-context.json'));
    const result = {};
    for (const key of Object.keys(json)) {
        if (Array.isArray(json[key])) {
            result[key] = json[key].map(v => v.replace(':9002', `:${server.port}`));
        }
        else {
            result[key] = json[key].replace(':9002', `:${server.port}`);
        }
    }
    return result;
}
function makeFollowRequest(to, by) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const follow = {
            type: 'Follow',
            id: by.url + '/' + new Date().getTime(),
            actor: by.url,
            object: to.url
        };
        const body = yield (0, context_1.activityPubContextify)(follow, 'Follow');
        const httpSignature = {
            algorithm: constants_1.HTTP_SIGNATURE.ALGORITHM,
            authorizationHeaderName: constants_1.HTTP_SIGNATURE.HEADER_NAME,
            keyId: by.url,
            key: by.privateKey,
            headers: constants_1.HTTP_SIGNATURE.HEADERS_TO_SIGN
        };
        const headers = {
            'digest': (0, peertube_crypto_1.buildDigest)(body),
            'content-type': 'application/activity+json',
            'accept': constants_1.ACTIVITY_PUB.ACCEPT_HEADER
        };
        return (0, shared_1.makePOSTAPRequest)(to.url + '/inbox', body, httpSignature, headers);
    });
}
describe('Test ActivityPub security', function () {
    let servers;
    let sqlCommands = [];
    let url;
    const keys = require((0, core_utils_1.buildAbsoluteFixturePath)('./ap-json/peertube/keys.json'));
    const invalidKeys = require((0, core_utils_1.buildAbsoluteFixturePath)('./ap-json/peertube/invalid-keys.json'));
    const baseHttpSignature = () => ({
        algorithm: constants_1.HTTP_SIGNATURE.ALGORITHM,
        authorizationHeaderName: constants_1.HTTP_SIGNATURE.HEADER_NAME,
        keyId: 'acct:peertube@' + servers[1].host,
        key: keys.privateKey,
        headers: constants_1.HTTP_SIGNATURE.HEADERS_TO_SIGN
    });
    before(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(60000);
            servers = yield (0, server_commands_1.createMultipleServers)(3);
            sqlCommands = servers.map(s => new shared_1.SQLCommand(s));
            url = servers[0].url + '/inbox';
            yield setKeysOfServer(sqlCommands[0], servers[1].url, keys.publicKey, null);
            yield setKeysOfServer(sqlCommands[1], servers[1].url, keys.publicKey, keys.privateKey);
            const to = { url: servers[0].url + '/accounts/peertube' };
            const by = { url: servers[1].url + '/accounts/peertube', privateKey: keys.privateKey };
            yield makeFollowRequest(to, by);
        });
    });
    describe('When checking HTTP signature', function () {
        it('Should fail with an invalid digest', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const body = yield (0, context_1.activityPubContextify)(getAnnounceWithoutContext(servers[1]), 'Announce');
                const headers = {
                    Digest: (0, peertube_crypto_1.buildDigest)({ hello: 'coucou' })
                };
                try {
                    yield (0, shared_1.makePOSTAPRequest)(url, body, baseHttpSignature(), headers);
                    (0, chai_1.expect)(true, 'Did not throw').to.be.false;
                }
                catch (err) {
                    (0, chai_1.expect)(err.statusCode).to.equal(403);
                }
            });
        });
        it('Should fail with an invalid date', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const body = yield (0, context_1.activityPubContextify)(getAnnounceWithoutContext(servers[1]), 'Announce');
                const headers = (0, send_1.buildGlobalHeaders)(body);
                headers['date'] = 'Wed, 21 Oct 2015 07:28:00 GMT';
                try {
                    yield (0, shared_1.makePOSTAPRequest)(url, body, baseHttpSignature(), headers);
                    (0, chai_1.expect)(true, 'Did not throw').to.be.false;
                }
                catch (err) {
                    (0, chai_1.expect)(err.statusCode).to.equal(403);
                }
            });
        });
        it('Should fail with bad keys', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield setKeysOfServer(sqlCommands[0], servers[1].url, invalidKeys.publicKey, invalidKeys.privateKey);
                yield setKeysOfServer(sqlCommands[1], servers[1].url, invalidKeys.publicKey, invalidKeys.privateKey);
                const body = yield (0, context_1.activityPubContextify)(getAnnounceWithoutContext(servers[1]), 'Announce');
                const headers = (0, send_1.buildGlobalHeaders)(body);
                try {
                    yield (0, shared_1.makePOSTAPRequest)(url, body, baseHttpSignature(), headers);
                    (0, chai_1.expect)(true, 'Did not throw').to.be.false;
                }
                catch (err) {
                    (0, chai_1.expect)(err.statusCode).to.equal(403);
                }
            });
        });
        it('Should reject requests without appropriate signed headers', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield setKeysOfServer(sqlCommands[0], servers[1].url, keys.publicKey, keys.privateKey);
                yield setKeysOfServer(sqlCommands[1], servers[1].url, keys.publicKey, keys.privateKey);
                const body = yield (0, context_1.activityPubContextify)(getAnnounceWithoutContext(servers[1]), 'Announce');
                const headers = (0, send_1.buildGlobalHeaders)(body);
                const signatureOptions = baseHttpSignature();
                const badHeadersMatrix = [
                    ['(request-target)', 'date', 'digest'],
                    ['host', 'date', 'digest'],
                    ['(request-target)', 'host', 'digest']
                ];
                for (const badHeaders of badHeadersMatrix) {
                    signatureOptions.headers = badHeaders;
                    try {
                        yield (0, shared_1.makePOSTAPRequest)(url, body, signatureOptions, headers);
                        (0, chai_1.expect)(true, 'Did not throw').to.be.false;
                    }
                    catch (err) {
                        (0, chai_1.expect)(err.statusCode).to.equal(403);
                    }
                }
            });
        });
        it('Should succeed with a valid HTTP signature draft 11 (without date but with (created))', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const body = yield (0, context_1.activityPubContextify)(getAnnounceWithoutContext(servers[1]), 'Announce');
                const headers = (0, send_1.buildGlobalHeaders)(body);
                const signatureOptions = baseHttpSignature();
                signatureOptions.headers = ['(request-target)', '(created)', 'host', 'digest'];
                const { statusCode } = yield (0, shared_1.makePOSTAPRequest)(url, body, signatureOptions, headers);
                (0, chai_1.expect)(statusCode).to.equal(204);
            });
        });
        it('Should succeed with a valid HTTP signature', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const body = yield (0, context_1.activityPubContextify)(getAnnounceWithoutContext(servers[1]), 'Announce');
                const headers = (0, send_1.buildGlobalHeaders)(body);
                const { statusCode } = yield (0, shared_1.makePOSTAPRequest)(url, body, baseHttpSignature(), headers);
                (0, chai_1.expect)(statusCode).to.equal(204);
            });
        });
        it('Should refresh the actor keys', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(20000);
                yield setKeysOfServer(sqlCommands[1], servers[1].url, invalidKeys.publicKey, invalidKeys.privateKey);
                yield setUpdatedAtOfServer(sqlCommands[0], servers[1].url, '2015-07-17 22:00:00+00');
                yield (0, server_commands_1.killallServers)([servers[1]]);
                yield servers[1].run();
                const body = yield (0, context_1.activityPubContextify)(getAnnounceWithoutContext(servers[1]), 'Announce');
                const headers = (0, send_1.buildGlobalHeaders)(body);
                try {
                    yield (0, shared_1.makePOSTAPRequest)(url, body, baseHttpSignature(), headers);
                    (0, chai_1.expect)(true, 'Did not throw').to.be.false;
                }
                catch (err) {
                    console.error(err);
                    (0, chai_1.expect)(err.statusCode).to.equal(403);
                }
            });
        });
    });
    describe('When checking Linked Data Signature', function () {
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield setKeysOfServer(sqlCommands[0], servers[1].url, keys.publicKey, keys.privateKey);
                yield setKeysOfServer(sqlCommands[1], servers[1].url, keys.publicKey, keys.privateKey);
                yield setKeysOfServer(sqlCommands[2], servers[2].url, keys.publicKey, keys.privateKey);
                const to = { url: servers[0].url + '/accounts/peertube' };
                const by = { url: servers[2].url + '/accounts/peertube', privateKey: keys.privateKey };
                yield makeFollowRequest(to, by);
            });
        });
        it('Should fail with bad keys', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield setKeysOfServer(sqlCommands[0], servers[2].url, invalidKeys.publicKey, invalidKeys.privateKey);
                yield setKeysOfServer(sqlCommands[2], servers[2].url, invalidKeys.publicKey, invalidKeys.privateKey);
                const body = getAnnounceWithoutContext(servers[1]);
                body.actor = servers[2].url + '/accounts/peertube';
                const signer = { privateKey: invalidKeys.privateKey, url: servers[2].url + '/accounts/peertube' };
                const signedBody = yield (0, send_1.signAndContextify)(signer, body, 'Announce');
                const headers = (0, send_1.buildGlobalHeaders)(signedBody);
                try {
                    yield (0, shared_1.makePOSTAPRequest)(url, signedBody, baseHttpSignature(), headers);
                    (0, chai_1.expect)(true, 'Did not throw').to.be.false;
                }
                catch (err) {
                    (0, chai_1.expect)(err.statusCode).to.equal(403);
                }
            });
        });
        it('Should fail with an altered body', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield setKeysOfServer(sqlCommands[0], servers[2].url, keys.publicKey, keys.privateKey);
                yield setKeysOfServer(sqlCommands[0], servers[2].url, keys.publicKey, keys.privateKey);
                const body = getAnnounceWithoutContext(servers[1]);
                body.actor = servers[2].url + '/accounts/peertube';
                const signer = { privateKey: keys.privateKey, url: servers[2].url + '/accounts/peertube' };
                const signedBody = yield (0, send_1.signAndContextify)(signer, body, 'Announce');
                signedBody.actor = servers[2].url + '/account/peertube';
                const headers = (0, send_1.buildGlobalHeaders)(signedBody);
                try {
                    yield (0, shared_1.makePOSTAPRequest)(url, signedBody, baseHttpSignature(), headers);
                    (0, chai_1.expect)(true, 'Did not throw').to.be.false;
                }
                catch (err) {
                    (0, chai_1.expect)(err.statusCode).to.equal(403);
                }
            });
        });
        it('Should succeed with a valid signature', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const body = getAnnounceWithoutContext(servers[1]);
                body.actor = servers[2].url + '/accounts/peertube';
                const signer = { privateKey: keys.privateKey, url: servers[2].url + '/accounts/peertube' };
                const signedBody = yield (0, send_1.signAndContextify)(signer, body, 'Announce');
                const headers = (0, send_1.buildGlobalHeaders)(signedBody);
                const { statusCode } = yield (0, shared_1.makePOSTAPRequest)(url, signedBody, baseHttpSignature(), headers);
                (0, chai_1.expect)(statusCode).to.equal(204);
            });
        });
        it('Should refresh the actor keys', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(20000);
                yield (0, core_utils_1.wait)(10000);
                yield setKeysOfServer(sqlCommands[2], servers[2].url, invalidKeys.publicKey, invalidKeys.privateKey);
                const body = getAnnounceWithoutContext(servers[1]);
                body.actor = servers[2].url + '/accounts/peertube';
                const signer = { privateKey: keys.privateKey, url: servers[2].url + '/accounts/peertube' };
                const signedBody = yield (0, send_1.signAndContextify)(signer, body, 'Announce');
                const headers = (0, send_1.buildGlobalHeaders)(signedBody);
                try {
                    yield (0, shared_1.makePOSTAPRequest)(url, signedBody, baseHttpSignature(), headers);
                    (0, chai_1.expect)(true, 'Did not throw').to.be.false;
                }
                catch (err) {
                    (0, chai_1.expect)(err.statusCode).to.equal(403);
                }
            });
        });
    });
    after(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            for (const sql of sqlCommands) {
                yield sql.cleanup();
            }
            yield (0, server_commands_1.cleanupTests)(servers);
        });
    });
});
//# sourceMappingURL=security.js.map