"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const shared_1 = require("../../shared");
const core_utils_1 = require("../../../../shared/core-utils");
const server_commands_1 = require("../../../../shared/server-commands");
describe('Test registrations API validators', function () {
    let server;
    let userToken;
    let moderatorToken;
    before(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(30000);
            server = yield (0, server_commands_1.createSingleServer)(1);
            yield (0, server_commands_1.setAccessTokensToServers)([server]);
            yield (0, server_commands_1.setDefaultAccountAvatar)([server]);
            yield (0, server_commands_1.setDefaultChannelAvatar)([server]);
            yield server.config.enableSignup(false);
            ({ token: moderatorToken } = yield server.users.generate('moderator', 1));
            ({ token: userToken } = yield server.users.generate('user', 2));
        });
    });
    describe('Register', function () {
        const registrationPath = '/api/v1/users/register';
        const registrationRequestPath = '/api/v1/users/registrations/request';
        const baseCorrectParams = {
            username: 'user3',
            displayName: 'super user',
            email: 'test3@example.com',
            password: 'my super password',
            registrationReason: 'my super registration reason'
        };
        describe('When registering a new user or requesting user registration', function () {
            function check(fields, expectedStatus = 400) {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.config.enableSignup(false);
                    yield (0, server_commands_1.makePostBodyRequest)({ url: server.url, path: registrationPath, fields, expectedStatus });
                    yield server.config.enableSignup(true);
                    yield (0, server_commands_1.makePostBodyRequest)({ url: server.url, path: registrationRequestPath, fields, expectedStatus });
                });
            }
            it('Should fail with a too small username', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const fields = Object.assign(Object.assign({}, baseCorrectParams), { username: '' });
                    yield check(fields);
                });
            });
            it('Should fail with a too long username', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const fields = Object.assign(Object.assign({}, baseCorrectParams), { username: 'super'.repeat(50) });
                    yield check(fields);
                });
            });
            it('Should fail with an incorrect username', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const fields = Object.assign(Object.assign({}, baseCorrectParams), { username: 'my username' });
                    yield check(fields);
                });
            });
            it('Should fail with a missing email', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const fields = (0, core_utils_1.omit)(baseCorrectParams, ['email']);
                    yield check(fields);
                });
            });
            it('Should fail with an invalid email', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const fields = Object.assign(Object.assign({}, baseCorrectParams), { email: 'test_example.com' });
                    yield check(fields);
                });
            });
            it('Should fail with a too small password', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const fields = Object.assign(Object.assign({}, baseCorrectParams), { password: 'bla' });
                    yield check(fields);
                });
            });
            it('Should fail with a too long password', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const fields = Object.assign(Object.assign({}, baseCorrectParams), { password: 'super'.repeat(61) });
                    yield check(fields);
                });
            });
            it('Should fail if we register a user with the same username', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const fields = Object.assign(Object.assign({}, baseCorrectParams), { username: 'root' });
                    yield check(fields, 409);
                });
            });
            it('Should fail with a "peertube" username', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const fields = Object.assign(Object.assign({}, baseCorrectParams), { username: 'peertube' });
                    yield check(fields, 409);
                });
            });
            it('Should fail if we register a user with the same email', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const fields = Object.assign(Object.assign({}, baseCorrectParams), { email: 'admin' + server.internalServerNumber + '@example.com' });
                    yield check(fields, 409);
                });
            });
            it('Should fail with a bad display name', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const fields = Object.assign(Object.assign({}, baseCorrectParams), { displayName: 'a'.repeat(150) });
                    yield check(fields);
                });
            });
            it('Should fail with a bad channel name', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const fields = Object.assign(Object.assign({}, baseCorrectParams), { channel: { name: '[]azf', displayName: 'toto' } });
                    yield check(fields);
                });
            });
            it('Should fail with a bad channel display name', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const fields = Object.assign(Object.assign({}, baseCorrectParams), { channel: { name: 'toto', displayName: '' } });
                    yield check(fields);
                });
            });
            it('Should fail with a channel name that is the same as username', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const source = { username: 'super_user', channel: { name: 'super_user', displayName: 'display name' } };
                    const fields = Object.assign(Object.assign({}, baseCorrectParams), source);
                    yield check(fields);
                });
            });
            it('Should fail with an existing channel', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const attributes = { name: 'existing_channel', displayName: 'hello', description: 'super description' };
                    yield server.channels.create({ attributes });
                    const fields = Object.assign(Object.assign({}, baseCorrectParams), { channel: { name: 'existing_channel', displayName: 'toto' } });
                    yield check(fields, 409);
                });
            });
            it('Should fail on a server with registration disabled', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    this.timeout(60000);
                    yield server.config.updateExistingSubConfig({
                        newConfig: {
                            signup: {
                                enabled: false
                            }
                        }
                    });
                    yield server.registrations.register({ username: 'user4', expectedStatus: 403 });
                    yield server.registrations.requestRegistration({
                        username: 'user4',
                        registrationReason: 'reason',
                        expectedStatus: 403
                    });
                });
            });
            it('Should fail if the user limit is reached', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    this.timeout(60000);
                    const { total } = yield server.users.list();
                    yield server.config.enableSignup(false, total);
                    yield server.registrations.register({ username: 'user42', expectedStatus: 403 });
                    yield server.config.enableSignup(true, total);
                    yield server.registrations.requestRegistration({
                        username: 'user42',
                        registrationReason: 'reason',
                        expectedStatus: 403
                    });
                });
            });
            it('Should succeed if the user limit is not reached', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    this.timeout(60000);
                    const { total } = yield server.users.list();
                    yield server.config.enableSignup(false, total + 1);
                    yield server.registrations.register({ username: 'user43', expectedStatus: 204 });
                    yield server.config.enableSignup(true, total + 2);
                    yield server.registrations.requestRegistration({
                        username: 'user44',
                        registrationReason: 'reason',
                        expectedStatus: 200
                    });
                });
            });
        });
        describe('On direct registration', function () {
            it('Should succeed with the correct params', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.config.enableSignup(false);
                    const fields = {
                        username: 'user_direct_1',
                        displayName: 'super user direct 1',
                        email: 'user_direct_1@example.com',
                        password: 'my super password',
                        channel: { name: 'super_user_direct_1_channel', displayName: 'super user direct 1 channel' }
                    };
                    yield (0, server_commands_1.makePostBodyRequest)({ url: server.url, path: registrationPath, fields, expectedStatus: 204 });
                });
            });
            it('Should fail if the instance requires approval', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    this.timeout(60000);
                    yield server.config.enableSignup(true);
                    yield server.registrations.register({ username: 'user42', expectedStatus: 403 });
                });
            });
        });
        describe('On registration request', function () {
            before(function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    this.timeout(60000);
                    yield server.config.enableSignup(true);
                });
            });
            it('Should fail with an invalid registration reason', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    for (const registrationReason of ['', 't', 't'.repeat(5000)]) {
                        yield server.registrations.requestRegistration({
                            username: 'user_request_1',
                            registrationReason,
                            expectedStatus: 400
                        });
                    }
                });
            });
            it('Should succeed with the correct params', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.registrations.requestRegistration({
                        username: 'user_request_2',
                        registrationReason: 'tt',
                        channel: {
                            displayName: 'my user request 2 channel',
                            name: 'user_request_2_channel'
                        }
                    });
                });
            });
            it('Should fail if the username is already awaiting registration approval', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.registrations.requestRegistration({
                        username: 'user_request_2',
                        registrationReason: 'tt',
                        channel: {
                            displayName: 'my user request 42 channel',
                            name: 'user_request_42_channel'
                        },
                        expectedStatus: 409
                    });
                });
            });
            it('Should fail if the email is already awaiting registration approval', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.registrations.requestRegistration({
                        username: 'user42',
                        email: 'user_request_2@example.com',
                        registrationReason: 'tt',
                        channel: {
                            displayName: 'my user request 42 channel',
                            name: 'user_request_42_channel'
                        },
                        expectedStatus: 409
                    });
                });
            });
            it('Should fail if the channel is already awaiting registration approval', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.registrations.requestRegistration({
                        username: 'user42',
                        registrationReason: 'tt',
                        channel: {
                            displayName: 'my user request 2 channel',
                            name: 'user_request_2_channel'
                        },
                        expectedStatus: 409
                    });
                });
            });
            it('Should fail if the instance does not require approval', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    this.timeout(60000);
                    yield server.config.enableSignup(false);
                    yield server.registrations.requestRegistration({
                        username: 'user42',
                        registrationReason: 'toto',
                        expectedStatus: 400
                    });
                });
            });
        });
    });
    describe('Registrations accept/reject', function () {
        let id1;
        let id2;
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(60000);
                yield server.config.enableSignup(true);
                ({ id: id1 } = yield server.registrations.requestRegistration({ username: 'request_2', registrationReason: 'toto' }));
                ({ id: id2 } = yield server.registrations.requestRegistration({ username: 'request_3', registrationReason: 'toto' }));
            });
        });
        it('Should fail to accept/reject registration without token', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const options = { id: id1, moderationResponse: 'tt', token: null, expectedStatus: 401 };
                yield server.registrations.accept(options);
                yield server.registrations.reject(options);
            });
        });
        it('Should fail to accept/reject registration with a non moderator user', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const options = { id: id1, moderationResponse: 'tt', token: userToken, expectedStatus: 403 };
                yield server.registrations.accept(options);
                yield server.registrations.reject(options);
            });
        });
        it('Should fail to accept/reject registration with a bad registration id', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                {
                    const options = { id: 't', moderationResponse: 'tt', token: moderatorToken, expectedStatus: 400 };
                    yield server.registrations.accept(options);
                    yield server.registrations.reject(options);
                }
                {
                    const options = { id: 42, moderationResponse: 'tt', token: moderatorToken, expectedStatus: 404 };
                    yield server.registrations.accept(options);
                    yield server.registrations.reject(options);
                }
            });
        });
        it('Should fail to accept/reject registration with a bad moderation resposne', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                for (const moderationResponse of ['', 't', 't'.repeat(5000)]) {
                    const options = { id: id1, moderationResponse, token: moderatorToken, expectedStatus: 400 };
                    yield server.registrations.accept(options);
                    yield server.registrations.reject(options);
                }
            });
        });
        it('Should succeed to accept a registration', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.registrations.accept({ id: id1, moderationResponse: 'tt', token: moderatorToken });
            });
        });
        it('Should succeed to reject a registration', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.registrations.reject({ id: id2, moderationResponse: 'tt', token: moderatorToken });
            });
        });
        it('Should fail to accept/reject a registration that was already accepted/rejected', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                for (const id of [id1, id2]) {
                    const options = { id, moderationResponse: 'tt', token: moderatorToken, expectedStatus: 409 };
                    yield server.registrations.accept(options);
                    yield server.registrations.reject(options);
                }
            });
        });
    });
    describe('Registrations deletion', function () {
        let id1;
        let id2;
        let id3;
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                ({ id: id1 } = yield server.registrations.requestRegistration({ username: 'request_4', registrationReason: 'toto' }));
                ({ id: id2 } = yield server.registrations.requestRegistration({ username: 'request_5', registrationReason: 'toto' }));
                ({ id: id3 } = yield server.registrations.requestRegistration({ username: 'request_6', registrationReason: 'toto' }));
                yield server.registrations.accept({ id: id2, moderationResponse: 'tt' });
                yield server.registrations.reject({ id: id3, moderationResponse: 'tt' });
            });
        });
        it('Should fail to delete registration without token', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.registrations.delete({ id: id1, token: null, expectedStatus: 401 });
            });
        });
        it('Should fail to delete registration with a non moderator user', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.registrations.delete({ id: id1, token: userToken, expectedStatus: 403 });
            });
        });
        it('Should fail to delete registration with a bad registration id', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.registrations.delete({ id: 't', token: moderatorToken, expectedStatus: 400 });
                yield server.registrations.delete({ id: 42, token: moderatorToken, expectedStatus: 404 });
            });
        });
        it('Should succeed with the correct params', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.registrations.delete({ id: id1, token: moderatorToken });
                yield server.registrations.delete({ id: id2, token: moderatorToken });
                yield server.registrations.delete({ id: id3, token: moderatorToken });
            });
        });
    });
    describe('Listing registrations', function () {
        const path = '/api/v1/users/registrations';
        it('Should fail with a bad start pagination', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield (0, shared_1.checkBadStartPagination)(server.url, path, server.accessToken);
            });
        });
        it('Should fail with a bad count pagination', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield (0, shared_1.checkBadCountPagination)(server.url, path, server.accessToken);
            });
        });
        it('Should fail with an incorrect sort', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield (0, shared_1.checkBadSortPagination)(server.url, path, server.accessToken);
            });
        });
        it('Should fail with a non authenticated user', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.registrations.list({
                    token: null,
                    expectedStatus: 401
                });
            });
        });
        it('Should fail with a non admin user', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.registrations.list({
                    token: userToken,
                    expectedStatus: 403
                });
            });
        });
        it('Should succeed with the correct params', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.registrations.list({
                    token: moderatorToken,
                    search: 'toto'
                });
            });
        });
    });
    after(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield (0, server_commands_1.cleanupTests)([server]);
        });
    });
});
//# sourceMappingURL=registrations.js.map