"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path_1 = require("path");
const shared_1 = require("../../shared");
const models_1 = require("../../../../shared/models");
const server_commands_1 = require("../../../../shared/server-commands");
const badUUID = '910ec12a-d9e6-458b-a274-0abb655f9464';
describe('Test managing runners', function () {
    let server;
    let userToken;
    let registrationTokenId;
    let registrationToken;
    let runnerToken;
    let runnerToken2;
    let completedJobToken;
    let completedJobUUID;
    let cancelledJobUUID;
    before(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(120000);
            const config = {
                rates_limit: {
                    api: {
                        max: 5000
                    }
                }
            };
            server = yield (0, server_commands_1.createSingleServer)(1, config);
            yield (0, server_commands_1.setAccessTokensToServers)([server]);
            yield (0, server_commands_1.setDefaultVideoChannel)([server]);
            userToken = yield server.users.generateUserAndToken('user1');
            const { data } = yield server.runnerRegistrationTokens.list();
            registrationToken = data[0].registrationToken;
            registrationTokenId = data[0].id;
            yield server.config.enableTranscoding(true, true);
            yield server.config.enableStudio();
            yield server.config.enableRemoteTranscoding();
            yield server.config.enableRemoteStudio();
            runnerToken = yield server.runners.autoRegisterRunner();
            runnerToken2 = yield server.runners.autoRegisterRunner();
            {
                yield server.videos.quickUpload({ name: 'video 1' });
                yield server.videos.quickUpload({ name: 'video 2' });
                yield (0, server_commands_1.waitJobs)([server]);
                {
                    const job = yield server.runnerJobs.autoProcessWebVideoJob(runnerToken);
                    completedJobToken = job.jobToken;
                    completedJobUUID = job.uuid;
                }
                {
                    const { job } = yield server.runnerJobs.autoAccept({ runnerToken });
                    cancelledJobUUID = job.uuid;
                    yield server.runnerJobs.cancelByAdmin({ jobUUID: cancelledJobUUID });
                }
            }
        });
    });
    describe('Managing runner registration tokens', function () {
        describe('Common', function () {
            it('Should fail to generate, list or delete runner registration token without oauth token', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const expectedStatus = 401;
                    yield server.runnerRegistrationTokens.generate({ token: null, expectedStatus });
                    yield server.runnerRegistrationTokens.list({ token: null, expectedStatus });
                    yield server.runnerRegistrationTokens.delete({ token: null, id: registrationTokenId, expectedStatus });
                });
            });
            it('Should fail to generate, list or delete runner registration token without admin rights', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const expectedStatus = 403;
                    yield server.runnerRegistrationTokens.generate({ token: userToken, expectedStatus });
                    yield server.runnerRegistrationTokens.list({ token: userToken, expectedStatus });
                    yield server.runnerRegistrationTokens.delete({ token: userToken, id: registrationTokenId, expectedStatus });
                });
            });
        });
        describe('Delete', function () {
            it('Should fail to delete with a bad id', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerRegistrationTokens.delete({ id: 404, expectedStatus: 404 });
                });
            });
        });
        describe('List', function () {
            const path = '/api/v1/runners/registration-tokens';
            it('Should fail to list with a bad start pagination', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield (0, shared_1.checkBadStartPagination)(server.url, path, server.accessToken);
                });
            });
            it('Should fail to list with a bad count pagination', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield (0, shared_1.checkBadCountPagination)(server.url, path, server.accessToken);
                });
            });
            it('Should fail to list with an incorrect sort', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield (0, shared_1.checkBadSortPagination)(server.url, path, server.accessToken);
                });
            });
            it('Should succeed to list with the correct params', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerRegistrationTokens.list({ start: 0, count: 5, sort: '-createdAt' });
                });
            });
        });
    });
    describe('Managing runners', function () {
        let toDeleteId;
        describe('Register', function () {
            const name = 'runner name';
            it('Should fail with a bad registration token', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const expectedStatus = 400;
                    yield server.runners.register({ name, registrationToken: 'a'.repeat(4000), expectedStatus });
                    yield server.runners.register({ name, registrationToken: null, expectedStatus });
                });
            });
            it('Should fail with an unknown registration token', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runners.register({ name, registrationToken: 'aaa', expectedStatus: 404 });
                });
            });
            it('Should fail with a bad name', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const expectedStatus = 400;
                    yield server.runners.register({ name: '', registrationToken, expectedStatus });
                    yield server.runners.register({ name: 'a'.repeat(200), registrationToken, expectedStatus });
                });
            });
            it('Should fail with an invalid description', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const expectedStatus = 400;
                    yield server.runners.register({ name, description: '', registrationToken, expectedStatus });
                    yield server.runners.register({ name, description: 'a'.repeat(5000), registrationToken, expectedStatus });
                });
            });
            it('Should succeed with the correct params', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const { id } = yield server.runners.register({ name, description: 'super description', registrationToken });
                    toDeleteId = id;
                });
            });
        });
        describe('Delete', function () {
            it('Should fail without oauth token', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runners.delete({ token: null, id: toDeleteId, expectedStatus: 401 });
                });
            });
            it('Should fail without admin rights', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runners.delete({ token: userToken, id: toDeleteId, expectedStatus: 403 });
                });
            });
            it('Should fail with a bad id', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runners.delete({ id: 'hi', expectedStatus: 400 });
                });
            });
            it('Should fail with an unknown id', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runners.delete({ id: 404, expectedStatus: 404 });
                });
            });
            it('Should succeed with the correct params', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runners.delete({ id: toDeleteId });
                });
            });
        });
        describe('List', function () {
            const path = '/api/v1/runners';
            it('Should fail without oauth token', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runners.list({ token: null, expectedStatus: 401 });
                });
            });
            it('Should fail without admin rights', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runners.list({ token: userToken, expectedStatus: 403 });
                });
            });
            it('Should fail to list with a bad start pagination', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield (0, shared_1.checkBadStartPagination)(server.url, path, server.accessToken);
                });
            });
            it('Should fail to list with a bad count pagination', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield (0, shared_1.checkBadCountPagination)(server.url, path, server.accessToken);
                });
            });
            it('Should fail to list with an incorrect sort', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield (0, shared_1.checkBadSortPagination)(server.url, path, server.accessToken);
                });
            });
            it('Should succeed to list with the correct params', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runners.list({ start: 0, count: 5, sort: '-createdAt' });
                });
            });
        });
    });
    describe('Runner jobs by admin', function () {
        describe('Cancel', function () {
            let jobUUID;
            before(function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    this.timeout(60000);
                    yield server.videos.quickUpload({ name: 'video' });
                    yield (0, server_commands_1.waitJobs)([server]);
                    const { availableJobs } = yield server.runnerJobs.request({ runnerToken });
                    jobUUID = availableJobs[0].uuid;
                });
            });
            it('Should fail without oauth token', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.cancelByAdmin({ token: null, jobUUID, expectedStatus: 401 });
                });
            });
            it('Should fail without admin rights', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.cancelByAdmin({ token: userToken, jobUUID, expectedStatus: 403 });
                });
            });
            it('Should fail with a bad job uuid', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.cancelByAdmin({ jobUUID: 'hello', expectedStatus: 400 });
                });
            });
            it('Should fail with an unknown job uuid', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const jobUUID = badUUID;
                    yield server.runnerJobs.cancelByAdmin({ jobUUID, expectedStatus: 404 });
                });
            });
            it('Should fail with an already cancelled job', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.cancelByAdmin({ jobUUID: cancelledJobUUID, expectedStatus: 400 });
                });
            });
            it('Should succeed with the correct params', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.cancelByAdmin({ jobUUID });
                });
            });
        });
        describe('List', function () {
            const path = '/api/v1/runners/jobs';
            it('Should fail without oauth token', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.list({ token: null, expectedStatus: 401 });
                });
            });
            it('Should fail without admin rights', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.list({ token: userToken, expectedStatus: 403 });
                });
            });
            it('Should fail to list with a bad start pagination', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield (0, shared_1.checkBadStartPagination)(server.url, path, server.accessToken);
                });
            });
            it('Should fail to list with a bad count pagination', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield (0, shared_1.checkBadCountPagination)(server.url, path, server.accessToken);
                });
            });
            it('Should fail to list with an incorrect sort', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield (0, shared_1.checkBadSortPagination)(server.url, path, server.accessToken);
                });
            });
            it('Should succeed to list with the correct params', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.list({ start: 0, count: 5, sort: '-createdAt' });
                });
            });
        });
    });
    describe('Runner jobs by runners', function () {
        let jobUUID;
        let jobToken;
        let videoUUID;
        let jobUUID2;
        let jobToken2;
        let videoUUID2;
        let pendingUUID;
        let videoStudioUUID;
        let studioFile;
        let liveAcceptedJob;
        let studioAcceptedJob;
        function fetchVideoInputFiles(options) {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { jobUUID, expectedStatus, videoUUID, runnerToken, jobToken } = options;
                const basePath = '/api/v1/runners/jobs/' + jobUUID + '/files/videos/' + videoUUID;
                const paths = [`${basePath}/max-quality`, `${basePath}/previews/max-quality`];
                for (const path of paths) {
                    yield (0, server_commands_1.makePostBodyRequest)({ url: server.url, path, fields: { runnerToken, jobToken }, expectedStatus });
                }
            });
        }
        function fetchStudioFiles(options) {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { jobUUID, expectedStatus, videoUUID, runnerToken, jobToken, studioFile } = options;
                const path = `/api/v1/runners/jobs/${jobUUID}/files/videos/${videoUUID}/studio/task-files/${studioFile}`;
                yield (0, server_commands_1.makePostBodyRequest)({ url: server.url, path, fields: { runnerToken, jobToken }, expectedStatus });
            });
        }
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(120000);
                {
                    yield server.runnerJobs.cancelAllJobs({ state: models_1.RunnerJobState.PENDING });
                }
                {
                    const { uuid } = yield server.videos.quickUpload({ name: 'video' });
                    videoUUID = uuid;
                    yield (0, server_commands_1.waitJobs)([server]);
                    const { job } = yield server.runnerJobs.autoAccept({ runnerToken });
                    jobUUID = job.uuid;
                    jobToken = job.jobToken;
                }
                {
                    const { uuid } = yield server.videos.quickUpload({ name: 'video' });
                    videoUUID2 = uuid;
                    yield (0, server_commands_1.waitJobs)([server]);
                    const { job } = yield server.runnerJobs.autoAccept({ runnerToken: runnerToken2 });
                    jobUUID2 = job.uuid;
                    jobToken2 = job.jobToken;
                }
                {
                    yield server.videos.quickUpload({ name: 'video' });
                    yield (0, server_commands_1.waitJobs)([server]);
                    const { availableJobs } = yield server.runnerJobs.request({ runnerToken });
                    pendingUUID = availableJobs[0].uuid;
                }
                {
                    yield server.config.disableTranscoding();
                    const { uuid } = yield server.videos.quickUpload({ name: 'video studio' });
                    videoStudioUUID = uuid;
                    yield server.config.enableTranscoding(true, true);
                    yield server.config.enableStudio();
                    yield server.videoStudio.createEditionTasks({
                        videoId: videoStudioUUID,
                        tasks: server_commands_1.VideoStudioCommand.getComplexTask()
                    });
                    const { job } = yield server.runnerJobs.autoAccept({ runnerToken, type: 'video-studio-transcoding' });
                    studioAcceptedJob = job;
                    const tasks = job.payload.tasks;
                    const fileUrl = tasks.find(t => (0, models_1.isVideoStudioTaskIntro)(t)).options.file;
                    studioFile = (0, path_1.basename)(fileUrl);
                }
                {
                    yield server.config.enableLive({
                        allowReplay: false,
                        resolutions: 'max',
                        transcoding: true
                    });
                    const { live } = yield server.live.quickCreate({ permanentLive: true, saveReplay: false, privacy: 1 });
                    const ffmpegCommand = (0, server_commands_1.sendRTMPStream)({ rtmpBaseUrl: live.rtmpUrl, streamKey: live.streamKey });
                    yield (0, server_commands_1.waitJobs)([server]);
                    yield server.runnerJobs.requestLiveJob(runnerToken);
                    const { job } = yield server.runnerJobs.autoAccept({ runnerToken, type: 'live-rtmp-hls-transcoding' });
                    liveAcceptedJob = job;
                    yield (0, server_commands_1.stopFfmpeg)(ffmpegCommand);
                }
            });
        });
        describe('Common runner tokens validations', function () {
            function testEndpoints(options) {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.abort(Object.assign(Object.assign({}, options), { reason: 'reason' }));
                    yield server.runnerJobs.update(Object.assign({}, options));
                    yield server.runnerJobs.error(Object.assign(Object.assign({}, options), { message: 'message' }));
                    yield server.runnerJobs.success(Object.assign(Object.assign({}, options), { payload: { videoFile: 'video_short.mp4' } }));
                });
            }
            it('Should fail with an invalid job uuid', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const options = { jobUUID: 'a', runnerToken, expectedStatus: 400 };
                    yield testEndpoints(Object.assign(Object.assign({}, options), { jobToken }));
                    yield fetchVideoInputFiles(Object.assign(Object.assign({}, options), { videoUUID, jobToken }));
                    yield fetchStudioFiles(Object.assign(Object.assign({}, options), { videoUUID, jobToken: studioAcceptedJob.jobToken, studioFile }));
                });
            });
            it('Should fail with an unknown job uuid', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const options = { jobUUID: badUUID, runnerToken, expectedStatus: 404 };
                    yield testEndpoints(Object.assign(Object.assign({}, options), { jobToken }));
                    yield fetchVideoInputFiles(Object.assign(Object.assign({}, options), { videoUUID, jobToken }));
                    yield fetchStudioFiles(Object.assign(Object.assign({}, options), { jobToken: studioAcceptedJob.jobToken, videoUUID, studioFile }));
                });
            });
            it('Should fail with an invalid runner token', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const options = { runnerToken: '', expectedStatus: 400 };
                    yield testEndpoints(Object.assign(Object.assign({}, options), { jobUUID, jobToken }));
                    yield fetchVideoInputFiles(Object.assign(Object.assign({}, options), { jobUUID, videoUUID, jobToken }));
                    yield fetchStudioFiles(Object.assign(Object.assign({}, options), { jobToken: studioAcceptedJob.jobToken, jobUUID: studioAcceptedJob.uuid, videoUUID: videoStudioUUID, studioFile }));
                });
            });
            it('Should fail with an unknown runner token', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const options = { runnerToken: badUUID, expectedStatus: 404 };
                    yield testEndpoints(Object.assign(Object.assign({}, options), { jobUUID, jobToken }));
                    yield fetchVideoInputFiles(Object.assign(Object.assign({}, options), { jobUUID, videoUUID, jobToken }));
                    yield fetchStudioFiles(Object.assign(Object.assign({}, options), { jobToken: studioAcceptedJob.jobToken, jobUUID: studioAcceptedJob.uuid, videoUUID: videoStudioUUID, studioFile }));
                });
            });
            it('Should fail with an invalid job token job uuid', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const options = { runnerToken, jobToken: '', expectedStatus: 400 };
                    yield testEndpoints(Object.assign(Object.assign({}, options), { jobUUID }));
                    yield fetchVideoInputFiles(Object.assign(Object.assign({}, options), { jobUUID, videoUUID }));
                    yield fetchStudioFiles(Object.assign(Object.assign({}, options), { jobUUID: studioAcceptedJob.uuid, videoUUID: videoStudioUUID, studioFile }));
                });
            });
            it('Should fail with an unknown job token job uuid', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const options = { runnerToken, jobToken: badUUID, expectedStatus: 404 };
                    yield testEndpoints(Object.assign(Object.assign({}, options), { jobUUID }));
                    yield fetchVideoInputFiles(Object.assign(Object.assign({}, options), { jobUUID, videoUUID }));
                    yield fetchStudioFiles(Object.assign(Object.assign({}, options), { jobUUID: studioAcceptedJob.uuid, videoUUID: videoStudioUUID, studioFile }));
                });
            });
            it('Should fail with a runner token not associated to this job', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const options = { runnerToken: runnerToken2, expectedStatus: 404 };
                    yield testEndpoints(Object.assign(Object.assign({}, options), { jobUUID, jobToken }));
                    yield fetchVideoInputFiles(Object.assign(Object.assign({}, options), { jobUUID, videoUUID, jobToken }));
                    yield fetchStudioFiles(Object.assign(Object.assign({}, options), { jobToken: studioAcceptedJob.jobToken, jobUUID: studioAcceptedJob.uuid, videoUUID: videoStudioUUID, studioFile }));
                });
            });
            it('Should fail with a job uuid not associated to the job token', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    {
                        const options = { jobUUID: jobUUID2, runnerToken, expectedStatus: 404 };
                        yield testEndpoints(Object.assign(Object.assign({}, options), { jobToken }));
                        yield fetchVideoInputFiles(Object.assign(Object.assign({}, options), { jobToken, videoUUID }));
                        yield fetchStudioFiles(Object.assign(Object.assign({}, options), { jobToken: studioAcceptedJob.jobToken, videoUUID: videoStudioUUID, studioFile }));
                    }
                    {
                        const options = { runnerToken, jobToken: jobToken2, expectedStatus: 404 };
                        yield testEndpoints(Object.assign(Object.assign({}, options), { jobUUID }));
                        yield fetchVideoInputFiles(Object.assign(Object.assign({}, options), { jobUUID, videoUUID }));
                        yield fetchStudioFiles(Object.assign(Object.assign({}, options), { jobUUID: studioAcceptedJob.uuid, videoUUID: videoStudioUUID, studioFile }));
                    }
                });
            });
        });
        describe('Unregister', function () {
            it('Should fail without a runner token', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runners.unregister({ runnerToken: null, expectedStatus: 400 });
                });
            });
            it('Should fail with a bad a runner token', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runners.unregister({ runnerToken: '', expectedStatus: 400 });
                });
            });
            it('Should fail with an unknown runner token', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runners.unregister({ runnerToken: badUUID, expectedStatus: 404 });
                });
            });
        });
        describe('Request', function () {
            it('Should fail without a runner token', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.request({ runnerToken: null, expectedStatus: 400 });
                });
            });
            it('Should fail with a bad a runner token', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.request({ runnerToken: '', expectedStatus: 400 });
                });
            });
            it('Should fail with an unknown runner token', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.request({ runnerToken: badUUID, expectedStatus: 404 });
                });
            });
        });
        describe('Accept', function () {
            it('Should fail with a bad a job uuid', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.accept({ jobUUID: '', runnerToken, expectedStatus: 400 });
                });
            });
            it('Should fail with an unknown job uuid', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.accept({ jobUUID: badUUID, runnerToken, expectedStatus: 404 });
                });
            });
            it('Should fail with a job not in pending state', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.accept({ jobUUID: completedJobUUID, runnerToken, expectedStatus: 400 });
                    yield server.runnerJobs.accept({ jobUUID: cancelledJobUUID, runnerToken, expectedStatus: 400 });
                });
            });
            it('Should fail without a runner token', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.accept({ jobUUID: pendingUUID, runnerToken: null, expectedStatus: 400 });
                });
            });
            it('Should fail with a bad a runner token', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.accept({ jobUUID: pendingUUID, runnerToken: '', expectedStatus: 400 });
                });
            });
            it('Should fail with an unknown runner token', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.accept({ jobUUID: pendingUUID, runnerToken: badUUID, expectedStatus: 404 });
                });
            });
        });
        describe('Abort', function () {
            it('Should fail without a reason', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.abort({ jobUUID, jobToken, runnerToken, reason: null, expectedStatus: 400 });
                });
            });
            it('Should fail with a bad reason', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const reason = 'reason'.repeat(5000);
                    yield server.runnerJobs.abort({ jobUUID, jobToken, runnerToken, reason, expectedStatus: 400 });
                });
            });
            it('Should fail with a job not in processing state', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.abort({
                        jobUUID: completedJobUUID,
                        jobToken: completedJobToken,
                        runnerToken,
                        reason: 'reason',
                        expectedStatus: 400
                    });
                });
            });
        });
        describe('Update', function () {
            describe('Common', function () {
                it('Should fail with an invalid progress', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield server.runnerJobs.update({ jobUUID, jobToken, runnerToken, progress: 101, expectedStatus: 400 });
                    });
                });
                it('Should fail with a job not in processing state', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield server.runnerJobs.update({
                            jobUUID: completedJobUUID,
                            jobToken: completedJobToken,
                            runnerToken,
                            expectedStatus: 400
                        });
                    });
                });
            });
            describe('Live RTMP to HLS', function () {
                const base = {
                    masterPlaylistFile: 'live/master.m3u8',
                    resolutionPlaylistFilename: '0.m3u8',
                    resolutionPlaylistFile: 'live/1.m3u8',
                    type: 'add-chunk',
                    videoChunkFile: 'live/1-000069.ts',
                    videoChunkFilename: '1-000068.ts'
                };
                function testUpdate(payload) {
                    return server.runnerJobs.update({
                        jobUUID: liveAcceptedJob.uuid,
                        jobToken: liveAcceptedJob.jobToken,
                        payload,
                        runnerToken,
                        expectedStatus: 400
                    });
                }
                it('Should fail with an invalid resolutionPlaylistFilename', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield testUpdate(Object.assign(Object.assign({}, base), { resolutionPlaylistFilename: undefined }));
                        yield testUpdate(Object.assign(Object.assign({}, base), { resolutionPlaylistFilename: 'coucou/hello' }));
                        yield testUpdate(Object.assign(Object.assign({}, base), { resolutionPlaylistFilename: 'hello' }));
                    });
                });
                it('Should fail with an invalid videoChunkFilename', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield testUpdate(Object.assign(Object.assign({}, base), { resolutionPlaylistFilename: undefined }));
                        yield testUpdate(Object.assign(Object.assign({}, base), { resolutionPlaylistFilename: 'coucou/hello' }));
                        yield testUpdate(Object.assign(Object.assign({}, base), { resolutionPlaylistFilename: 'hello' }));
                    });
                });
                it('Should fail with an invalid type', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield testUpdate(Object.assign(Object.assign({}, base), { type: undefined }));
                        yield testUpdate(Object.assign(Object.assign({}, base), { type: 'toto' }));
                    });
                });
            });
        });
        describe('Error', function () {
            it('Should fail with a missing error message', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.error({ jobUUID, jobToken, runnerToken, message: null, expectedStatus: 400 });
                });
            });
            it('Should fail with an invalid error messgae', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const message = 'a'.repeat(6000);
                    yield server.runnerJobs.error({ jobUUID, jobToken, runnerToken, message, expectedStatus: 400 });
                });
            });
            it('Should fail with a job not in processing state', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.error({
                        jobUUID: completedJobUUID,
                        jobToken: completedJobToken,
                        message: 'my message',
                        runnerToken,
                        expectedStatus: 400
                    });
                });
            });
        });
        describe('Success', function () {
            let vodJobUUID;
            let vodJobToken;
            describe('Common', function () {
                it('Should fail with a job not in processing state', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield server.runnerJobs.success({
                            jobUUID: completedJobUUID,
                            jobToken: completedJobToken,
                            payload: { videoFile: 'video_short.mp4' },
                            runnerToken,
                            expectedStatus: 400
                        });
                    });
                });
            });
            describe('VOD', function () {
                it('Should fail with an invalid vod web video payload', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        const { job } = yield server.runnerJobs.autoAccept({ runnerToken, type: 'vod-web-video-transcoding' });
                        yield server.runnerJobs.success({
                            jobUUID: job.uuid,
                            jobToken: job.jobToken,
                            payload: { hello: 'video_short.mp4' },
                            runnerToken,
                            expectedStatus: 400
                        });
                        vodJobUUID = job.uuid;
                        vodJobToken = job.jobToken;
                    });
                });
                it('Should fail with an invalid vod hls payload', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        const payload = { videoFile: 'video_short.mp4' };
                        yield server.runnerJobs.success({ runnerToken, jobUUID: vodJobUUID, jobToken: vodJobToken, payload });
                        yield (0, server_commands_1.waitJobs)([server]);
                        const { job } = yield server.runnerJobs.autoAccept({ runnerToken, type: 'vod-hls-transcoding' });
                        yield server.runnerJobs.success({
                            jobUUID: job.uuid,
                            jobToken: job.jobToken,
                            payload: { videoFile: 'video_short.mp4' },
                            runnerToken,
                            expectedStatus: 400
                        });
                    });
                });
                it('Should fail with an invalid vod audio merge payload', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        const attributes = { name: 'audio_with_preview', previewfile: 'preview.jpg', fixture: 'sample.ogg' };
                        yield server.videos.upload({ attributes, mode: 'legacy' });
                        yield (0, server_commands_1.waitJobs)([server]);
                        const { job } = yield server.runnerJobs.autoAccept({ runnerToken, type: 'vod-audio-merge-transcoding' });
                        yield server.runnerJobs.success({
                            jobUUID: job.uuid,
                            jobToken: job.jobToken,
                            payload: { hello: 'video_short.mp4' },
                            runnerToken,
                            expectedStatus: 400
                        });
                    });
                });
            });
            describe('Video studio', function () {
                it('Should fail with an invalid video studio transcoding payload', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield server.runnerJobs.success({
                            jobUUID: studioAcceptedJob.uuid,
                            jobToken: studioAcceptedJob.jobToken,
                            payload: { hello: 'video_short.mp4' },
                            runnerToken,
                            expectedStatus: 400
                        });
                    });
                });
            });
        });
        describe('Job files', function () {
            describe('Check video param for common job file routes', function () {
                function fetchFiles(options) {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield fetchVideoInputFiles(Object.assign(Object.assign({ videoUUID }, options), { jobToken, jobUUID, runnerToken }));
                        yield fetchStudioFiles(Object.assign(Object.assign({ videoUUID: videoStudioUUID }, options), { jobToken: studioAcceptedJob.jobToken, jobUUID: studioAcceptedJob.uuid, runnerToken,
                            studioFile }));
                    });
                }
                it('Should fail with an invalid video id', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield fetchFiles({
                            videoUUID: 'a',
                            expectedStatus: 400
                        });
                    });
                });
                it('Should fail with an unknown video id', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        const videoUUID = '910ec12a-d9e6-458b-a274-0abb655f9464';
                        yield fetchFiles({
                            videoUUID,
                            expectedStatus: 404
                        });
                    });
                });
                it('Should fail with a video id not associated to this job', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield fetchFiles({
                            videoUUID: videoUUID2,
                            expectedStatus: 403
                        });
                    });
                });
                it('Should succeed with the correct params', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield fetchFiles({ expectedStatus: 200 });
                    });
                });
            });
            describe('Video studio tasks file routes', function () {
                it('Should fail with an invalid studio filename', function () {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        yield fetchStudioFiles({
                            videoUUID: videoStudioUUID,
                            jobUUID: studioAcceptedJob.uuid,
                            runnerToken,
                            jobToken: studioAcceptedJob.jobToken,
                            studioFile: 'toto',
                            expectedStatus: 400
                        });
                    });
                });
            });
        });
    });
    after(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield (0, server_commands_1.cleanupTests)([server]);
        });
    });
});
//# sourceMappingURL=runners.js.map