"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chai_1 = require("chai");
const core_utils_1 = require("../../../../shared/core-utils");
const models_1 = require("../../../../shared/models");
const server_commands_1 = require("../../../../shared/server-commands");
describe('Test runner common actions', function () {
    let server;
    let registrationToken;
    let runnerToken;
    let jobMaxPriority;
    before(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(120000);
            server = yield (0, server_commands_1.createSingleServer)(1, {
                remote_runners: {
                    stalled_jobs: {
                        vod: '5 seconds'
                    }
                }
            });
            yield (0, server_commands_1.setAccessTokensToServers)([server]);
            yield (0, server_commands_1.setDefaultVideoChannel)([server]);
            yield server.config.enableTranscoding(true, true);
            yield server.config.enableRemoteTranscoding();
        });
    });
    describe('Managing runner registration tokens', function () {
        let base;
        let registrationTokenToDelete;
        it('Should have a default registration token', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { total, data } = yield server.runnerRegistrationTokens.list();
                (0, chai_1.expect)(total).to.equal(1);
                (0, chai_1.expect)(data).to.have.lengthOf(1);
                const token = data[0];
                (0, chai_1.expect)(token.id).to.exist;
                (0, chai_1.expect)(token.createdAt).to.exist;
                (0, chai_1.expect)(token.updatedAt).to.exist;
                (0, chai_1.expect)(token.registeredRunnersCount).to.equal(0);
                (0, chai_1.expect)(token.registrationToken).to.exist;
            });
        });
        it('Should create other registration tokens', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.runnerRegistrationTokens.generate();
                yield server.runnerRegistrationTokens.generate();
                const { total, data } = yield server.runnerRegistrationTokens.list();
                (0, chai_1.expect)(total).to.equal(3);
                (0, chai_1.expect)(data).to.have.lengthOf(3);
            });
        });
        it('Should list registration tokens', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                {
                    const { total, data } = yield server.runnerRegistrationTokens.list({ sort: 'createdAt' });
                    (0, chai_1.expect)(total).to.equal(3);
                    (0, chai_1.expect)(data).to.have.lengthOf(3);
                    (0, chai_1.expect)(new Date(data[0].createdAt)).to.be.below(new Date(data[1].createdAt));
                    (0, chai_1.expect)(new Date(data[1].createdAt)).to.be.below(new Date(data[2].createdAt));
                    base = data;
                    registrationTokenToDelete = data[0];
                    registrationToken = data[1].registrationToken;
                }
                {
                    const { total, data } = yield server.runnerRegistrationTokens.list({ sort: '-createdAt', start: 2, count: 1 });
                    (0, chai_1.expect)(total).to.equal(3);
                    (0, chai_1.expect)(data).to.have.lengthOf(1);
                    (0, chai_1.expect)(data[0].registrationToken).to.equal(base[0].registrationToken);
                }
            });
        });
        it('Should have appropriate registeredRunnersCount for registration tokens', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.runners.register({ name: 'to delete 1', registrationToken: registrationTokenToDelete.registrationToken });
                yield server.runners.register({ name: 'to delete 2', registrationToken: registrationTokenToDelete.registrationToken });
                const { data } = yield server.runnerRegistrationTokens.list();
                for (const d of data) {
                    if (d.registrationToken === registrationTokenToDelete.registrationToken) {
                        (0, chai_1.expect)(d.registeredRunnersCount).to.equal(2);
                    }
                    else {
                        (0, chai_1.expect)(d.registeredRunnersCount).to.equal(0);
                    }
                }
                const { data: runners } = yield server.runners.list();
                (0, chai_1.expect)(runners).to.have.lengthOf(2);
            });
        });
        it('Should delete a registration token', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.runnerRegistrationTokens.delete({ id: registrationTokenToDelete.id });
                const { total, data } = yield server.runnerRegistrationTokens.list({ sort: 'createdAt' });
                (0, chai_1.expect)(total).to.equal(2);
                (0, chai_1.expect)(data).to.have.lengthOf(2);
                for (const d of data) {
                    (0, chai_1.expect)(d.registeredRunnersCount).to.equal(0);
                    (0, chai_1.expect)(d.registrationToken).to.not.equal(registrationTokenToDelete.registrationToken);
                }
            });
        });
        it('Should have removed runners of this registration token', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { data: runners } = yield server.runners.list();
                (0, chai_1.expect)(runners).to.have.lengthOf(0);
            });
        });
    });
    describe('Managing runners', function () {
        let toDelete;
        it('Should not have runners available', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { total, data } = yield server.runners.list();
                (0, chai_1.expect)(data).to.have.lengthOf(0);
                (0, chai_1.expect)(total).to.equal(0);
            });
        });
        it('Should register runners', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const now = new Date();
                const result = yield server.runners.register({
                    name: 'runner 1',
                    description: 'my super runner 1',
                    registrationToken
                });
                (0, chai_1.expect)(result.runnerToken).to.exist;
                runnerToken = result.runnerToken;
                yield server.runners.register({
                    name: 'runner 2',
                    registrationToken
                });
                const { total, data } = yield server.runners.list({ sort: 'createdAt' });
                (0, chai_1.expect)(total).to.equal(2);
                (0, chai_1.expect)(data).to.have.lengthOf(2);
                for (const d of data) {
                    (0, chai_1.expect)(d.id).to.exist;
                    (0, chai_1.expect)(d.createdAt).to.exist;
                    (0, chai_1.expect)(d.updatedAt).to.exist;
                    (0, chai_1.expect)(new Date(d.createdAt)).to.be.above(now);
                    (0, chai_1.expect)(new Date(d.updatedAt)).to.be.above(now);
                    (0, chai_1.expect)(new Date(d.lastContact)).to.be.above(now);
                    (0, chai_1.expect)(d.ip).to.exist;
                }
                (0, chai_1.expect)(data[0].name).to.equal('runner 1');
                (0, chai_1.expect)(data[0].description).to.equal('my super runner 1');
                (0, chai_1.expect)(data[1].name).to.equal('runner 2');
                (0, chai_1.expect)(data[1].description).to.be.null;
                toDelete = data[1];
            });
        });
        it('Should list runners', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { total, data } = yield server.runners.list({ sort: '-createdAt', start: 1, count: 1 });
                (0, chai_1.expect)(total).to.equal(2);
                (0, chai_1.expect)(data).to.have.lengthOf(1);
                (0, chai_1.expect)(data[0].name).to.equal('runner 1');
            });
        });
        it('Should delete a runner', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.runners.delete({ id: toDelete.id });
                const { total, data } = yield server.runners.list();
                (0, chai_1.expect)(total).to.equal(1);
                (0, chai_1.expect)(data).to.have.lengthOf(1);
                (0, chai_1.expect)(data[0].name).to.equal('runner 1');
            });
        });
        it('Should unregister a runner', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const registered = yield server.runners.autoRegisterRunner();
                {
                    const { total, data } = yield server.runners.list();
                    (0, chai_1.expect)(total).to.equal(2);
                    (0, chai_1.expect)(data).to.have.lengthOf(2);
                }
                yield server.runners.unregister({ runnerToken: registered });
                {
                    const { total, data } = yield server.runners.list();
                    (0, chai_1.expect)(total).to.equal(1);
                    (0, chai_1.expect)(data).to.have.lengthOf(1);
                    (0, chai_1.expect)(data[0].name).to.equal('runner 1');
                }
            });
        });
    });
    describe('Managing runner jobs', function () {
        let jobUUID;
        let jobToken;
        let lastRunnerContact;
        let failedJob;
        function checkMainJobState(mainJobState, otherJobStates = [models_1.RunnerJobState.PENDING, models_1.RunnerJobState.WAITING_FOR_PARENT_JOB]) {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { data } = yield server.runnerJobs.list({ count: 10, sort: '-updatedAt' });
                for (const job of data) {
                    if (job.uuid === jobUUID) {
                        (0, chai_1.expect)(job.state.id).to.equal(mainJobState);
                    }
                    else {
                        (0, chai_1.expect)(otherJobStates).to.include(job.state.id);
                    }
                }
            });
        }
        function getMainJob() {
            return server.runnerJobs.getJob({ uuid: jobUUID });
        }
        describe('List jobs', function () {
            it('Should not have jobs', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const { total, data } = yield server.runnerJobs.list();
                    (0, chai_1.expect)(data).to.have.lengthOf(0);
                    (0, chai_1.expect)(total).to.equal(0);
                });
            });
            it('Should upload a video and have available jobs', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.videos.quickUpload({ name: 'to transcode' });
                    yield (0, server_commands_1.waitJobs)([server]);
                    const { total, data } = yield server.runnerJobs.list();
                    (0, chai_1.expect)(data).to.have.lengthOf(10);
                    (0, chai_1.expect)(total).to.equal(10);
                    for (const job of data) {
                        (0, chai_1.expect)(job.startedAt).to.not.exist;
                        (0, chai_1.expect)(job.finishedAt).to.not.exist;
                        (0, chai_1.expect)(job.payload).to.exist;
                        (0, chai_1.expect)(job.privatePayload).to.exist;
                    }
                    const hlsJobs = data.filter(d => d.type === 'vod-hls-transcoding');
                    const webVideoJobs = data.filter(d => d.type === 'vod-web-video-transcoding');
                    (0, chai_1.expect)(hlsJobs).to.have.lengthOf(5);
                    (0, chai_1.expect)(webVideoJobs).to.have.lengthOf(5);
                    const pendingJobs = data.filter(d => d.state.id === models_1.RunnerJobState.PENDING);
                    const waitingJobs = data.filter(d => d.state.id === models_1.RunnerJobState.WAITING_FOR_PARENT_JOB);
                    (0, chai_1.expect)(pendingJobs).to.have.lengthOf(1);
                    (0, chai_1.expect)(waitingJobs).to.have.lengthOf(9);
                });
            });
            it('Should upload another video and list/sort jobs', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.videos.quickUpload({ name: 'to transcode 2' });
                    yield (0, server_commands_1.waitJobs)([server]);
                    {
                        const { total, data } = yield server.runnerJobs.list({ start: 0, count: 30 });
                        (0, chai_1.expect)(data).to.have.lengthOf(20);
                        (0, chai_1.expect)(total).to.equal(20);
                        jobUUID = data[16].uuid;
                    }
                    {
                        const { total, data } = yield server.runnerJobs.list({ start: 3, count: 1, sort: 'createdAt' });
                        (0, chai_1.expect)(total).to.equal(20);
                        (0, chai_1.expect)(data).to.have.lengthOf(1);
                        (0, chai_1.expect)(data[0].uuid).to.equal(jobUUID);
                    }
                    {
                        let previousPriority = Infinity;
                        const { total, data } = yield server.runnerJobs.list({ start: 0, count: 100, sort: '-priority' });
                        (0, chai_1.expect)(total).to.equal(20);
                        for (const job of data) {
                            (0, chai_1.expect)(job.priority).to.be.at.most(previousPriority);
                            previousPriority = job.priority;
                            if (job.state.id === models_1.RunnerJobState.PENDING) {
                                jobMaxPriority = job.uuid;
                            }
                        }
                    }
                });
            });
            it('Should search jobs', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    {
                        const { total, data } = yield server.runnerJobs.list({ search: jobUUID });
                        (0, chai_1.expect)(data).to.have.lengthOf(1);
                        (0, chai_1.expect)(total).to.equal(1);
                        (0, chai_1.expect)(data[0].uuid).to.equal(jobUUID);
                    }
                    {
                        const { total, data } = yield server.runnerJobs.list({ search: 'toto' });
                        (0, chai_1.expect)(data).to.have.lengthOf(0);
                        (0, chai_1.expect)(total).to.equal(0);
                    }
                    {
                        const { total, data } = yield server.runnerJobs.list({ search: 'hls' });
                        (0, chai_1.expect)(data).to.not.have.lengthOf(0);
                        (0, chai_1.expect)(total).to.not.equal(0);
                    }
                });
            });
        });
        describe('Accept/update/abort/process a job', function () {
            it('Should request available jobs', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    lastRunnerContact = new Date();
                    const { availableJobs } = yield server.runnerJobs.request({ runnerToken });
                    (0, chai_1.expect)(availableJobs).to.have.lengthOf(2);
                    for (const job of availableJobs) {
                        (0, chai_1.expect)(job.uuid).to.exist;
                        (0, chai_1.expect)(job.payload.input).to.exist;
                        (0, chai_1.expect)(job.payload.output).to.exist;
                        (0, chai_1.expect)(job.privatePayload).to.not.exist;
                    }
                    const hlsJobs = availableJobs.filter(d => d.type === 'vod-hls-transcoding');
                    const webVideoJobs = availableJobs.filter(d => d.type === 'vod-web-video-transcoding');
                    (0, chai_1.expect)(hlsJobs).to.have.lengthOf(0);
                    (0, chai_1.expect)(webVideoJobs).to.have.lengthOf(2);
                    jobUUID = webVideoJobs[0].uuid;
                });
            });
            it('Should have sorted available jobs by priority', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const { availableJobs } = yield server.runnerJobs.request({ runnerToken });
                    (0, chai_1.expect)(availableJobs[0].uuid).to.equal(jobMaxPriority);
                });
            });
            it('Should have last runner contact updated', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield (0, core_utils_1.wait)(1000);
                    const { data } = yield server.runners.list({ sort: 'createdAt' });
                    (0, chai_1.expect)(new Date(data[0].lastContact)).to.be.above(lastRunnerContact);
                });
            });
            it('Should accept a job', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const startedAt = new Date();
                    const { job } = yield server.runnerJobs.accept({ runnerToken, jobUUID });
                    jobToken = job.jobToken;
                    const checkProcessingJob = (job, fromAccept) => {
                        (0, chai_1.expect)(job.uuid).to.equal(jobUUID);
                        (0, chai_1.expect)(job.type).to.equal('vod-web-video-transcoding');
                        (0, chai_1.expect)(job.state.label).to.equal('Processing');
                        (0, chai_1.expect)(job.state.id).to.equal(models_1.RunnerJobState.PROCESSING);
                        (0, chai_1.expect)(job.runner).to.exist;
                        (0, chai_1.expect)(job.runner.name).to.equal('runner 1');
                        (0, chai_1.expect)(job.runner.description).to.equal('my super runner 1');
                        (0, chai_1.expect)(job.progress).to.be.null;
                        (0, chai_1.expect)(job.startedAt).to.exist;
                        (0, chai_1.expect)(new Date(job.startedAt)).to.be.above(startedAt);
                        (0, chai_1.expect)(job.finishedAt).to.not.exist;
                        (0, chai_1.expect)(job.failures).to.equal(0);
                        (0, chai_1.expect)(job.payload).to.exist;
                        if (fromAccept) {
                            (0, chai_1.expect)(job.jobToken).to.exist;
                            (0, chai_1.expect)(job.privatePayload).to.not.exist;
                        }
                        else {
                            (0, chai_1.expect)(job.jobToken).to.not.exist;
                            (0, chai_1.expect)(job.privatePayload).to.exist;
                        }
                    };
                    checkProcessingJob(job, true);
                    const { data } = yield server.runnerJobs.list({ count: 10, sort: '-updatedAt' });
                    const processingJob = data.find(j => j.uuid === jobUUID);
                    checkProcessingJob(processingJob, false);
                    yield checkMainJobState(models_1.RunnerJobState.PROCESSING);
                });
            });
            it('Should update a job', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.update({ runnerToken, jobUUID, jobToken, progress: 53 });
                    const { data } = yield server.runnerJobs.list({ count: 10, sort: '-updatedAt' });
                    for (const job of data) {
                        if (job.state.id === models_1.RunnerJobState.PROCESSING) {
                            (0, chai_1.expect)(job.progress).to.equal(53);
                        }
                        else {
                            (0, chai_1.expect)(job.progress).to.be.null;
                        }
                    }
                });
            });
            it('Should abort a job', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.abort({ runnerToken, jobUUID, jobToken, reason: 'for tests' });
                    yield checkMainJobState(models_1.RunnerJobState.PENDING);
                    const { data } = yield server.runnerJobs.list({ count: 10, sort: '-updatedAt' });
                    for (const job of data) {
                        (0, chai_1.expect)(job.progress).to.be.null;
                    }
                });
            });
            it('Should accept the same job again and post a success', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const { availableJobs } = yield server.runnerJobs.request({ runnerToken });
                    (0, chai_1.expect)(availableJobs.find(j => j.uuid === jobUUID)).to.exist;
                    const { job } = yield server.runnerJobs.accept({ runnerToken, jobUUID });
                    jobToken = job.jobToken;
                    yield checkMainJobState(models_1.RunnerJobState.PROCESSING);
                    const { data } = yield server.runnerJobs.list({ count: 10, sort: '-updatedAt' });
                    for (const job of data) {
                        (0, chai_1.expect)(job.progress).to.be.null;
                    }
                    const payload = {
                        videoFile: 'video_short.mp4'
                    };
                    yield server.runnerJobs.success({ runnerToken, jobUUID, jobToken, payload });
                });
            });
            it('Should not have available jobs anymore', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield checkMainJobState(models_1.RunnerJobState.COMPLETED);
                    const job = yield getMainJob();
                    (0, chai_1.expect)(job.finishedAt).to.exist;
                    const { availableJobs } = yield server.runnerJobs.request({ runnerToken });
                    (0, chai_1.expect)(availableJobs.find(j => j.uuid === jobUUID)).to.not.exist;
                });
            });
        });
        describe('Error job', function () {
            it('Should accept another job and post an error', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.runnerJobs.cancelAllJobs();
                    yield server.videos.quickUpload({ name: 'video' });
                    yield (0, server_commands_1.waitJobs)([server]);
                    const { availableJobs } = yield server.runnerJobs.request({ runnerToken });
                    jobUUID = availableJobs[0].uuid;
                    const { job } = yield server.runnerJobs.accept({ runnerToken, jobUUID });
                    jobToken = job.jobToken;
                    yield server.runnerJobs.error({ runnerToken, jobUUID, jobToken, message: 'Error' });
                });
            });
            it('Should have job failures increased', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const job = yield getMainJob();
                    (0, chai_1.expect)(job.state.id).to.equal(models_1.RunnerJobState.PENDING);
                    (0, chai_1.expect)(job.failures).to.equal(1);
                    (0, chai_1.expect)(job.error).to.be.null;
                    (0, chai_1.expect)(job.progress).to.be.null;
                    (0, chai_1.expect)(job.finishedAt).to.not.exist;
                });
            });
            it('Should error a job when job attempts is too big', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    for (let i = 0; i < 4; i++) {
                        const { job } = yield server.runnerJobs.accept({ runnerToken, jobUUID });
                        jobToken = job.jobToken;
                        yield server.runnerJobs.error({ runnerToken, jobUUID, jobToken, message: 'Error ' + i });
                    }
                    const job = yield getMainJob();
                    (0, chai_1.expect)(job.failures).to.equal(5);
                    (0, chai_1.expect)(job.state.id).to.equal(models_1.RunnerJobState.ERRORED);
                    (0, chai_1.expect)(job.state.label).to.equal('Errored');
                    (0, chai_1.expect)(job.error).to.equal('Error 3');
                    (0, chai_1.expect)(job.progress).to.be.null;
                    (0, chai_1.expect)(job.finishedAt).to.exist;
                    failedJob = job;
                });
            });
            it('Should have failed children jobs too', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const { data } = yield server.runnerJobs.list({ count: 50, sort: '-updatedAt' });
                    const children = data.filter(j => { var _a; return ((_a = j.parent) === null || _a === void 0 ? void 0 : _a.uuid) === failedJob.uuid; });
                    (0, chai_1.expect)(children).to.have.lengthOf(9);
                    for (const child of children) {
                        (0, chai_1.expect)(child.parent.uuid).to.equal(failedJob.uuid);
                        (0, chai_1.expect)(child.parent.type).to.equal(failedJob.type);
                        (0, chai_1.expect)(child.parent.state.id).to.equal(failedJob.state.id);
                        (0, chai_1.expect)(child.parent.state.label).to.equal(failedJob.state.label);
                        (0, chai_1.expect)(child.state.id).to.equal(models_1.RunnerJobState.PARENT_ERRORED);
                        (0, chai_1.expect)(child.state.label).to.equal('Parent job failed');
                    }
                });
            });
        });
        describe('Cancel', function () {
            it('Should cancel a pending job', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.videos.quickUpload({ name: 'video' });
                    yield (0, server_commands_1.waitJobs)([server]);
                    {
                        const { data } = yield server.runnerJobs.list({ count: 10, sort: '-updatedAt' });
                        const pendingJob = data.find(j => j.state.id === models_1.RunnerJobState.PENDING);
                        jobUUID = pendingJob.uuid;
                        yield server.runnerJobs.cancelByAdmin({ jobUUID });
                    }
                    {
                        const job = yield getMainJob();
                        (0, chai_1.expect)(job.state.id).to.equal(models_1.RunnerJobState.CANCELLED);
                        (0, chai_1.expect)(job.state.label).to.equal('Cancelled');
                    }
                    {
                        const { data } = yield server.runnerJobs.list({ count: 10, sort: '-updatedAt' });
                        const children = data.filter(j => { var _a; return ((_a = j.parent) === null || _a === void 0 ? void 0 : _a.uuid) === jobUUID; });
                        (0, chai_1.expect)(children).to.have.lengthOf(9);
                        for (const child of children) {
                            (0, chai_1.expect)(child.state.id).to.equal(models_1.RunnerJobState.PARENT_CANCELLED);
                        }
                    }
                });
            });
            it('Should cancel an already accepted job and skip success/error', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.videos.quickUpload({ name: 'video' });
                    yield (0, server_commands_1.waitJobs)([server]);
                    const { availableJobs } = yield server.runnerJobs.request({ runnerToken });
                    jobUUID = availableJobs[0].uuid;
                    const { job } = yield server.runnerJobs.accept({ runnerToken, jobUUID });
                    jobToken = job.jobToken;
                    yield server.runnerJobs.cancelByAdmin({ jobUUID });
                    yield server.runnerJobs.abort({ runnerToken, jobUUID, jobToken, reason: 'aborted', expectedStatus: 404 });
                });
            });
        });
        describe('Stalled jobs', function () {
            it('Should abort stalled jobs', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    this.timeout(60000);
                    yield server.videos.quickUpload({ name: 'video' });
                    yield server.videos.quickUpload({ name: 'video' });
                    yield (0, server_commands_1.waitJobs)([server]);
                    const { job: job1 } = yield server.runnerJobs.autoAccept({ runnerToken });
                    const { job: stalledJob } = yield server.runnerJobs.autoAccept({ runnerToken });
                    for (let i = 0; i < 6; i++) {
                        yield (0, core_utils_1.wait)(2000);
                        yield server.runnerJobs.update({ runnerToken, jobToken: job1.jobToken, jobUUID: job1.uuid });
                    }
                    const refreshedJob1 = yield server.runnerJobs.getJob({ uuid: job1.uuid });
                    const refreshedStalledJob = yield server.runnerJobs.getJob({ uuid: stalledJob.uuid });
                    (0, chai_1.expect)(refreshedJob1.state.id).to.equal(models_1.RunnerJobState.PROCESSING);
                    (0, chai_1.expect)(refreshedStalledJob.state.id).to.equal(models_1.RunnerJobState.PENDING);
                });
            });
        });
        describe('Rate limit', function () {
            before(function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    this.timeout(60000);
                    yield server.kill();
                    yield server.run({
                        rates_limit: {
                            api: {
                                max: 10
                            }
                        }
                    });
                });
            });
            it('Should rate limit an unknown runner, but not a registered one', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    this.timeout(60000);
                    yield server.videos.quickUpload({ name: 'video' });
                    yield (0, server_commands_1.waitJobs)([server]);
                    const { job } = yield server.runnerJobs.autoAccept({ runnerToken });
                    for (let i = 0; i < 20; i++) {
                        try {
                            yield server.runnerJobs.request({ runnerToken });
                            yield server.runnerJobs.update({ runnerToken, jobToken: job.jobToken, jobUUID: job.uuid });
                        }
                        catch (_a) { }
                    }
                    {
                        yield server.runnerJobs.request({ runnerToken: 'toto', expectedStatus: 429 });
                        yield server.runnerJobs.update({
                            runnerToken: 'toto',
                            jobToken: job.jobToken,
                            jobUUID: job.uuid,
                            expectedStatus: 429
                        });
                    }
                    {
                        yield server.runnerJobs.request({ runnerToken: undefined, expectedStatus: 429 });
                        yield server.runnerJobs.update({
                            runnerToken: undefined,
                            jobToken: job.jobToken,
                            jobUUID: job.uuid,
                            expectedStatus: 429
                        });
                    }
                    {
                        yield server.runnerJobs.request({ runnerToken });
                        yield server.runnerJobs.update({ runnerToken, jobToken: job.jobToken, jobUUID: job.uuid });
                    }
                });
            });
        });
    });
    after(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield (0, server_commands_1.cleanupTests)([server]);
        });
    });
});
//# sourceMappingURL=runner-common.js.map