"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chai_1 = require("chai");
const fs_extra_1 = require("fs-extra");
const core_utils_1 = require("../../../../shared/core-utils");
const server_commands_1 = require("../../../../shared/server-commands");
describe('Test runner live transcoding', function () {
    let server;
    let runnerToken;
    let baseUrl;
    before(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(120000);
            server = yield (0, server_commands_1.createSingleServer)(1);
            yield (0, server_commands_1.setAccessTokensToServers)([server]);
            yield (0, server_commands_1.setDefaultVideoChannel)([server]);
            yield server.config.enableRemoteTranscoding();
            yield server.config.enableTranscoding();
            runnerToken = yield server.runners.autoRegisterRunner();
            baseUrl = server.url + '/static/streaming-playlists/hls';
        });
    });
    describe('Without transcoding enabled', function () {
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.config.enableLive({
                    allowReplay: false,
                    resolutions: 'min',
                    transcoding: false
                });
            });
        });
        it('Should not have available jobs', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(120000);
                const { live, video } = yield server.live.quickCreate({ permanentLive: true, saveReplay: false, privacy: 1 });
                const ffmpegCommand = (0, server_commands_1.sendRTMPStream)({ rtmpBaseUrl: live.rtmpUrl, streamKey: live.streamKey });
                yield server.live.waitUntilPublished({ videoId: video.id });
                yield (0, server_commands_1.waitJobs)([server]);
                const { availableJobs } = yield server.runnerJobs.requestLive({ runnerToken });
                (0, chai_1.expect)(availableJobs).to.have.lengthOf(0);
                yield (0, server_commands_1.stopFfmpeg)(ffmpegCommand);
            });
        });
    });
    describe('With transcoding enabled on classic live', function () {
        let live;
        let video;
        let ffmpegCommand;
        let jobUUID;
        let acceptedJob;
        function testPlaylistFile(fixture, expected) {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const text = yield server.streamingPlaylists.get({ url: `${baseUrl}/${video.uuid}/${fixture}` });
                (0, chai_1.expect)(yield (0, fs_extra_1.readFile)((0, core_utils_1.buildAbsoluteFixturePath)(expected), 'utf-8')).to.equal(text);
            });
        }
        function testTSFile(fixture, expected) {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { body } = yield (0, server_commands_1.makeRawRequest)({ url: `${baseUrl}/${video.uuid}/${fixture}`, expectedStatus: 200 });
                (0, chai_1.expect)(yield (0, fs_extra_1.readFile)((0, core_utils_1.buildAbsoluteFixturePath)(expected))).to.deep.equal(body);
            });
        }
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.config.enableLive({
                    allowReplay: true,
                    resolutions: 'max',
                    transcoding: true
                });
            });
        });
        it('Should publish a a live and have available jobs', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(120000);
                const data = yield server.live.quickCreate({ permanentLive: false, saveReplay: false, privacy: 1 });
                live = data.live;
                video = data.video;
                ffmpegCommand = (0, server_commands_1.sendRTMPStream)({ rtmpBaseUrl: live.rtmpUrl, streamKey: live.streamKey });
                yield (0, server_commands_1.waitJobs)([server]);
                const job = yield server.runnerJobs.requestLiveJob(runnerToken);
                jobUUID = job.uuid;
                (0, chai_1.expect)(job.type).to.equal('live-rtmp-hls-transcoding');
                (0, chai_1.expect)(job.payload.input.rtmpUrl).to.exist;
                (0, chai_1.expect)(job.payload.output.toTranscode).to.have.lengthOf(5);
                for (const { resolution, fps } of job.payload.output.toTranscode) {
                    (0, chai_1.expect)([720, 480, 360, 240, 144]).to.contain(resolution);
                    (0, chai_1.expect)(fps).to.be.above(25);
                    (0, chai_1.expect)(fps).to.be.below(70);
                }
            });
        });
        it('Should update the live with a new chunk', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(120000);
                const { job } = yield server.runnerJobs.accept({ jobUUID, runnerToken });
                acceptedJob = job;
                {
                    const payload = {
                        masterPlaylistFile: 'live/master.m3u8',
                        resolutionPlaylistFile: 'live/0.m3u8',
                        resolutionPlaylistFilename: '0.m3u8',
                        type: 'add-chunk',
                        videoChunkFile: 'live/0-000067.ts',
                        videoChunkFilename: '0-000067.ts'
                    };
                    yield server.runnerJobs.update({ jobUUID, runnerToken, jobToken: job.jobToken, payload, progress: 50 });
                    const updatedJob = yield server.runnerJobs.getJob({ uuid: job.uuid });
                    (0, chai_1.expect)(updatedJob.progress).to.equal(50);
                }
                {
                    const payload = {
                        resolutionPlaylistFile: 'live/1.m3u8',
                        resolutionPlaylistFilename: '1.m3u8',
                        type: 'add-chunk',
                        videoChunkFile: 'live/1-000068.ts',
                        videoChunkFilename: '1-000068.ts'
                    };
                    yield server.runnerJobs.update({ jobUUID, runnerToken, jobToken: job.jobToken, payload });
                }
                yield (0, core_utils_1.wait)(1000);
                yield testPlaylistFile('master.m3u8', 'live/master.m3u8');
                yield testPlaylistFile('0.m3u8', 'live/0.m3u8');
                yield testPlaylistFile('1.m3u8', 'live/1.m3u8');
                yield testTSFile('0-000067.ts', 'live/0-000067.ts');
                yield testTSFile('1-000068.ts', 'live/1-000068.ts');
            });
        });
        it('Should replace existing m3u8 on update', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(120000);
                const payload = {
                    masterPlaylistFile: 'live/1.m3u8',
                    resolutionPlaylistFilename: '0.m3u8',
                    resolutionPlaylistFile: 'live/1.m3u8',
                    type: 'add-chunk',
                    videoChunkFile: 'live/1-000069.ts',
                    videoChunkFilename: '1-000068.ts'
                };
                yield server.runnerJobs.update({ jobUUID, runnerToken, jobToken: acceptedJob.jobToken, payload });
                yield (0, core_utils_1.wait)(1000);
                yield testPlaylistFile('master.m3u8', 'live/1.m3u8');
                yield testPlaylistFile('0.m3u8', 'live/1.m3u8');
                yield testTSFile('1-000068.ts', 'live/1-000069.ts');
            });
        });
        it('Should update the live with removed chunks', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(120000);
                const payload = {
                    resolutionPlaylistFile: 'live/0.m3u8',
                    resolutionPlaylistFilename: '0.m3u8',
                    type: 'remove-chunk',
                    videoChunkFilename: '1-000068.ts'
                };
                yield server.runnerJobs.update({ jobUUID, runnerToken, jobToken: acceptedJob.jobToken, payload });
                yield (0, core_utils_1.wait)(1000);
                yield server.streamingPlaylists.get({ url: `${baseUrl}/${video.uuid}/master.m3u8` });
                yield server.streamingPlaylists.get({ url: `${baseUrl}/${video.uuid}/0.m3u8` });
                yield server.streamingPlaylists.get({ url: `${baseUrl}/${video.uuid}/1.m3u8` });
                yield (0, server_commands_1.makeRawRequest)({ url: `${baseUrl}/${video.uuid}/0-000067.ts`, expectedStatus: 200 });
                yield (0, server_commands_1.makeRawRequest)({ url: `${baseUrl}/${video.uuid}/1-000068.ts`, expectedStatus: 404 });
            });
        });
        it('Should complete the live and save the replay', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(120000);
                for (const segment of ['0-000069.ts', '0-000070.ts']) {
                    const payload = {
                        masterPlaylistFile: 'live/master.m3u8',
                        resolutionPlaylistFilename: '0.m3u8',
                        resolutionPlaylistFile: 'live/0.m3u8',
                        type: 'add-chunk',
                        videoChunkFile: 'live/' + segment,
                        videoChunkFilename: segment
                    };
                    yield server.runnerJobs.update({ jobUUID, runnerToken, jobToken: acceptedJob.jobToken, payload });
                    yield (0, core_utils_1.wait)(1000);
                }
                yield (0, server_commands_1.waitJobs)([server]);
                {
                    const { state } = yield server.videos.get({ id: video.uuid });
                    (0, chai_1.expect)(state.id).to.equal(1);
                }
                yield (0, server_commands_1.stopFfmpeg)(ffmpegCommand);
                yield server.runnerJobs.success({ jobUUID, runnerToken, jobToken: acceptedJob.jobToken, payload: {} });
                yield (0, core_utils_1.wait)(1500);
                yield (0, server_commands_1.waitJobs)([server]);
                {
                    const { state } = yield server.videos.get({ id: video.uuid });
                    (0, chai_1.expect)(state.id).to.equal(5);
                    const session = yield server.live.findLatestSession({ videoId: video.uuid });
                    (0, chai_1.expect)(session.error).to.be.null;
                }
            });
        });
    });
    describe('With transcoding enabled on cancelled/aborted/errored live', function () {
        let live;
        let video;
        let ffmpegCommand;
        function prepare() {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                ffmpegCommand = (0, server_commands_1.sendRTMPStream)({ rtmpBaseUrl: live.rtmpUrl, streamKey: live.streamKey });
                yield server.runnerJobs.requestLiveJob(runnerToken);
                const { job } = yield server.runnerJobs.autoAccept({ runnerToken, type: 'live-rtmp-hls-transcoding' });
                return job;
            });
        }
        function checkSessionError(error) {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield (0, core_utils_1.wait)(1500);
                yield (0, server_commands_1.waitJobs)([server]);
                const session = yield server.live.findLatestSession({ videoId: video.uuid });
                (0, chai_1.expect)(session.error).to.equal(error);
            });
        }
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.config.enableLive({
                    allowReplay: true,
                    resolutions: 'max',
                    transcoding: true
                });
                const data = yield server.live.quickCreate({ permanentLive: true, saveReplay: false, privacy: 1 });
                live = data.live;
                video = data.video;
            });
        });
        it('Should abort a running live', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(120000);
                const job = yield prepare();
                yield Promise.all([
                    server.runnerJobs.abort({ jobUUID: job.uuid, runnerToken, jobToken: job.jobToken, reason: 'abort' }),
                    (0, server_commands_1.testFfmpegStreamError)(ffmpegCommand, true)
                ]);
                yield checkSessionError(6);
            });
        });
        it('Should cancel a running live', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(120000);
                const job = yield prepare();
                yield Promise.all([
                    server.runnerJobs.cancelByAdmin({ jobUUID: job.uuid }),
                    (0, server_commands_1.testFfmpegStreamError)(ffmpegCommand, true)
                ]);
                yield checkSessionError(7);
            });
        });
        it('Should error a running live', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(120000);
                const job = yield prepare();
                yield Promise.all([
                    server.runnerJobs.error({ jobUUID: job.uuid, runnerToken, jobToken: job.jobToken, message: 'error' }),
                    (0, server_commands_1.testFfmpegStreamError)(ffmpegCommand, true)
                ]);
                yield checkSessionError(6);
            });
        });
    });
    after(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield (0, server_commands_1.cleanupTests)([server]);
        });
    });
});
//# sourceMappingURL=runner-live-transcoding.js.map