"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chai_1 = require("chai");
const core_utils_1 = require("../../../../shared/core-utils");
const server_commands_1 = require("../../../../shared/server-commands");
describe('Test runner socket', function () {
    let server;
    let runnerToken;
    before(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(120000);
            server = yield (0, server_commands_1.createSingleServer)(1);
            yield (0, server_commands_1.setAccessTokensToServers)([server]);
            yield (0, server_commands_1.setDefaultVideoChannel)([server]);
            yield server.config.enableTranscoding(true, true);
            yield server.config.enableRemoteTranscoding();
            runnerToken = yield server.runners.autoRegisterRunner();
        });
    });
    it('Should throw an error without runner token', function (done) {
        const localSocket = server.socketIO.getRunnersSocket({ runnerToken: null });
        localSocket.on('connect_error', err => {
            (0, chai_1.expect)(err.message).to.contain('No runner token provided');
            done();
        });
    });
    it('Should throw an error with a bad runner token', function (done) {
        const localSocket = server.socketIO.getRunnersSocket({ runnerToken: 'ergag' });
        localSocket.on('connect_error', err => {
            (0, chai_1.expect)(err.message).to.contain('Invalid runner token');
            done();
        });
    });
    it('Should not send ping if there is no available jobs', function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let pings = 0;
            const localSocket = server.socketIO.getRunnersSocket({ runnerToken });
            localSocket.on('available-jobs', () => pings++);
            (0, chai_1.expect)(pings).to.equal(0);
        });
    });
    it('Should send a ping on available job', function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let pings = 0;
            const localSocket = server.socketIO.getRunnersSocket({ runnerToken });
            localSocket.on('available-jobs', () => pings++);
            yield server.videos.quickUpload({ name: 'video1' });
            yield (0, core_utils_1.wait)(1000);
            yield (0, server_commands_1.waitJobs)([server]);
            (0, chai_1.expect)(pings).to.equal(1);
            yield server.videos.quickUpload({ name: 'video2' });
            yield (0, core_utils_1.wait)(1000);
            yield (0, server_commands_1.waitJobs)([server]);
            (0, chai_1.expect)(pings).to.equal(2);
            yield server.runnerJobs.cancelAllJobs();
        });
    });
    it('Should send a ping when a child is ready', function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let pings = 0;
            const localSocket = server.socketIO.getRunnersSocket({ runnerToken });
            localSocket.on('available-jobs', () => pings++);
            yield server.videos.quickUpload({ name: 'video3' });
            yield (0, core_utils_1.wait)(1000);
            yield (0, server_commands_1.waitJobs)([server]);
            (0, chai_1.expect)(pings).to.equal(1);
            yield server.runnerJobs.autoProcessWebVideoJob(runnerToken);
            yield (0, core_utils_1.wait)(1000);
            yield (0, server_commands_1.waitJobs)([server]);
            (0, chai_1.expect)(pings).to.equal(2);
        });
    });
    it('Should not send a ping if the ended job does not have a child', function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let pings = 0;
            const localSocket = server.socketIO.getRunnersSocket({ runnerToken });
            localSocket.on('available-jobs', () => pings++);
            const { availableJobs } = yield server.runnerJobs.request({ runnerToken });
            const job = availableJobs.find(j => j.type === 'vod-web-video-transcoding');
            yield server.runnerJobs.autoProcessWebVideoJob(runnerToken, job.uuid);
            yield (0, core_utils_1.wait)(1000);
            yield (0, server_commands_1.waitJobs)([server]);
            (0, chai_1.expect)(pings).to.equal(0);
        });
    });
    after(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield (0, server_commands_1.cleanupTests)([server]);
        });
    });
});
//# sourceMappingURL=runner-socket.js.map