"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chai_1 = require("chai");
const fs_extra_1 = require("fs-extra");
const shared_1 = require("../../shared");
const core_utils_1 = require("../../../../shared/core-utils");
const server_commands_1 = require("../../../../shared/server-commands");
describe('Test runner video studio transcoding', function () {
    let servers = [];
    let runnerToken;
    let videoUUID;
    let jobUUID;
    function renewStudio(tasks = server_commands_1.VideoStudioCommand.getComplexTask()) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { uuid } = yield servers[0].videos.quickUpload({ name: 'video' });
            videoUUID = uuid;
            yield (0, server_commands_1.waitJobs)(servers);
            yield servers[0].videoStudio.createEditionTasks({ videoId: uuid, tasks });
            yield (0, server_commands_1.waitJobs)(servers);
            const { availableJobs } = yield servers[0].runnerJobs.request({ runnerToken });
            (0, chai_1.expect)(availableJobs).to.have.lengthOf(1);
            jobUUID = availableJobs[0].uuid;
        });
    }
    before(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(120000);
            servers = yield (0, server_commands_1.createMultipleServers)(2);
            yield (0, server_commands_1.setAccessTokensToServers)(servers);
            yield (0, server_commands_1.setDefaultVideoChannel)(servers);
            yield (0, server_commands_1.doubleFollow)(servers[0], servers[1]);
            yield servers[0].config.enableTranscoding(true, true);
            yield servers[0].config.enableStudio();
            yield servers[0].config.enableRemoteStudio();
            runnerToken = yield servers[0].runners.autoRegisterRunner();
        });
    });
    it('Should error a studio transcoding job', function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(60000);
            yield renewStudio();
            for (let i = 0; i < 5; i++) {
                const { job } = yield servers[0].runnerJobs.accept({ runnerToken, jobUUID });
                const jobToken = job.jobToken;
                yield servers[0].runnerJobs.error({ runnerToken, jobUUID, jobToken, message: 'Error' });
            }
            const video = yield servers[0].videos.get({ id: videoUUID });
            (0, chai_1.expect)(video.state.id).to.equal(1);
            yield (0, shared_1.checkPersistentTmpIsEmpty)(servers[0]);
        });
    });
    it('Should cancel a transcoding job', function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(60000);
            yield renewStudio();
            yield servers[0].runnerJobs.cancelByAdmin({ jobUUID });
            const video = yield servers[0].videos.get({ id: videoUUID });
            (0, chai_1.expect)(video.state.id).to.equal(1);
            yield (0, shared_1.checkPersistentTmpIsEmpty)(servers[0]);
        });
    });
    it('Should execute a remote studio job', function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(240000);
            const tasks = [
                {
                    name: 'add-outro',
                    options: {
                        file: 'video_short.webm'
                    }
                },
                {
                    name: 'add-watermark',
                    options: {
                        file: 'thumbnail.png'
                    }
                },
                {
                    name: 'add-intro',
                    options: {
                        file: 'video_very_short_240p.mp4'
                    }
                }
            ];
            yield renewStudio(tasks);
            for (const server of servers) {
                yield (0, shared_1.checkVideoDuration)(server, videoUUID, 5);
            }
            const { job } = yield servers[0].runnerJobs.accept({ runnerToken, jobUUID });
            const jobToken = job.jobToken;
            (0, chai_1.expect)(job.type === 'video-studio-transcoding');
            (0, chai_1.expect)(job.payload.input.videoFileUrl).to.exist;
            {
                yield servers[0].runnerJobs.getJobFile({ url: job.payload.input.videoFileUrl, jobToken, runnerToken });
            }
            for (let i = 0; i < tasks.length; i++) {
                const task = tasks[i];
                const payloadTask = job.payload.tasks[i];
                (0, chai_1.expect)(payloadTask.name).to.equal(task.name);
                const inputFile = yield (0, fs_extra_1.readFile)((0, core_utils_1.buildAbsoluteFixturePath)(task.options.file));
                const { body } = yield servers[0].runnerJobs.getJobFile({
                    url: payloadTask.options.file,
                    jobToken,
                    runnerToken
                });
                (0, chai_1.expect)(body).to.deep.equal(inputFile);
            }
            const payload = { videoFile: 'video_very_short_240p.mp4' };
            yield servers[0].runnerJobs.success({ runnerToken, jobUUID, jobToken, payload });
            yield (0, server_commands_1.waitJobs)(servers);
            for (const server of servers) {
                yield (0, shared_1.checkVideoDuration)(server, videoUUID, 2);
            }
            yield (0, shared_1.checkPersistentTmpIsEmpty)(servers[0]);
        });
    });
    after(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield (0, server_commands_1.cleanupTests)(servers);
        });
    });
});
//# sourceMappingURL=runner-studio-transcoding.js.map