"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chai_1 = require("chai");
const fs_extra_1 = require("fs-extra");
const shared_1 = require("../../shared");
const core_utils_1 = require("../../../../shared/core-utils");
const server_commands_1 = require("../../../../shared/server-commands");
function processAllJobs(server, runnerToken) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        do {
            const { availableJobs } = yield server.runnerJobs.requestVOD({ runnerToken });
            if (availableJobs.length === 0)
                break;
            const { job } = yield server.runnerJobs.accept({ runnerToken, jobUUID: availableJobs[0].uuid });
            const payload = {
                videoFile: `video_short_${job.payload.output.resolution}p.mp4`,
                resolutionPlaylistFile: `video_short_${job.payload.output.resolution}p.m3u8`
            };
            yield server.runnerJobs.success({ runnerToken, jobUUID: job.uuid, jobToken: job.jobToken, payload });
        } while (true);
        yield (0, server_commands_1.waitJobs)([server]);
    });
}
describe('Test runner VOD transcoding', function () {
    let servers = [];
    let runnerToken;
    before(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(120000);
            servers = yield (0, server_commands_1.createMultipleServers)(2);
            yield (0, server_commands_1.setAccessTokensToServers)(servers);
            yield (0, server_commands_1.setDefaultVideoChannel)(servers);
            yield (0, server_commands_1.doubleFollow)(servers[0], servers[1]);
            yield servers[0].config.enableRemoteTranscoding();
            runnerToken = yield servers[0].runners.autoRegisterRunner();
        });
    });
    describe('Without transcoding', function () {
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(60000);
                yield servers[0].config.disableTranscoding();
                yield servers[0].videos.quickUpload({ name: 'video' });
                yield (0, server_commands_1.waitJobs)(servers);
            });
        });
        it('Should not have available jobs', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { availableJobs } = yield servers[0].runnerJobs.requestVOD({ runnerToken });
                (0, chai_1.expect)(availableJobs).to.have.lengthOf(0);
            });
        });
    });
    describe('With classic transcoding enabled', function () {
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(60000);
                yield servers[0].config.enableTranscoding(true, true);
            });
        });
        it('Should error a transcoding job', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(60000);
                yield servers[0].runnerJobs.cancelAllJobs();
                const { uuid } = yield servers[0].videos.quickUpload({ name: 'video' });
                yield (0, server_commands_1.waitJobs)(servers);
                const { availableJobs } = yield servers[0].runnerJobs.request({ runnerToken });
                const jobUUID = availableJobs[0].uuid;
                for (let i = 0; i < 5; i++) {
                    const { job } = yield servers[0].runnerJobs.accept({ runnerToken, jobUUID });
                    const jobToken = job.jobToken;
                    yield servers[0].runnerJobs.error({ runnerToken, jobUUID, jobToken, message: 'Error' });
                }
                const video = yield servers[0].videos.get({ id: uuid });
                (0, chai_1.expect)(video.state.id).to.equal(7);
            });
        });
        it('Should cancel a transcoding job', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield servers[0].runnerJobs.cancelAllJobs();
                const { uuid } = yield servers[0].videos.quickUpload({ name: 'video' });
                yield (0, server_commands_1.waitJobs)(servers);
                const { availableJobs } = yield servers[0].runnerJobs.request({ runnerToken });
                const jobUUID = availableJobs[0].uuid;
                yield servers[0].runnerJobs.cancelByAdmin({ jobUUID });
                const video = yield servers[0].videos.get({ id: uuid });
                (0, chai_1.expect)(video.state.id).to.equal(1);
            });
        });
    });
    describe('Web video transcoding only', function () {
        let videoUUID;
        let jobToken;
        let jobUUID;
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(60000);
                yield servers[0].runnerJobs.cancelAllJobs();
                yield servers[0].config.enableTranscoding(true, false);
                const { uuid } = yield servers[0].videos.quickUpload({ name: 'web video', fixture: 'video_short.webm' });
                videoUUID = uuid;
                yield (0, server_commands_1.waitJobs)(servers);
            });
        });
        it('Should have jobs available for remote runners', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { availableJobs } = yield servers[0].runnerJobs.requestVOD({ runnerToken });
                (0, chai_1.expect)(availableJobs).to.have.lengthOf(1);
                jobUUID = availableJobs[0].uuid;
            });
        });
        it('Should have a valid first transcoding job', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { job } = yield servers[0].runnerJobs.accept({ runnerToken, jobUUID });
                jobToken = job.jobToken;
                (0, chai_1.expect)(job.type === 'vod-web-video-transcoding');
                (0, chai_1.expect)(job.payload.input.videoFileUrl).to.exist;
                (0, chai_1.expect)(job.payload.output.resolution).to.equal(720);
                (0, chai_1.expect)(job.payload.output.fps).to.equal(25);
                const { body } = yield servers[0].runnerJobs.getJobFile({ url: job.payload.input.videoFileUrl, jobToken, runnerToken });
                const inputFile = yield (0, fs_extra_1.readFile)((0, core_utils_1.buildAbsoluteFixturePath)('video_short.webm'));
                (0, chai_1.expect)(body).to.deep.equal(inputFile);
            });
        });
        it('Should transcode the max video resolution and send it back to the server', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(60000);
                const payload = {
                    videoFile: 'video_short.mp4'
                };
                yield servers[0].runnerJobs.success({ runnerToken, jobUUID, jobToken, payload });
                yield (0, server_commands_1.waitJobs)(servers);
            });
        });
        it('Should have the video updated', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                for (const server of servers) {
                    const video = yield server.videos.get({ id: videoUUID });
                    (0, chai_1.expect)(video.files).to.have.lengthOf(1);
                    (0, chai_1.expect)(video.streamingPlaylists).to.have.lengthOf(0);
                    const { body } = yield (0, server_commands_1.makeRawRequest)({ url: video.files[0].fileUrl, expectedStatus: 200 });
                    (0, chai_1.expect)(body).to.deep.equal(yield (0, fs_extra_1.readFile)((0, core_utils_1.buildAbsoluteFixturePath)('video_short.mp4')));
                }
            });
        });
        it('Should have 4 lower resolution to transcode', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { availableJobs } = yield servers[0].runnerJobs.requestVOD({ runnerToken });
                (0, chai_1.expect)(availableJobs).to.have.lengthOf(4);
                for (const resolution of [480, 360, 240, 144]) {
                    const job = availableJobs.find(j => j.payload.output.resolution === resolution);
                    (0, chai_1.expect)(job).to.exist;
                    (0, chai_1.expect)(job.type).to.equal('vod-web-video-transcoding');
                    if (resolution === 240)
                        jobUUID = job.uuid;
                }
            });
        });
        it('Should process one of these transcoding jobs', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { job } = yield servers[0].runnerJobs.accept({ runnerToken, jobUUID });
                jobToken = job.jobToken;
                const { body } = yield servers[0].runnerJobs.getJobFile({ url: job.payload.input.videoFileUrl, jobToken, runnerToken });
                const inputFile = yield (0, fs_extra_1.readFile)((0, core_utils_1.buildAbsoluteFixturePath)('video_short.mp4'));
                (0, chai_1.expect)(body).to.deep.equal(inputFile);
                const payload = { videoFile: `video_short_${job.payload.output.resolution}p.mp4` };
                yield servers[0].runnerJobs.success({ runnerToken, jobUUID, jobToken, payload });
            });
        });
        it('Should process all other jobs', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { availableJobs } = yield servers[0].runnerJobs.requestVOD({ runnerToken });
                (0, chai_1.expect)(availableJobs).to.have.lengthOf(3);
                for (const resolution of [480, 360, 144]) {
                    const availableJob = availableJobs.find(j => j.payload.output.resolution === resolution);
                    (0, chai_1.expect)(availableJob).to.exist;
                    jobUUID = availableJob.uuid;
                    const { job } = yield servers[0].runnerJobs.accept({ runnerToken, jobUUID });
                    jobToken = job.jobToken;
                    const { body } = yield servers[0].runnerJobs.getJobFile({ url: job.payload.input.videoFileUrl, jobToken, runnerToken });
                    const inputFile = yield (0, fs_extra_1.readFile)((0, core_utils_1.buildAbsoluteFixturePath)('video_short.mp4'));
                    (0, chai_1.expect)(body).to.deep.equal(inputFile);
                    const payload = { videoFile: `video_short_${resolution}p.mp4` };
                    yield servers[0].runnerJobs.success({ runnerToken, jobUUID, jobToken, payload });
                }
                yield (0, server_commands_1.waitJobs)(servers);
            });
        });
        it('Should have the video updated', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                for (const server of servers) {
                    const video = yield server.videos.get({ id: videoUUID });
                    (0, chai_1.expect)(video.files).to.have.lengthOf(5);
                    (0, chai_1.expect)(video.streamingPlaylists).to.have.lengthOf(0);
                    const { body } = yield (0, server_commands_1.makeRawRequest)({ url: video.files[0].fileUrl, expectedStatus: 200 });
                    (0, chai_1.expect)(body).to.deep.equal(yield (0, fs_extra_1.readFile)((0, core_utils_1.buildAbsoluteFixturePath)('video_short.mp4')));
                    for (const file of video.files) {
                        yield (0, server_commands_1.makeRawRequest)({ url: file.fileUrl, expectedStatus: 200 });
                        yield (0, server_commands_1.makeRawRequest)({ url: file.torrentUrl, expectedStatus: 200 });
                    }
                }
            });
        });
        it('Should not have available jobs anymore', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { availableJobs } = yield servers[0].runnerJobs.requestVOD({ runnerToken });
                (0, chai_1.expect)(availableJobs).to.have.lengthOf(0);
            });
        });
    });
    describe('HLS transcoding only', function () {
        let videoUUID;
        let jobToken;
        let jobUUID;
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(60000);
                yield servers[0].config.enableTranscoding(false, true);
                const { uuid } = yield servers[0].videos.quickUpload({ name: 'hls video', fixture: 'video_short.webm' });
                videoUUID = uuid;
                yield (0, server_commands_1.waitJobs)(servers);
            });
        });
        it('Should run the optimize job', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(60000);
                yield servers[0].runnerJobs.autoProcessWebVideoJob(runnerToken);
            });
        });
        it('Should have 5 HLS resolution to transcode', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { availableJobs } = yield servers[0].runnerJobs.requestVOD({ runnerToken });
                (0, chai_1.expect)(availableJobs).to.have.lengthOf(5);
                for (const resolution of [720, 480, 360, 240, 144]) {
                    const job = availableJobs.find(j => j.payload.output.resolution === resolution);
                    (0, chai_1.expect)(job).to.exist;
                    (0, chai_1.expect)(job.type).to.equal('vod-hls-transcoding');
                    if (resolution === 480)
                        jobUUID = job.uuid;
                }
            });
        });
        it('Should process one of these transcoding jobs', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(60000);
                const { job } = yield servers[0].runnerJobs.accept({ runnerToken, jobUUID });
                jobToken = job.jobToken;
                const { body } = yield servers[0].runnerJobs.getJobFile({ url: job.payload.input.videoFileUrl, jobToken, runnerToken });
                const inputFile = yield (0, fs_extra_1.readFile)((0, core_utils_1.buildAbsoluteFixturePath)('video_short.mp4'));
                (0, chai_1.expect)(body).to.deep.equal(inputFile);
                const payload = {
                    videoFile: 'video_short_480p.mp4',
                    resolutionPlaylistFile: 'video_short_480p.m3u8'
                };
                yield servers[0].runnerJobs.success({ runnerToken, jobUUID, jobToken, payload });
                yield (0, server_commands_1.waitJobs)(servers);
            });
        });
        it('Should have the video updated', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                for (const server of servers) {
                    const video = yield server.videos.get({ id: videoUUID });
                    (0, chai_1.expect)(video.files).to.have.lengthOf(1);
                    (0, chai_1.expect)(video.streamingPlaylists).to.have.lengthOf(1);
                    const hls = video.streamingPlaylists[0];
                    (0, chai_1.expect)(hls.files).to.have.lengthOf(1);
                    yield (0, shared_1.completeCheckHlsPlaylist)({ videoUUID, hlsOnly: false, servers, resolutions: [480] });
                }
            });
        });
        it('Should process all other jobs', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(60000);
                const { availableJobs } = yield servers[0].runnerJobs.requestVOD({ runnerToken });
                (0, chai_1.expect)(availableJobs).to.have.lengthOf(4);
                let maxQualityFile = 'video_short.mp4';
                for (const resolution of [720, 360, 240, 144]) {
                    const availableJob = availableJobs.find(j => j.payload.output.resolution === resolution);
                    (0, chai_1.expect)(availableJob).to.exist;
                    jobUUID = availableJob.uuid;
                    const { job } = yield servers[0].runnerJobs.accept({ runnerToken, jobUUID });
                    jobToken = job.jobToken;
                    const { body } = yield servers[0].runnerJobs.getJobFile({ url: job.payload.input.videoFileUrl, jobToken, runnerToken });
                    const inputFile = yield (0, fs_extra_1.readFile)((0, core_utils_1.buildAbsoluteFixturePath)(maxQualityFile));
                    (0, chai_1.expect)(body).to.deep.equal(inputFile);
                    const payload = {
                        videoFile: `video_short_${resolution}p.mp4`,
                        resolutionPlaylistFile: `video_short_${resolution}p.m3u8`
                    };
                    yield servers[0].runnerJobs.success({ runnerToken, jobUUID, jobToken, payload });
                    if (resolution === 720) {
                        maxQualityFile = 'video_short_720p.mp4';
                    }
                }
                yield (0, server_commands_1.waitJobs)(servers);
            });
        });
        it('Should have the video updated', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                for (const server of servers) {
                    const video = yield server.videos.get({ id: videoUUID });
                    (0, chai_1.expect)(video.files).to.have.lengthOf(0);
                    (0, chai_1.expect)(video.streamingPlaylists).to.have.lengthOf(1);
                    const hls = video.streamingPlaylists[0];
                    (0, chai_1.expect)(hls.files).to.have.lengthOf(5);
                    yield (0, shared_1.completeCheckHlsPlaylist)({ videoUUID, hlsOnly: true, servers, resolutions: [720, 480, 360, 240, 144] });
                }
            });
        });
        it('Should not have available jobs anymore', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { availableJobs } = yield servers[0].runnerJobs.requestVOD({ runnerToken });
                (0, chai_1.expect)(availableJobs).to.have.lengthOf(0);
            });
        });
    });
    describe('Web video and HLS transcoding', function () {
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(60000);
                yield servers[0].config.enableTranscoding(true, true);
                yield servers[0].videos.quickUpload({ name: 'web video and hls video', fixture: 'video_short.webm' });
                yield (0, server_commands_1.waitJobs)(servers);
            });
        });
        it('Should process the first optimize job', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(60000);
                yield servers[0].runnerJobs.autoProcessWebVideoJob(runnerToken);
            });
        });
        it('Should have 9 jobs to process', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { availableJobs } = yield servers[0].runnerJobs.requestVOD({ runnerToken });
                (0, chai_1.expect)(availableJobs).to.have.lengthOf(9);
                const webVideoJobs = availableJobs.filter(j => j.type === 'vod-web-video-transcoding');
                const hlsJobs = availableJobs.filter(j => j.type === 'vod-hls-transcoding');
                (0, chai_1.expect)(webVideoJobs).to.have.lengthOf(4);
                (0, chai_1.expect)(hlsJobs).to.have.lengthOf(5);
            });
        });
        it('Should process all available jobs', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield processAllJobs(servers[0], runnerToken);
            });
        });
    });
    describe('Audio merge transcoding', function () {
        let videoUUID;
        let jobToken;
        let jobUUID;
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(60000);
                yield servers[0].config.enableTranscoding(true, true);
                const attributes = { name: 'audio_with_preview', previewfile: 'preview.jpg', fixture: 'sample.ogg' };
                const { uuid } = yield servers[0].videos.upload({ attributes, mode: 'legacy' });
                videoUUID = uuid;
                yield (0, server_commands_1.waitJobs)(servers);
            });
        });
        it('Should have an audio merge transcoding job', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { availableJobs } = yield servers[0].runnerJobs.requestVOD({ runnerToken });
                (0, chai_1.expect)(availableJobs).to.have.lengthOf(1);
                (0, chai_1.expect)(availableJobs[0].type).to.equal('vod-audio-merge-transcoding');
                jobUUID = availableJobs[0].uuid;
            });
        });
        it('Should have a valid remote audio merge transcoding job', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { job } = yield servers[0].runnerJobs.accept({ runnerToken, jobUUID });
                jobToken = job.jobToken;
                (0, chai_1.expect)(job.type === 'vod-audio-merge-transcoding');
                (0, chai_1.expect)(job.payload.input.audioFileUrl).to.exist;
                (0, chai_1.expect)(job.payload.input.previewFileUrl).to.exist;
                (0, chai_1.expect)(job.payload.output.resolution).to.equal(480);
                {
                    const { body } = yield servers[0].runnerJobs.getJobFile({ url: job.payload.input.audioFileUrl, jobToken, runnerToken });
                    const inputFile = yield (0, fs_extra_1.readFile)((0, core_utils_1.buildAbsoluteFixturePath)('sample.ogg'));
                    (0, chai_1.expect)(body).to.deep.equal(inputFile);
                }
                {
                    const { body } = yield servers[0].runnerJobs.getJobFile({ url: job.payload.input.previewFileUrl, jobToken, runnerToken });
                    const video = yield servers[0].videos.get({ id: videoUUID });
                    const { body: inputFile } = yield (0, server_commands_1.makeGetRequest)({
                        url: servers[0].url,
                        path: video.previewPath,
                        expectedStatus: 200
                    });
                    (0, chai_1.expect)(body).to.deep.equal(inputFile);
                }
            });
        });
        it('Should merge the audio', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(60000);
                const payload = { videoFile: 'video_short_480p.mp4' };
                yield servers[0].runnerJobs.success({ runnerToken, jobUUID, jobToken, payload });
                yield (0, server_commands_1.waitJobs)(servers);
            });
        });
        it('Should have the video updated', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                for (const server of servers) {
                    const video = yield server.videos.get({ id: videoUUID });
                    (0, chai_1.expect)(video.files).to.have.lengthOf(1);
                    (0, chai_1.expect)(video.streamingPlaylists).to.have.lengthOf(0);
                    const { body } = yield (0, server_commands_1.makeRawRequest)({ url: video.files[0].fileUrl, expectedStatus: 200 });
                    (0, chai_1.expect)(body).to.deep.equal(yield (0, fs_extra_1.readFile)((0, core_utils_1.buildAbsoluteFixturePath)('video_short_480p.mp4')));
                }
            });
        });
        it('Should have 7 lower resolutions to transcode', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { availableJobs } = yield servers[0].runnerJobs.requestVOD({ runnerToken });
                (0, chai_1.expect)(availableJobs).to.have.lengthOf(7);
                for (const resolution of [360, 240, 144]) {
                    const jobs = availableJobs.filter(j => j.payload.output.resolution === resolution);
                    (0, chai_1.expect)(jobs).to.have.lengthOf(2);
                }
                jobUUID = availableJobs.find(j => j.payload.output.resolution === 480).uuid;
            });
        });
        it('Should process one other job', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(60000);
                const { job } = yield servers[0].runnerJobs.accept({ runnerToken, jobUUID });
                jobToken = job.jobToken;
                const { body } = yield servers[0].runnerJobs.getJobFile({ url: job.payload.input.videoFileUrl, jobToken, runnerToken });
                const inputFile = yield (0, fs_extra_1.readFile)((0, core_utils_1.buildAbsoluteFixturePath)('video_short_480p.mp4'));
                (0, chai_1.expect)(body).to.deep.equal(inputFile);
                const payload = {
                    videoFile: `video_short_480p.mp4`,
                    resolutionPlaylistFile: `video_short_480p.m3u8`
                };
                yield servers[0].runnerJobs.success({ runnerToken, jobUUID, jobToken, payload });
                yield (0, server_commands_1.waitJobs)(servers);
            });
        });
        it('Should have the video updated', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                for (const server of servers) {
                    const video = yield server.videos.get({ id: videoUUID });
                    (0, chai_1.expect)(video.files).to.have.lengthOf(1);
                    (0, chai_1.expect)(video.streamingPlaylists).to.have.lengthOf(1);
                    const hls = video.streamingPlaylists[0];
                    (0, chai_1.expect)(hls.files).to.have.lengthOf(1);
                    yield (0, shared_1.completeCheckHlsPlaylist)({ videoUUID, hlsOnly: false, servers, resolutions: [480] });
                }
            });
        });
        it('Should process all available jobs', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield processAllJobs(servers[0], runnerToken);
            });
        });
    });
    after(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield (0, server_commands_1.cleanupTests)(servers);
        });
    });
});
//# sourceMappingURL=runner-vod-transcoding.js.map