"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chai_1 = require("chai");
const shared_1 = require("../../shared");
const server_commands_1 = require("../../../../shared/server-commands");
describe('Test config defaults', function () {
    let server;
    let channelId;
    before(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(30000);
            server = yield (0, server_commands_1.createSingleServer)(1);
            yield (0, server_commands_1.setAccessTokensToServers)([server]);
            yield (0, server_commands_1.setDefaultVideoChannel)([server]);
            channelId = server.store.channel.id;
        });
    });
    describe('Default publish values', function () {
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const overrideConfig = {
                    defaults: {
                        publish: {
                            comments_enabled: false,
                            download_enabled: false,
                            privacy: 4,
                            licence: 4
                        }
                    }
                };
                yield server.kill();
                yield server.run(overrideConfig);
            });
        });
        const attributes = {
            name: 'video',
            downloadEnabled: undefined,
            commentsEnabled: undefined,
            licence: undefined,
            privacy: 1
        };
        function checkVideo(video) {
            (0, chai_1.expect)(video.downloadEnabled).to.be.false;
            (0, chai_1.expect)(video.commentsEnabled).to.be.false;
            (0, chai_1.expect)(video.licence.id).to.equal(4);
        }
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.config.disableTranscoding();
                yield server.config.enableImports();
                yield server.config.enableLive({ allowReplay: false, transcoding: false });
            });
        });
        it('Should have the correct server configuration', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const config = yield server.config.getConfig();
                (0, chai_1.expect)(config.defaults.publish.commentsEnabled).to.be.false;
                (0, chai_1.expect)(config.defaults.publish.downloadEnabled).to.be.false;
                (0, chai_1.expect)(config.defaults.publish.licence).to.equal(4);
                (0, chai_1.expect)(config.defaults.publish.privacy).to.equal(4);
            });
        });
        it('Should respect default values when uploading a video', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                for (const mode of ['legacy', 'resumable']) {
                    const { id } = yield server.videos.upload({ attributes, mode });
                    const video = yield server.videos.get({ id });
                    checkVideo(video);
                }
            });
        });
        it('Should respect default values when importing a video using URL', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { video: { id } } = yield server.imports.importVideo({
                    attributes: Object.assign(Object.assign({}, attributes), { channelId, targetUrl: shared_1.FIXTURE_URLS.goodVideo })
                });
                const video = yield server.videos.get({ id });
                checkVideo(video);
            });
        });
        it('Should respect default values when importing a video using magnet URI', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { video: { id } } = yield server.imports.importVideo({
                    attributes: Object.assign(Object.assign({}, attributes), { channelId, magnetUri: shared_1.FIXTURE_URLS.magnet })
                });
                const video = yield server.videos.get({ id });
                checkVideo(video);
            });
        });
        it('Should respect default values when creating a live', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { id } = yield server.live.create({
                    fields: Object.assign(Object.assign({}, attributes), { channelId })
                });
                const video = yield server.videos.get({ id });
                checkVideo(video);
            });
        });
    });
    describe('Default P2P values', function () {
        describe('Webapp default value', function () {
            before(function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const overrideConfig = {
                        defaults: {
                            p2p: {
                                webapp: {
                                    enabled: false
                                }
                            }
                        }
                    };
                    yield server.kill();
                    yield server.run(overrideConfig);
                });
            });
            it('Should have appropriate P2P config', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const config = yield server.config.getConfig();
                    (0, chai_1.expect)(config.defaults.p2p.webapp.enabled).to.be.false;
                    (0, chai_1.expect)(config.defaults.p2p.embed.enabled).to.be.true;
                });
            });
            it('Should create a user with this default setting', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.users.create({ username: 'user_p2p_1' });
                    const userToken = yield server.login.getAccessToken('user_p2p_1');
                    const { p2pEnabled } = yield server.users.getMyInfo({ token: userToken });
                    (0, chai_1.expect)(p2pEnabled).to.be.false;
                });
            });
            it('Should register a user with this default setting', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.registrations.register({ username: 'user_p2p_2' });
                    const userToken = yield server.login.getAccessToken('user_p2p_2');
                    const { p2pEnabled } = yield server.users.getMyInfo({ token: userToken });
                    (0, chai_1.expect)(p2pEnabled).to.be.false;
                });
            });
        });
        describe('Embed default value', function () {
            before(function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const overrideConfig = {
                        defaults: {
                            p2p: {
                                embed: {
                                    enabled: false
                                }
                            }
                        },
                        signup: {
                            limit: 15
                        }
                    };
                    yield server.kill();
                    yield server.run(overrideConfig);
                });
            });
            it('Should have appropriate P2P config', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    const config = yield server.config.getConfig();
                    (0, chai_1.expect)(config.defaults.p2p.webapp.enabled).to.be.true;
                    (0, chai_1.expect)(config.defaults.p2p.embed.enabled).to.be.false;
                });
            });
            it('Should create a user with this default setting', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.users.create({ username: 'user_p2p_3' });
                    const userToken = yield server.login.getAccessToken('user_p2p_3');
                    const { p2pEnabled } = yield server.users.getMyInfo({ token: userToken });
                    (0, chai_1.expect)(p2pEnabled).to.be.true;
                });
            });
            it('Should register a user with this default setting', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield server.registrations.register({ username: 'user_p2p_4' });
                    const userToken = yield server.login.getAccessToken('user_p2p_4');
                    const { p2pEnabled } = yield server.users.getMyInfo({ token: userToken });
                    (0, chai_1.expect)(p2pEnabled).to.be.true;
                });
            });
        });
    });
    describe('Default user attributes', function () {
        it('Should create a user and register a user with the default config', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.config.updateCustomSubConfig({
                    newConfig: {
                        user: {
                            history: {
                                videos: {
                                    enabled: true
                                }
                            },
                            videoQuota: -1,
                            videoQuotaDaily: -1
                        },
                        signup: {
                            enabled: true,
                            requiresApproval: false
                        }
                    }
                });
                const config = yield server.config.getConfig();
                (0, chai_1.expect)(config.user.videoQuota).to.equal(-1);
                (0, chai_1.expect)(config.user.videoQuotaDaily).to.equal(-1);
                const user1Token = yield server.users.generateUserAndToken('user1');
                const user1 = yield server.users.getMyInfo({ token: user1Token });
                const user = { displayName: 'super user 2', username: 'user2', password: 'super password' };
                const channel = { name: 'my_user_2_channel', displayName: 'my channel' };
                yield server.registrations.register(Object.assign(Object.assign({}, user), { channel }));
                const user2Token = yield server.login.getAccessToken(user);
                const user2 = yield server.users.getMyInfo({ token: user2Token });
                for (const user of [user1, user2]) {
                    (0, chai_1.expect)(user.videosHistoryEnabled).to.be.true;
                    (0, chai_1.expect)(user.videoQuota).to.equal(-1);
                    (0, chai_1.expect)(user.videoQuotaDaily).to.equal(-1);
                }
            });
        });
        it('Should update config and create a user and register a user with the new default config', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.config.updateCustomSubConfig({
                    newConfig: {
                        user: {
                            history: {
                                videos: {
                                    enabled: false
                                }
                            },
                            videoQuota: 5242881,
                            videoQuotaDaily: 318742
                        },
                        signup: {
                            enabled: true,
                            requiresApproval: false
                        }
                    }
                });
                const user3Token = yield server.users.generateUserAndToken('user3');
                const user3 = yield server.users.getMyInfo({ token: user3Token });
                const user = { displayName: 'super user 4', username: 'user4', password: 'super password' };
                const channel = { name: 'my_user_4_channel', displayName: 'my channel' };
                yield server.registrations.register(Object.assign(Object.assign({}, user), { channel }));
                const user4Token = yield server.login.getAccessToken(user);
                const user4 = yield server.users.getMyInfo({ token: user4Token });
                for (const user of [user3, user4]) {
                    (0, chai_1.expect)(user.videosHistoryEnabled).to.be.false;
                    (0, chai_1.expect)(user.videoQuota).to.equal(5242881);
                    (0, chai_1.expect)(user.videoQuotaDaily).to.equal(318742);
                }
            });
        });
    });
    after(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield (0, server_commands_1.cleanupTests)([server]);
        });
    });
});
//# sourceMappingURL=config-defaults.js.map