"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chai_1 = require("chai");
const shared_1 = require("../../shared");
const core_utils_1 = require("../../../../shared/core-utils");
const server_commands_1 = require("../../../../shared/server-commands");
describe('Test oauth', function () {
    let server;
    let sqlCommand;
    before(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(30000);
            server = yield (0, server_commands_1.createSingleServer)(1, {
                rates_limit: {
                    login: {
                        max: 30
                    }
                }
            });
            yield (0, server_commands_1.setAccessTokensToServers)([server]);
            sqlCommand = new shared_1.SQLCommand(server);
        });
    });
    describe('OAuth client', function () {
        function expectInvalidClient(body) {
            (0, chai_1.expect)(body.code).to.equal("invalid_client");
            (0, chai_1.expect)(body.error).to.contain('client is invalid');
            (0, chai_1.expect)(body.type.startsWith('https://')).to.be.true;
            (0, chai_1.expect)(body.type).to.contain("invalid_client");
        }
        it('Should create a new client');
        it('Should return the first client');
        it('Should remove the last client');
        it('Should not login with an invalid client id', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const client = { id: 'client', secret: server.store.client.secret };
                const body = yield server.login.login({ client, expectedStatus: 400 });
                expectInvalidClient(body);
            });
        });
        it('Should not login with an invalid client secret', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const client = { id: server.store.client.id, secret: 'coucou' };
                const body = yield server.login.login({ client, expectedStatus: 400 });
                expectInvalidClient(body);
            });
        });
    });
    describe('Login', function () {
        function expectInvalidCredentials(body) {
            (0, chai_1.expect)(body.code).to.equal("invalid_grant");
            (0, chai_1.expect)(body.error).to.contain('credentials are invalid');
            (0, chai_1.expect)(body.type.startsWith('https://')).to.be.true;
            (0, chai_1.expect)(body.type).to.contain("invalid_grant");
        }
        it('Should not login with an invalid username', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const user = { username: 'captain crochet', password: server.store.user.password };
                const body = yield server.login.login({ user, expectedStatus: 400 });
                expectInvalidCredentials(body);
            });
        });
        it('Should not login with an invalid password', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const user = { username: server.store.user.username, password: 'mew_three' };
                const body = yield server.login.login({ user, expectedStatus: 400 });
                expectInvalidCredentials(body);
            });
        });
        it('Should be able to login', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.login.login({ expectedStatus: 200 });
            });
        });
        it('Should be able to login with an insensitive username', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const user = { username: 'RoOt', password: server.store.user.password };
                yield server.login.login({ user, expectedStatus: 200 });
                const user2 = { username: 'rOoT', password: server.store.user.password };
                yield server.login.login({ user: user2, expectedStatus: 200 });
                const user3 = { username: 'ROOt', password: server.store.user.password };
                yield server.login.login({ user: user3, expectedStatus: 200 });
            });
        });
    });
    describe('Logout', function () {
        it('Should logout (revoke token)', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.login.logout({ token: server.accessToken });
            });
        });
        it('Should not be able to get the user information', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.users.getMyInfo({ expectedStatus: 401 });
            });
        });
        it('Should not be able to upload a video', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.videos.upload({ attributes: { name: 'video' }, expectedStatus: 401 });
            });
        });
        it('Should be able to login again', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const body = yield server.login.login();
                server.accessToken = body.access_token;
                server.refreshToken = body.refresh_token;
            });
        });
        it('Should be able to get my user information again', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.users.getMyInfo();
            });
        });
        it('Should have an expired access token', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(60000);
                yield sqlCommand.setTokenField(server.accessToken, 'accessTokenExpiresAt', new Date().toISOString());
                yield sqlCommand.setTokenField(server.accessToken, 'refreshTokenExpiresAt', new Date().toISOString());
                yield (0, server_commands_1.killallServers)([server]);
                yield server.run();
                yield server.users.getMyInfo({ expectedStatus: 401 });
            });
        });
        it('Should not be able to refresh an access token with an expired refresh token', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.login.refreshToken({ refreshToken: server.refreshToken, expectedStatus: 400 });
            });
        });
        it('Should refresh the token', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(50000);
                const futureDate = new Date(new Date().getTime() + 1000 * 60).toISOString();
                yield sqlCommand.setTokenField(server.accessToken, 'refreshTokenExpiresAt', futureDate);
                yield (0, server_commands_1.killallServers)([server]);
                yield server.run();
                const res = yield server.login.refreshToken({ refreshToken: server.refreshToken });
                server.accessToken = res.body.access_token;
                server.refreshToken = res.body.refresh_token;
            });
        });
        it('Should be able to get my user information again', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.users.getMyInfo();
            });
        });
    });
    describe('Custom token lifetime', function () {
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(120000);
                yield server.kill();
                yield server.run({
                    oauth2: {
                        token_lifetime: {
                            access_token: '2 seconds',
                            refresh_token: '2 seconds'
                        }
                    }
                });
            });
        });
        it('Should have a very short access token lifetime', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(50000);
                const { access_token: accessToken } = yield server.login.login();
                yield server.users.getMyInfo({ token: accessToken });
                yield (0, core_utils_1.wait)(3000);
                yield server.users.getMyInfo({ token: accessToken, expectedStatus: 401 });
            });
        });
        it('Should have a very short refresh token lifetime', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(50000);
                const { refresh_token: refreshToken } = yield server.login.login();
                yield server.login.refreshToken({ refreshToken });
                yield (0, core_utils_1.wait)(3000);
                yield server.login.refreshToken({ refreshToken, expectedStatus: 400 });
            });
        });
    });
    after(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield sqlCommand.cleanup();
            yield (0, server_commands_1.cleanupTests)([server]);
        });
    });
});
//# sourceMappingURL=oauth.js.map