"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chai_1 = require("chai");
const shared_1 = require("../../shared");
const server_commands_1 = require("../../../../shared/server-commands");
describe('Test registrations', function () {
    let server;
    const emails = [];
    let emailPort;
    before(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(30000);
            emailPort = yield shared_1.MockSmtpServer.Instance.collectEmails(emails);
            server = yield (0, server_commands_1.createSingleServer)(1, server_commands_1.ConfigCommand.getEmailOverrideConfig(emailPort));
            yield (0, server_commands_1.setAccessTokensToServers)([server]);
            yield server.config.enableSignup(false);
        });
    });
    describe('Direct registrations of a new user', function () {
        let user1Token;
        it('Should register a new user', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const user = { displayName: 'super user 1', username: 'user_1', password: 'my super password' };
                const channel = { name: 'my_user_1_channel', displayName: 'my channel rocks' };
                yield server.registrations.register(Object.assign(Object.assign({}, user), { channel }));
            });
        });
        it('Should be able to login with this registered user', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const user1 = { username: 'user_1', password: 'my super password' };
                user1Token = yield server.login.getAccessToken(user1);
            });
        });
        it('Should have the correct display name', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const user = yield server.users.getMyInfo({ token: user1Token });
                (0, chai_1.expect)(user.account.displayName).to.equal('super user 1');
            });
        });
        it('Should have the correct video quota', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const user = yield server.users.getMyInfo({ token: user1Token });
                (0, chai_1.expect)(user.videoQuota).to.equal(5 * 1024 * 1024);
            });
        });
        it('Should have created the channel', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { displayName } = yield server.channels.get({ channelName: 'my_user_1_channel' });
                (0, chai_1.expect)(displayName).to.equal('my channel rocks');
            });
        });
        it('Should remove me', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                {
                    const { data } = yield server.users.list();
                    (0, chai_1.expect)(data.find(u => u.username === 'user_1')).to.not.be.undefined;
                }
                yield server.users.deleteMe({ token: user1Token });
                {
                    const { data } = yield server.users.list();
                    (0, chai_1.expect)(data.find(u => u.username === 'user_1')).to.be.undefined;
                }
            });
        });
    });
    describe('Registration requests', function () {
        let id2;
        let id3;
        let id4;
        let user2Token;
        let user3Token;
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(60000);
                yield server.config.enableSignup(true);
                {
                    const { id } = yield server.registrations.requestRegistration({
                        username: 'user4',
                        registrationReason: 'registration reason 4'
                    });
                    id4 = id;
                }
            });
        });
        it('Should request a registration without a channel', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                {
                    const { id } = yield server.registrations.requestRegistration({
                        username: 'user2',
                        displayName: 'my super user 2',
                        email: 'user2@example.com',
                        password: 'user2password',
                        registrationReason: 'registration reason 2'
                    });
                    id2 = id;
                }
            });
        });
        it('Should request a registration with a channel', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { id } = yield server.registrations.requestRegistration({
                    username: 'user3',
                    displayName: 'my super user 3',
                    channel: {
                        displayName: 'my user 3 channel',
                        name: 'super_user3_channel'
                    },
                    email: 'user3@example.com',
                    password: 'user3password',
                    registrationReason: 'registration reason 3'
                });
                id3 = id;
            });
        });
        it('Should list these registration requests', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                {
                    const { total, data } = yield server.registrations.list({ sort: '-createdAt' });
                    (0, chai_1.expect)(total).to.equal(3);
                    (0, chai_1.expect)(data).to.have.lengthOf(3);
                    {
                        (0, chai_1.expect)(data[0].id).to.equal(id3);
                        (0, chai_1.expect)(data[0].username).to.equal('user3');
                        (0, chai_1.expect)(data[0].accountDisplayName).to.equal('my super user 3');
                        (0, chai_1.expect)(data[0].channelDisplayName).to.equal('my user 3 channel');
                        (0, chai_1.expect)(data[0].channelHandle).to.equal('super_user3_channel');
                        (0, chai_1.expect)(data[0].createdAt).to.exist;
                        (0, chai_1.expect)(data[0].updatedAt).to.exist;
                        (0, chai_1.expect)(data[0].email).to.equal('user3@example.com');
                        (0, chai_1.expect)(data[0].emailVerified).to.be.null;
                        (0, chai_1.expect)(data[0].moderationResponse).to.be.null;
                        (0, chai_1.expect)(data[0].registrationReason).to.equal('registration reason 3');
                        (0, chai_1.expect)(data[0].state.id).to.equal(1);
                        (0, chai_1.expect)(data[0].state.label).to.equal('Pending');
                        (0, chai_1.expect)(data[0].user).to.be.null;
                    }
                    {
                        (0, chai_1.expect)(data[1].id).to.equal(id2);
                        (0, chai_1.expect)(data[1].username).to.equal('user2');
                        (0, chai_1.expect)(data[1].accountDisplayName).to.equal('my super user 2');
                        (0, chai_1.expect)(data[1].channelDisplayName).to.be.null;
                        (0, chai_1.expect)(data[1].channelHandle).to.be.null;
                        (0, chai_1.expect)(data[1].createdAt).to.exist;
                        (0, chai_1.expect)(data[1].updatedAt).to.exist;
                        (0, chai_1.expect)(data[1].email).to.equal('user2@example.com');
                        (0, chai_1.expect)(data[1].emailVerified).to.be.null;
                        (0, chai_1.expect)(data[1].moderationResponse).to.be.null;
                        (0, chai_1.expect)(data[1].registrationReason).to.equal('registration reason 2');
                        (0, chai_1.expect)(data[1].state.id).to.equal(1);
                        (0, chai_1.expect)(data[1].state.label).to.equal('Pending');
                        (0, chai_1.expect)(data[1].user).to.be.null;
                    }
                    {
                        (0, chai_1.expect)(data[2].username).to.equal('user4');
                    }
                }
                {
                    const { total, data } = yield server.registrations.list({ count: 1, start: 1, sort: 'createdAt' });
                    (0, chai_1.expect)(total).to.equal(3);
                    (0, chai_1.expect)(data).to.have.lengthOf(1);
                    (0, chai_1.expect)(data[0].id).to.equal(id2);
                }
                {
                    const { total, data } = yield server.registrations.list({ search: 'user3' });
                    (0, chai_1.expect)(total).to.equal(1);
                    (0, chai_1.expect)(data).to.have.lengthOf(1);
                    (0, chai_1.expect)(data[0].id).to.equal(id3);
                }
            });
        });
        it('Should reject a registration request', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.registrations.reject({ id: id4, moderationResponse: 'I do not want id 4 on this instance' });
            });
        });
        it('Should have sent an email to the user explanining the registration has been rejected', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(50000);
                yield (0, server_commands_1.waitJobs)([server]);
                const email = emails.find(e => e['to'][0]['address'] === 'user4@example.com');
                (0, chai_1.expect)(email).to.exist;
                (0, chai_1.expect)(email['subject']).to.contain('been rejected');
                (0, chai_1.expect)(email['text']).to.contain('been rejected');
                (0, chai_1.expect)(email['text']).to.contain('I do not want id 4 on this instance');
            });
        });
        it('Should accept registration requests', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.registrations.accept({ id: id2, moderationResponse: 'Welcome id 2' });
                yield server.registrations.accept({ id: id3, moderationResponse: 'Welcome id 3' });
            });
        });
        it('Should have sent an email to the user explanining the registration has been accepted', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(50000);
                yield (0, server_commands_1.waitJobs)([server]);
                {
                    const email = emails.find(e => e['to'][0]['address'] === 'user2@example.com');
                    (0, chai_1.expect)(email).to.exist;
                    (0, chai_1.expect)(email['subject']).to.contain('been accepted');
                    (0, chai_1.expect)(email['text']).to.contain('been accepted');
                    (0, chai_1.expect)(email['text']).to.contain('Welcome id 2');
                }
                {
                    const email = emails.find(e => e['to'][0]['address'] === 'user3@example.com');
                    (0, chai_1.expect)(email).to.exist;
                    (0, chai_1.expect)(email['subject']).to.contain('been accepted');
                    (0, chai_1.expect)(email['text']).to.contain('been accepted');
                    (0, chai_1.expect)(email['text']).to.contain('Welcome id 3');
                }
            });
        });
        it('Should login with these users', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                user2Token = yield server.login.getAccessToken({ username: 'user2', password: 'user2password' });
                user3Token = yield server.login.getAccessToken({ username: 'user3', password: 'user3password' });
            });
        });
        it('Should have created the appropriate attributes for user 2', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const me = yield server.users.getMyInfo({ token: user2Token });
                (0, chai_1.expect)(me.username).to.equal('user2');
                (0, chai_1.expect)(me.account.displayName).to.equal('my super user 2');
                (0, chai_1.expect)(me.videoQuota).to.equal(5 * 1024 * 1024);
                (0, chai_1.expect)(me.videoChannels[0].name).to.equal('user2_channel');
                (0, chai_1.expect)(me.videoChannels[0].displayName).to.equal('Main user2 channel');
                (0, chai_1.expect)(me.role.id).to.equal(2);
                (0, chai_1.expect)(me.email).to.equal('user2@example.com');
            });
        });
        it('Should have created the appropriate attributes for user 3', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const me = yield server.users.getMyInfo({ token: user3Token });
                (0, chai_1.expect)(me.username).to.equal('user3');
                (0, chai_1.expect)(me.account.displayName).to.equal('my super user 3');
                (0, chai_1.expect)(me.videoQuota).to.equal(5 * 1024 * 1024);
                (0, chai_1.expect)(me.videoChannels[0].name).to.equal('super_user3_channel');
                (0, chai_1.expect)(me.videoChannels[0].displayName).to.equal('my user 3 channel');
                (0, chai_1.expect)(me.role.id).to.equal(2);
                (0, chai_1.expect)(me.email).to.equal('user3@example.com');
            });
        });
        it('Should list these accepted/rejected registration requests', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { data } = yield server.registrations.list({ sort: 'createdAt' });
                const { data: users } = yield server.users.list();
                {
                    (0, chai_1.expect)(data[0].id).to.equal(id4);
                    (0, chai_1.expect)(data[0].state.id).to.equal(2);
                    (0, chai_1.expect)(data[0].state.label).to.equal('Rejected');
                    (0, chai_1.expect)(data[0].moderationResponse).to.equal('I do not want id 4 on this instance');
                    (0, chai_1.expect)(data[0].user).to.be.null;
                    (0, chai_1.expect)(users.find(u => u.username === 'user4')).to.not.exist;
                }
                {
                    (0, chai_1.expect)(data[1].id).to.equal(id2);
                    (0, chai_1.expect)(data[1].state.id).to.equal(3);
                    (0, chai_1.expect)(data[1].state.label).to.equal('Accepted');
                    (0, chai_1.expect)(data[1].moderationResponse).to.equal('Welcome id 2');
                    (0, chai_1.expect)(data[1].user).to.exist;
                    const user2 = users.find(u => u.username === 'user2');
                    (0, chai_1.expect)(data[1].user.id).to.equal(user2.id);
                }
                {
                    (0, chai_1.expect)(data[2].id).to.equal(id3);
                    (0, chai_1.expect)(data[2].state.id).to.equal(3);
                    (0, chai_1.expect)(data[2].state.label).to.equal('Accepted');
                    (0, chai_1.expect)(data[2].moderationResponse).to.equal('Welcome id 3');
                    (0, chai_1.expect)(data[2].user).to.exist;
                    const user3 = users.find(u => u.username === 'user3');
                    (0, chai_1.expect)(data[2].user.id).to.equal(user3.id);
                }
            });
        });
        it('Shoulde delete a registration', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.registrations.delete({ id: id2 });
                yield server.registrations.delete({ id: id3 });
                const { total, data } = yield server.registrations.list();
                (0, chai_1.expect)(total).to.equal(1);
                (0, chai_1.expect)(data).to.have.lengthOf(1);
                (0, chai_1.expect)(data[0].id).to.equal(id4);
                const { data: users } = yield server.users.list();
                for (const username of ['user2', 'user3']) {
                    (0, chai_1.expect)(users.find(u => u.username === username)).to.exist;
                }
            });
        });
        it('Should be able to prevent email delivery on accept/reject', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(50000);
                let id1;
                let id2;
                {
                    const { id } = yield server.registrations.requestRegistration({
                        username: 'user7',
                        email: 'user7@example.com',
                        registrationReason: 'tt'
                    });
                    id1 = id;
                }
                {
                    const { id } = yield server.registrations.requestRegistration({
                        username: 'user8',
                        email: 'user8@example.com',
                        registrationReason: 'tt'
                    });
                    id2 = id;
                }
                yield server.registrations.accept({ id: id1, moderationResponse: 'tt', preventEmailDelivery: true });
                yield server.registrations.reject({ id: id2, moderationResponse: 'tt', preventEmailDelivery: true });
                yield (0, server_commands_1.waitJobs)([server]);
                const filtered = emails.filter(e => {
                    const address = e['to'][0]['address'];
                    return address === 'user7@example.com' || address === 'user8@example.com';
                });
                (0, chai_1.expect)(filtered).to.have.lengthOf(0);
            });
        });
        it('Should request a registration without a channel, that will conflict with an already existing channel', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                let id1;
                let id2;
                {
                    const { id } = yield server.registrations.requestRegistration({
                        registrationReason: 'tt',
                        username: 'user5',
                        password: 'user5password',
                        channel: {
                            displayName: 'channel 6',
                            name: 'user6_channel'
                        }
                    });
                    id1 = id;
                }
                {
                    const { id } = yield server.registrations.requestRegistration({
                        registrationReason: 'tt',
                        username: 'user6',
                        password: 'user6password'
                    });
                    id2 = id;
                }
                yield server.registrations.accept({ id: id1, moderationResponse: 'tt' });
                yield server.registrations.accept({ id: id2, moderationResponse: 'tt' });
                const user5Token = yield server.login.getAccessToken('user5', 'user5password');
                const user6Token = yield server.login.getAccessToken('user6', 'user6password');
                const user5 = yield server.users.getMyInfo({ token: user5Token });
                const user6 = yield server.users.getMyInfo({ token: user6Token });
                (0, chai_1.expect)(user5.videoChannels[0].name).to.equal('user6_channel');
                (0, chai_1.expect)(user6.videoChannels[0].name).to.equal('user6_channel-1');
            });
        });
    });
    after(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            shared_1.MockSmtpServer.Instance.kill();
            yield (0, server_commands_1.cleanupTests)([server]);
        });
    });
});
//# sourceMappingURL=registrations.js.map