"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chai_1 = require("chai");
const shared_1 = require("../../shared");
const server_commands_1 = require("../../../../shared/server-commands");
describe('Test users', function () {
    let server;
    let token;
    let userToken;
    let videoId;
    let userId;
    const user = {
        username: 'user_1',
        password: 'super password'
    };
    before(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(30000);
            server = yield (0, server_commands_1.createSingleServer)(1, {
                rates_limit: {
                    login: {
                        max: 30
                    }
                }
            });
            yield (0, server_commands_1.setAccessTokensToServers)([server]);
            yield server.plugins.install({ npmName: 'peertube-theme-background-red' });
        });
    });
    describe('Creating a user', function () {
        it('Should be able to create a new user', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.users.create(Object.assign(Object.assign({}, user), { videoQuota: 2 * 1024 * 1024, adminFlags: 1 }));
            });
        });
        it('Should be able to login with this user', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                userToken = yield server.login.getAccessToken(user);
            });
        });
        it('Should be able to get user information', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const userMe = yield server.users.getMyInfo({ token: userToken });
                const userGet = yield server.users.get({ userId: userMe.id, withStats: true });
                for (const user of [userMe, userGet]) {
                    (0, chai_1.expect)(user.username).to.equal('user_1');
                    (0, chai_1.expect)(user.email).to.equal('user_1@example.com');
                    (0, chai_1.expect)(user.nsfwPolicy).to.equal('display');
                    (0, chai_1.expect)(user.videoQuota).to.equal(2 * 1024 * 1024);
                    (0, chai_1.expect)(user.role.label).to.equal('User');
                    (0, chai_1.expect)(user.id).to.be.a('number');
                    (0, chai_1.expect)(user.account.displayName).to.equal('user_1');
                    (0, chai_1.expect)(user.account.description).to.be.null;
                }
                (0, chai_1.expect)(userMe.adminFlags).to.equal(1);
                (0, chai_1.expect)(userGet.adminFlags).to.equal(1);
                (0, chai_1.expect)(userMe.specialPlaylists).to.have.lengthOf(1);
                (0, chai_1.expect)(userMe.specialPlaylists[0].type).to.equal(2);
                (0, chai_1.expect)(userGet.videosCount).to.be.a('number');
                (0, chai_1.expect)(userGet.videosCount).to.equal(0);
                (0, chai_1.expect)(userGet.videoCommentsCount).to.be.a('number');
                (0, chai_1.expect)(userGet.videoCommentsCount).to.equal(0);
                (0, chai_1.expect)(userGet.abusesCount).to.be.a('number');
                (0, chai_1.expect)(userGet.abusesCount).to.equal(0);
                (0, chai_1.expect)(userGet.abusesAcceptedCount).to.be.a('number');
                (0, chai_1.expect)(userGet.abusesAcceptedCount).to.equal(0);
            });
        });
    });
    describe('Users listing', function () {
        it('Should list all the users', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { data, total } = yield server.users.list();
                (0, chai_1.expect)(total).to.equal(2);
                (0, chai_1.expect)(data).to.be.an('array');
                (0, chai_1.expect)(data.length).to.equal(2);
                const user = data[0];
                (0, chai_1.expect)(user.username).to.equal('user_1');
                (0, chai_1.expect)(user.email).to.equal('user_1@example.com');
                (0, chai_1.expect)(user.nsfwPolicy).to.equal('display');
                const rootUser = data[1];
                (0, chai_1.expect)(rootUser.username).to.equal('root');
                (0, chai_1.expect)(rootUser.email).to.equal('admin' + server.internalServerNumber + '@example.com');
                (0, chai_1.expect)(user.nsfwPolicy).to.equal('display');
                (0, chai_1.expect)(rootUser.lastLoginDate).to.exist;
                (0, chai_1.expect)(user.lastLoginDate).to.exist;
                userId = user.id;
            });
        });
        it('Should list only the first user by username asc', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { total, data } = yield server.users.list({ start: 0, count: 1, sort: 'username' });
                (0, chai_1.expect)(total).to.equal(2);
                (0, chai_1.expect)(data.length).to.equal(1);
                const user = data[0];
                (0, chai_1.expect)(user.username).to.equal('root');
                (0, chai_1.expect)(user.email).to.equal('admin' + server.internalServerNumber + '@example.com');
                (0, chai_1.expect)(user.role.label).to.equal('Administrator');
                (0, chai_1.expect)(user.nsfwPolicy).to.equal('display');
            });
        });
        it('Should list only the first user by username desc', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { total, data } = yield server.users.list({ start: 0, count: 1, sort: '-username' });
                (0, chai_1.expect)(total).to.equal(2);
                (0, chai_1.expect)(data.length).to.equal(1);
                const user = data[0];
                (0, chai_1.expect)(user.username).to.equal('user_1');
                (0, chai_1.expect)(user.email).to.equal('user_1@example.com');
                (0, chai_1.expect)(user.nsfwPolicy).to.equal('display');
            });
        });
        it('Should list only the second user by createdAt desc', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { data, total } = yield server.users.list({ start: 0, count: 1, sort: '-createdAt' });
                (0, chai_1.expect)(total).to.equal(2);
                (0, chai_1.expect)(data.length).to.equal(1);
                const user = data[0];
                (0, chai_1.expect)(user.username).to.equal('user_1');
                (0, chai_1.expect)(user.email).to.equal('user_1@example.com');
                (0, chai_1.expect)(user.nsfwPolicy).to.equal('display');
            });
        });
        it('Should list all the users by createdAt asc', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { data, total } = yield server.users.list({ start: 0, count: 2, sort: 'createdAt' });
                (0, chai_1.expect)(total).to.equal(2);
                (0, chai_1.expect)(data.length).to.equal(2);
                (0, chai_1.expect)(data[0].username).to.equal('root');
                (0, chai_1.expect)(data[0].email).to.equal('admin' + server.internalServerNumber + '@example.com');
                (0, chai_1.expect)(data[0].nsfwPolicy).to.equal('display');
                (0, chai_1.expect)(data[1].username).to.equal('user_1');
                (0, chai_1.expect)(data[1].email).to.equal('user_1@example.com');
                (0, chai_1.expect)(data[1].nsfwPolicy).to.equal('display');
            });
        });
        it('Should search user by username', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { data, total } = yield server.users.list({ start: 0, count: 2, sort: 'createdAt', search: 'oot' });
                (0, chai_1.expect)(total).to.equal(1);
                (0, chai_1.expect)(data.length).to.equal(1);
                (0, chai_1.expect)(data[0].username).to.equal('root');
            });
        });
        it('Should search user by email', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                {
                    const { total, data } = yield server.users.list({ start: 0, count: 2, sort: 'createdAt', search: 'r_1@exam' });
                    (0, chai_1.expect)(total).to.equal(1);
                    (0, chai_1.expect)(data.length).to.equal(1);
                    (0, chai_1.expect)(data[0].username).to.equal('user_1');
                    (0, chai_1.expect)(data[0].email).to.equal('user_1@example.com');
                }
                {
                    const { total, data } = yield server.users.list({ start: 0, count: 2, sort: 'createdAt', search: 'example' });
                    (0, chai_1.expect)(total).to.equal(2);
                    (0, chai_1.expect)(data.length).to.equal(2);
                    (0, chai_1.expect)(data[0].username).to.equal('root');
                    (0, chai_1.expect)(data[1].username).to.equal('user_1');
                }
            });
        });
    });
    describe('Update my account', function () {
        it('Should update my password', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.users.updateMe({
                    token: userToken,
                    currentPassword: 'super password',
                    password: 'new password'
                });
                user.password = 'new password';
                yield server.login.login({ user });
            });
        });
        it('Should be able to change the NSFW display attribute', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.users.updateMe({
                    token: userToken,
                    nsfwPolicy: 'do_not_list'
                });
                const user = yield server.users.getMyInfo({ token: userToken });
                (0, chai_1.expect)(user.username).to.equal('user_1');
                (0, chai_1.expect)(user.email).to.equal('user_1@example.com');
                (0, chai_1.expect)(user.nsfwPolicy).to.equal('do_not_list');
                (0, chai_1.expect)(user.videoQuota).to.equal(2 * 1024 * 1024);
                (0, chai_1.expect)(user.id).to.be.a('number');
                (0, chai_1.expect)(user.account.displayName).to.equal('user_1');
                (0, chai_1.expect)(user.account.description).to.be.null;
            });
        });
        it('Should be able to change the autoPlayVideo attribute', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.users.updateMe({
                    token: userToken,
                    autoPlayVideo: false
                });
                const user = yield server.users.getMyInfo({ token: userToken });
                (0, chai_1.expect)(user.autoPlayVideo).to.be.false;
            });
        });
        it('Should be able to change the autoPlayNextVideo attribute', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.users.updateMe({
                    token: userToken,
                    autoPlayNextVideo: true
                });
                const user = yield server.users.getMyInfo({ token: userToken });
                (0, chai_1.expect)(user.autoPlayNextVideo).to.be.true;
            });
        });
        it('Should be able to change the p2p attribute', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                {
                    yield server.users.updateMe({
                        token: userToken,
                        webTorrentEnabled: false
                    });
                    const user = yield server.users.getMyInfo({ token: userToken });
                    (0, chai_1.expect)(user.p2pEnabled).to.be.false;
                }
                {
                    yield server.users.updateMe({
                        token: userToken,
                        p2pEnabled: true
                    });
                    const user = yield server.users.getMyInfo({ token: userToken });
                    (0, chai_1.expect)(user.p2pEnabled).to.be.true;
                }
            });
        });
        it('Should be able to change the email attribute', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.users.updateMe({
                    token: userToken,
                    currentPassword: 'new password',
                    email: 'updated@example.com'
                });
                const user = yield server.users.getMyInfo({ token: userToken });
                (0, chai_1.expect)(user.username).to.equal('user_1');
                (0, chai_1.expect)(user.email).to.equal('updated@example.com');
                (0, chai_1.expect)(user.nsfwPolicy).to.equal('do_not_list');
                (0, chai_1.expect)(user.videoQuota).to.equal(2 * 1024 * 1024);
                (0, chai_1.expect)(user.id).to.be.a('number');
                (0, chai_1.expect)(user.account.displayName).to.equal('user_1');
                (0, chai_1.expect)(user.account.description).to.be.null;
            });
        });
        it('Should be able to update my avatar with a gif', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const fixture = 'avatar.gif';
                yield server.users.updateMyAvatar({ token: userToken, fixture });
                const user = yield server.users.getMyInfo({ token: userToken });
                for (const avatar of user.account.avatars) {
                    yield (0, shared_1.testImageSize)(server.url, `avatar-resized-${avatar.width}x${avatar.width}`, avatar.path, '.gif');
                }
            });
        });
        it('Should be able to update my avatar with a gif, and then a png', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                for (const extension of ['.png', '.gif']) {
                    const fixture = 'avatar' + extension;
                    yield server.users.updateMyAvatar({ token: userToken, fixture });
                    const user = yield server.users.getMyInfo({ token: userToken });
                    for (const avatar of user.account.avatars) {
                        yield (0, shared_1.testImageSize)(server.url, `avatar-resized-${avatar.width}x${avatar.width}`, avatar.path, extension);
                    }
                }
            });
        });
        it('Should be able to update my display name', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.users.updateMe({ token: userToken, displayName: 'new display name' });
                const user = yield server.users.getMyInfo({ token: userToken });
                (0, chai_1.expect)(user.username).to.equal('user_1');
                (0, chai_1.expect)(user.email).to.equal('updated@example.com');
                (0, chai_1.expect)(user.nsfwPolicy).to.equal('do_not_list');
                (0, chai_1.expect)(user.videoQuota).to.equal(2 * 1024 * 1024);
                (0, chai_1.expect)(user.id).to.be.a('number');
                (0, chai_1.expect)(user.account.displayName).to.equal('new display name');
                (0, chai_1.expect)(user.account.description).to.be.null;
            });
        });
        it('Should be able to update my description', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.users.updateMe({ token: userToken, description: 'my super description updated' });
                const user = yield server.users.getMyInfo({ token: userToken });
                (0, chai_1.expect)(user.username).to.equal('user_1');
                (0, chai_1.expect)(user.email).to.equal('updated@example.com');
                (0, chai_1.expect)(user.nsfwPolicy).to.equal('do_not_list');
                (0, chai_1.expect)(user.videoQuota).to.equal(2 * 1024 * 1024);
                (0, chai_1.expect)(user.id).to.be.a('number');
                (0, chai_1.expect)(user.account.displayName).to.equal('new display name');
                (0, chai_1.expect)(user.account.description).to.equal('my super description updated');
                (0, chai_1.expect)(user.noWelcomeModal).to.be.false;
                (0, chai_1.expect)(user.noInstanceConfigWarningModal).to.be.false;
                (0, chai_1.expect)(user.noAccountSetupWarningModal).to.be.false;
            });
        });
        it('Should be able to update my theme', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                for (const theme of ['background-red', 'default', 'instance-default']) {
                    yield server.users.updateMe({ token: userToken, theme });
                    const user = yield server.users.getMyInfo({ token: userToken });
                    (0, chai_1.expect)(user.theme).to.equal(theme);
                }
            });
        });
        it('Should be able to update my modal preferences', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.users.updateMe({
                    token: userToken,
                    noInstanceConfigWarningModal: true,
                    noWelcomeModal: true,
                    noAccountSetupWarningModal: true
                });
                const user = yield server.users.getMyInfo({ token: userToken });
                (0, chai_1.expect)(user.noWelcomeModal).to.be.true;
                (0, chai_1.expect)(user.noInstanceConfigWarningModal).to.be.true;
                (0, chai_1.expect)(user.noAccountSetupWarningModal).to.be.true;
            });
        });
    });
    describe('Updating another user', function () {
        it('Should be able to update another user', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.users.update({
                    userId,
                    token,
                    email: 'updated2@example.com',
                    emailVerified: true,
                    videoQuota: 42,
                    role: 1,
                    adminFlags: 0,
                    pluginAuth: 'toto'
                });
                const user = yield server.users.get({ token, userId });
                (0, chai_1.expect)(user.username).to.equal('user_1');
                (0, chai_1.expect)(user.email).to.equal('updated2@example.com');
                (0, chai_1.expect)(user.emailVerified).to.be.true;
                (0, chai_1.expect)(user.nsfwPolicy).to.equal('do_not_list');
                (0, chai_1.expect)(user.videoQuota).to.equal(42);
                (0, chai_1.expect)(user.role.label).to.equal('Moderator');
                (0, chai_1.expect)(user.id).to.be.a('number');
                (0, chai_1.expect)(user.adminFlags).to.equal(0);
                (0, chai_1.expect)(user.pluginAuth).to.equal('toto');
            });
        });
        it('Should reset the auth plugin', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.users.update({ userId, token, pluginAuth: null });
                const user = yield server.users.get({ token, userId });
                (0, chai_1.expect)(user.pluginAuth).to.be.null;
            });
        });
        it('Should have removed the user token', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.users.getMyQuotaUsed({ token: userToken, expectedStatus: 401 });
                userToken = yield server.login.getAccessToken(user);
            });
        });
        it('Should be able to update another user password', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.users.update({ userId, token, password: 'password updated' });
                yield server.users.getMyQuotaUsed({ token: userToken, expectedStatus: 401 });
                yield server.login.login({ user, expectedStatus: 400 });
                user.password = 'password updated';
                userToken = yield server.login.getAccessToken(user);
            });
        });
    });
    describe('Remove a user', function () {
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.users.update({
                    userId,
                    token,
                    videoQuota: 2 * 1024 * 1024
                });
                yield server.videos.quickUpload({ name: 'user video', token: userToken, fixture: 'video_short.webm' });
                yield server.videos.quickUpload({ name: 'root video' });
                const { total } = yield server.videos.list();
                (0, chai_1.expect)(total).to.equal(2);
            });
        });
        it('Should be able to remove this user', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.users.remove({ userId, token });
            });
        });
        it('Should not be able to login with this user', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.login.login({ user, expectedStatus: 400 });
            });
        });
        it('Should not have videos of this user', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const { data, total } = yield server.videos.list();
                (0, chai_1.expect)(total).to.equal(1);
                const video = data[0];
                (0, chai_1.expect)(video.account.name).to.equal('root');
            });
        });
    });
    describe('User blocking', function () {
        let user16Id;
        let user16AccessToken;
        const user16 = {
            username: 'user_16',
            password: 'my super password'
        };
        it('Should block a user', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const user = yield server.users.create(Object.assign({}, user16));
                user16Id = user.id;
                user16AccessToken = yield server.login.getAccessToken(user16);
                yield server.users.getMyInfo({ token: user16AccessToken, expectedStatus: 200 });
                yield server.users.banUser({ userId: user16Id });
                yield server.users.getMyInfo({ token: user16AccessToken, expectedStatus: 401 });
                yield server.login.login({ user: user16, expectedStatus: 400 });
            });
        });
        it('Should search user by banned status', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                {
                    const { data, total } = yield server.users.list({ start: 0, count: 2, sort: 'createdAt', blocked: true });
                    (0, chai_1.expect)(total).to.equal(1);
                    (0, chai_1.expect)(data.length).to.equal(1);
                    (0, chai_1.expect)(data[0].username).to.equal(user16.username);
                }
                {
                    const { data, total } = yield server.users.list({ start: 0, count: 2, sort: 'createdAt', blocked: false });
                    (0, chai_1.expect)(total).to.equal(1);
                    (0, chai_1.expect)(data.length).to.equal(1);
                    (0, chai_1.expect)(data[0].username).to.not.equal(user16.username);
                }
            });
        });
        it('Should unblock a user', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield server.users.unbanUser({ userId: user16Id });
                user16AccessToken = yield server.login.getAccessToken(user16);
                yield server.users.getMyInfo({ token: user16AccessToken, expectedStatus: 200 });
            });
        });
    });
    describe('User stats', function () {
        let user17Id;
        let user17AccessToken;
        it('Should report correct initial statistics about a user', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const user17 = {
                    username: 'user_17',
                    password: 'my super password'
                };
                const created = yield server.users.create(Object.assign({}, user17));
                user17Id = created.id;
                user17AccessToken = yield server.login.getAccessToken(user17);
                const user = yield server.users.get({ userId: user17Id, withStats: true });
                (0, chai_1.expect)(user.videosCount).to.equal(0);
                (0, chai_1.expect)(user.videoCommentsCount).to.equal(0);
                (0, chai_1.expect)(user.abusesCount).to.equal(0);
                (0, chai_1.expect)(user.abusesCreatedCount).to.equal(0);
                (0, chai_1.expect)(user.abusesAcceptedCount).to.equal(0);
            });
        });
        it('Should report correct videos count', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const attributes = { name: 'video to test user stats' };
                yield server.videos.upload({ token: user17AccessToken, attributes });
                const { data } = yield server.videos.list();
                videoId = data.find(video => video.name === attributes.name).id;
                const user = yield server.users.get({ userId: user17Id, withStats: true });
                (0, chai_1.expect)(user.videosCount).to.equal(1);
            });
        });
        it('Should report correct video comments for user', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const text = 'super comment';
                yield server.comments.createThread({ token: user17AccessToken, videoId, text });
                const user = yield server.users.get({ userId: user17Id, withStats: true });
                (0, chai_1.expect)(user.videoCommentsCount).to.equal(1);
            });
        });
        it('Should report correct abuses counts', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const reason = 'my super bad reason';
                yield server.abuses.report({ token: user17AccessToken, videoId, reason });
                const body1 = yield server.abuses.getAdminList();
                const abuseId = body1.data[0].id;
                const user2 = yield server.users.get({ userId: user17Id, withStats: true });
                (0, chai_1.expect)(user2.abusesCount).to.equal(1);
                (0, chai_1.expect)(user2.abusesCreatedCount).to.equal(1);
                yield server.abuses.update({ abuseId, body: { state: 3 } });
                const user3 = yield server.users.get({ userId: user17Id, withStats: true });
                (0, chai_1.expect)(user3.abusesAcceptedCount).to.equal(1);
            });
        });
    });
    after(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield (0, server_commands_1.cleanupTests)([server]);
        });
    });
});
//# sourceMappingURL=users.js.map