"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chai_1 = require("chai");
const shared_1 = require("../shared");
const core_utils_1 = require("../../../shared/core-utils");
const server_commands_1 = require("../../../shared/server-commands");
describe('Test Live transcoding in peertube-runner program', function () {
    let servers = [];
    let peertubeRunner;
    let sqlCommandServer1;
    function runSuite(options = {}) {
        const { objectStorage } = options;
        it('Should enable transcoding without additional resolutions', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(120000);
                const { video } = yield servers[0].live.quickCreate({ permanentLive: true, saveReplay: false, privacy: 1 });
                const ffmpegCommand = yield servers[0].live.sendRTMPStreamInVideo({ videoId: video.uuid });
                yield (0, server_commands_1.waitUntilLivePublishedOnAllServers)(servers, video.uuid);
                yield (0, server_commands_1.waitJobs)(servers);
                yield (0, shared_1.testLiveVideoResolutions)({
                    originServer: servers[0],
                    sqlCommand: sqlCommandServer1,
                    servers,
                    liveVideoId: video.uuid,
                    resolutions: [720, 480, 360, 240, 144],
                    objectStorage,
                    transcoded: true
                });
                yield (0, server_commands_1.stopFfmpeg)(ffmpegCommand);
                yield (0, server_commands_1.waitUntilLiveWaitingOnAllServers)(servers, video.uuid);
                yield servers[0].videos.remove({ id: video.id });
            });
        });
        it('Should transcode audio only RTMP stream', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(120000);
                const { video } = yield servers[0].live.quickCreate({ permanentLive: true, saveReplay: false, privacy: 2 });
                const ffmpegCommand = yield servers[0].live.sendRTMPStreamInVideo({ videoId: video.uuid, fixtureName: 'video_short_no_audio.mp4' });
                yield (0, server_commands_1.waitUntilLivePublishedOnAllServers)(servers, video.uuid);
                yield (0, server_commands_1.waitJobs)(servers);
                yield (0, server_commands_1.stopFfmpeg)(ffmpegCommand);
                yield (0, server_commands_1.waitUntilLiveWaitingOnAllServers)(servers, video.uuid);
                yield servers[0].videos.remove({ id: video.id });
            });
        });
        it('Should save a replay', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(240000);
                const { video } = yield servers[0].live.quickCreate({ permanentLive: true, saveReplay: true });
                const ffmpegCommand = yield servers[0].live.sendRTMPStreamInVideo({ videoId: video.uuid });
                yield (0, server_commands_1.waitUntilLivePublishedOnAllServers)(servers, video.uuid);
                yield (0, shared_1.testLiveVideoResolutions)({
                    originServer: servers[0],
                    sqlCommand: sqlCommandServer1,
                    servers,
                    liveVideoId: video.uuid,
                    resolutions: [720, 480, 360, 240, 144],
                    objectStorage,
                    transcoded: true
                });
                yield (0, server_commands_1.stopFfmpeg)(ffmpegCommand);
                yield (0, server_commands_1.waitUntilLiveWaitingOnAllServers)(servers, video.uuid);
                yield (0, server_commands_1.waitJobs)(servers);
                const session = yield servers[0].live.findLatestSession({ videoId: video.uuid });
                (0, chai_1.expect)(session.endingProcessed).to.be.true;
                (0, chai_1.expect)(session.endDate).to.exist;
                (0, chai_1.expect)(session.saveReplay).to.be.true;
                const videoLiveDetails = yield servers[0].videos.get({ id: video.uuid });
                const replay = yield (0, server_commands_1.findExternalSavedVideo)(servers[0], videoLiveDetails);
                for (const server of servers) {
                    const video = yield server.videos.get({ id: replay.uuid });
                    (0, chai_1.expect)(video.files).to.have.lengthOf(0);
                    (0, chai_1.expect)(video.streamingPlaylists).to.have.lengthOf(1);
                    const files = video.streamingPlaylists[0].files;
                    (0, chai_1.expect)(files).to.have.lengthOf(5);
                    for (const file of files) {
                        if (objectStorage) {
                            (0, shared_1.expectStartWith)(file.fileUrl, objectStorage.getMockPlaylistBaseUrl());
                        }
                        yield (0, server_commands_1.makeRawRequest)({ url: file.fileUrl, expectedStatus: 200 });
                    }
                }
            });
        });
    }
    before(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(120000);
            servers = yield (0, server_commands_1.createMultipleServers)(2);
            yield (0, server_commands_1.setAccessTokensToServers)(servers);
            yield (0, server_commands_1.setDefaultVideoChannel)(servers);
            yield (0, server_commands_1.doubleFollow)(servers[0], servers[1]);
            sqlCommandServer1 = new shared_1.SQLCommand(servers[0]);
            yield servers[0].config.enableRemoteTranscoding();
            yield servers[0].config.enableTranscoding(true, true, true);
            yield servers[0].config.enableLive({ allowReplay: true, resolutions: 'max', transcoding: true });
            const registrationToken = yield servers[0].runnerRegistrationTokens.getFirstRegistrationToken();
            peertubeRunner = new shared_1.PeerTubeRunnerProcess(servers[0]);
            yield peertubeRunner.runServer();
            yield peertubeRunner.registerPeerTubeInstance({ registrationToken, runnerName: 'runner' });
        });
    });
    describe('With lives on local filesystem storage', function () {
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield servers[0].config.enableTranscoding(true, false, true);
            });
        });
        runSuite();
    });
    describe('With lives on object storage', function () {
        if ((0, core_utils_1.areMockObjectStorageTestsDisabled)())
            return;
        const objectStorage = new server_commands_1.ObjectStorageCommand();
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield objectStorage.prepareDefaultMockBuckets();
                yield servers[0].kill();
                yield servers[0].run(objectStorage.getDefaultMockConfig());
                yield (0, core_utils_1.wait)(1500);
            });
        });
        runSuite({ objectStorage });
        after(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield objectStorage.cleanupMock();
            });
        });
    });
    describe('Check cleanup', function () {
        it('Should have an empty cache directory', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield (0, shared_1.checkPeerTubeRunnerCacheIsEmpty)(peertubeRunner);
            });
        });
    });
    after(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (peertubeRunner) {
                yield peertubeRunner.unregisterPeerTubeInstance({ runnerName: 'runner' });
                peertubeRunner.kill();
            }
            if (sqlCommandServer1)
                yield sqlCommandServer1.cleanup();
            yield (0, server_commands_1.cleanupTests)(servers);
        });
    });
});
//# sourceMappingURL=live-transcoding.js.map