"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chai_1 = require("chai");
const shared_1 = require("../shared");
const core_utils_1 = require("../../../shared/core-utils");
const server_commands_1 = require("../../../shared/server-commands");
describe('Test studio transcoding in peertube-runner program', function () {
    let servers = [];
    let peertubeRunner;
    function runSuite(options = {}) {
        const { objectStorage } = options;
        it('Should run a complex studio transcoding', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(120000);
                const { uuid } = yield servers[0].videos.quickUpload({ name: 'mp4', fixture: 'video_short.mp4' });
                yield (0, server_commands_1.waitJobs)(servers);
                const video = yield servers[0].videos.get({ id: uuid });
                const oldFileUrls = (0, core_utils_1.getAllFiles)(video).map(f => f.fileUrl);
                yield servers[0].videoStudio.createEditionTasks({ videoId: uuid, tasks: server_commands_1.VideoStudioCommand.getComplexTask() });
                yield (0, server_commands_1.waitJobs)(servers, { runnerJobs: true });
                for (const server of servers) {
                    const video = yield server.videos.get({ id: uuid });
                    const files = (0, core_utils_1.getAllFiles)(video);
                    for (const f of files) {
                        (0, chai_1.expect)(oldFileUrls).to.not.include(f.fileUrl);
                    }
                    if (objectStorage) {
                        for (const webtorrentFile of video.files) {
                            (0, shared_1.expectStartWith)(webtorrentFile.fileUrl, objectStorage.getMockWebVideosBaseUrl());
                        }
                        for (const hlsFile of video.streamingPlaylists[0].files) {
                            (0, shared_1.expectStartWith)(hlsFile.fileUrl, objectStorage.getMockPlaylistBaseUrl());
                        }
                    }
                    yield (0, shared_1.checkVideoDuration)(server, uuid, 9);
                }
            });
        });
    }
    before(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(120000);
            servers = yield (0, server_commands_1.createMultipleServers)(2);
            yield (0, server_commands_1.setAccessTokensToServers)(servers);
            yield (0, server_commands_1.setDefaultVideoChannel)(servers);
            yield (0, server_commands_1.doubleFollow)(servers[0], servers[1]);
            yield servers[0].config.enableTranscoding(true, true);
            yield servers[0].config.enableStudio();
            yield servers[0].config.enableRemoteStudio();
            const registrationToken = yield servers[0].runnerRegistrationTokens.getFirstRegistrationToken();
            peertubeRunner = new shared_1.PeerTubeRunnerProcess(servers[0]);
            yield peertubeRunner.runServer();
            yield peertubeRunner.registerPeerTubeInstance({ registrationToken, runnerName: 'runner' });
        });
    });
    describe('With videos on local filesystem storage', function () {
        runSuite();
    });
    describe('With videos on object storage', function () {
        if ((0, core_utils_1.areMockObjectStorageTestsDisabled)())
            return;
        const objectStorage = new server_commands_1.ObjectStorageCommand();
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield objectStorage.prepareDefaultMockBuckets();
                yield servers[0].kill();
                yield servers[0].run(objectStorage.getDefaultMockConfig());
                yield (0, core_utils_1.wait)(1500);
            });
        });
        runSuite({ objectStorage });
        after(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield objectStorage.cleanupMock();
            });
        });
    });
    describe('Check cleanup', function () {
        it('Should have an empty cache directory', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield (0, shared_1.checkPeerTubeRunnerCacheIsEmpty)(peertubeRunner);
            });
        });
    });
    after(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (peertubeRunner) {
                yield peertubeRunner.unregisterPeerTubeInstance({ runnerName: 'runner' });
                peertubeRunner.kill();
            }
            yield (0, server_commands_1.cleanupTests)(servers);
        });
    });
});
//# sourceMappingURL=studio-transcoding.js.map