"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chai_1 = require("chai");
const shared_1 = require("../shared");
const core_utils_1 = require("../../../shared/core-utils");
const server_commands_1 = require("../../../shared/server-commands");
describe('Test VOD transcoding in peertube-runner program', function () {
    let servers = [];
    let peertubeRunner;
    function runSuite(options) {
        const { webtorrentEnabled, hlsEnabled, objectStorage } = options;
        const objectStorageBaseUrlWebTorrent = objectStorage
            ? objectStorage.getMockWebVideosBaseUrl()
            : undefined;
        const objectStorageBaseUrlHLS = objectStorage
            ? objectStorage.getMockPlaylistBaseUrl()
            : undefined;
        it('Should upload a classic video mp4 and transcode it', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(120000);
                const { uuid } = yield servers[0].videos.quickUpload({ name: 'mp4', fixture: 'video_short.mp4' });
                yield (0, server_commands_1.waitJobs)(servers, { runnerJobs: true });
                for (const server of servers) {
                    if (webtorrentEnabled) {
                        yield (0, shared_1.completeWebVideoFilesCheck)({
                            server,
                            originServer: servers[0],
                            fixture: 'video_short.mp4',
                            videoUUID: uuid,
                            objectStorageBaseUrl: objectStorageBaseUrlWebTorrent,
                            files: [
                                { resolution: 0 },
                                { resolution: 144 },
                                { resolution: 240 },
                                { resolution: 360 },
                                { resolution: 480 },
                                { resolution: 720 }
                            ]
                        });
                    }
                    if (hlsEnabled) {
                        yield (0, shared_1.completeCheckHlsPlaylist)({
                            hlsOnly: !webtorrentEnabled,
                            servers,
                            videoUUID: uuid,
                            objectStorageBaseUrl: objectStorageBaseUrlHLS,
                            resolutions: [720, 480, 360, 240, 144, 0]
                        });
                    }
                }
            });
        });
        it('Should upload a webm video and transcode it', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(120000);
                const { uuid } = yield servers[0].videos.quickUpload({ name: 'mp4', fixture: 'video_short.webm' });
                yield (0, server_commands_1.waitJobs)(servers, { runnerJobs: true });
                for (const server of servers) {
                    if (webtorrentEnabled) {
                        yield (0, shared_1.completeWebVideoFilesCheck)({
                            server,
                            originServer: servers[0],
                            fixture: 'video_short.webm',
                            videoUUID: uuid,
                            objectStorageBaseUrl: objectStorageBaseUrlWebTorrent,
                            files: [
                                { resolution: 0 },
                                { resolution: 144 },
                                { resolution: 240 },
                                { resolution: 360 },
                                { resolution: 480 },
                                { resolution: 720 }
                            ]
                        });
                    }
                    if (hlsEnabled) {
                        yield (0, shared_1.completeCheckHlsPlaylist)({
                            hlsOnly: !webtorrentEnabled,
                            servers,
                            videoUUID: uuid,
                            objectStorageBaseUrl: objectStorageBaseUrlHLS,
                            resolutions: [720, 480, 360, 240, 144, 0]
                        });
                    }
                }
            });
        });
        it('Should upload an audio only video and transcode it', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(120000);
                const attributes = { name: 'audio_without_preview', fixture: 'sample.ogg' };
                const { uuid } = yield servers[0].videos.upload({ attributes, mode: 'resumable' });
                yield (0, server_commands_1.waitJobs)(servers, { runnerJobs: true });
                for (const server of servers) {
                    if (webtorrentEnabled) {
                        yield (0, shared_1.completeWebVideoFilesCheck)({
                            server,
                            originServer: servers[0],
                            fixture: 'sample.ogg',
                            videoUUID: uuid,
                            objectStorageBaseUrl: objectStorageBaseUrlWebTorrent,
                            files: [
                                { resolution: 0 },
                                { resolution: 144 },
                                { resolution: 240 },
                                { resolution: 360 },
                                { resolution: 480 }
                            ]
                        });
                    }
                    if (hlsEnabled) {
                        yield (0, shared_1.completeCheckHlsPlaylist)({
                            hlsOnly: !webtorrentEnabled,
                            servers,
                            videoUUID: uuid,
                            objectStorageBaseUrl: objectStorageBaseUrlHLS,
                            resolutions: [480, 360, 240, 144, 0]
                        });
                    }
                }
            });
        });
        it('Should upload a private video and transcode it', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(120000);
                const { uuid } = yield servers[0].videos.quickUpload({ name: 'mp4', fixture: 'video_short.mp4', privacy: 3 });
                yield (0, server_commands_1.waitJobs)(servers, { runnerJobs: true });
                if (webtorrentEnabled) {
                    yield (0, shared_1.completeWebVideoFilesCheck)({
                        server: servers[0],
                        originServer: servers[0],
                        fixture: 'video_short.mp4',
                        videoUUID: uuid,
                        objectStorageBaseUrl: objectStorageBaseUrlWebTorrent,
                        files: [
                            { resolution: 0 },
                            { resolution: 144 },
                            { resolution: 240 },
                            { resolution: 360 },
                            { resolution: 480 },
                            { resolution: 720 }
                        ]
                    });
                }
                if (hlsEnabled) {
                    yield (0, shared_1.completeCheckHlsPlaylist)({
                        hlsOnly: !webtorrentEnabled,
                        servers: [servers[0]],
                        videoUUID: uuid,
                        objectStorageBaseUrl: objectStorageBaseUrlHLS,
                        resolutions: [720, 480, 360, 240, 144, 0]
                    });
                }
            });
        });
        it('Should transcode videos on manual run', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.timeout(120000);
                yield servers[0].config.disableTranscoding();
                const { uuid } = yield servers[0].videos.quickUpload({ name: 'manual transcoding', fixture: 'video_short.mp4' });
                yield (0, server_commands_1.waitJobs)(servers, { runnerJobs: true });
                {
                    const video = yield servers[0].videos.get({ id: uuid });
                    (0, chai_1.expect)((0, core_utils_1.getAllFiles)(video)).to.have.lengthOf(1);
                }
                yield servers[0].config.enableTranscoding(true, true, true);
                yield servers[0].videos.runTranscoding({ transcodingType: 'webtorrent', videoId: uuid });
                yield (0, server_commands_1.waitJobs)(servers, { runnerJobs: true });
                yield (0, shared_1.completeWebVideoFilesCheck)({
                    server: servers[0],
                    originServer: servers[0],
                    fixture: 'video_short.mp4',
                    videoUUID: uuid,
                    objectStorageBaseUrl: objectStorageBaseUrlWebTorrent,
                    files: [
                        { resolution: 0 },
                        { resolution: 144 },
                        { resolution: 240 },
                        { resolution: 360 },
                        { resolution: 480 },
                        { resolution: 720 }
                    ]
                });
                yield servers[0].videos.runTranscoding({ transcodingType: 'hls', videoId: uuid });
                yield (0, server_commands_1.waitJobs)(servers, { runnerJobs: true });
                yield (0, shared_1.completeCheckHlsPlaylist)({
                    hlsOnly: false,
                    servers: [servers[0]],
                    videoUUID: uuid,
                    objectStorageBaseUrl: objectStorageBaseUrlHLS,
                    resolutions: [720, 480, 360, 240, 144, 0]
                });
            });
        });
    }
    before(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(120000);
            servers = yield (0, server_commands_1.createMultipleServers)(2);
            yield (0, server_commands_1.setAccessTokensToServers)(servers);
            yield (0, server_commands_1.setDefaultVideoChannel)(servers);
            yield (0, server_commands_1.doubleFollow)(servers[0], servers[1]);
            yield servers[0].config.enableRemoteTranscoding();
            const registrationToken = yield servers[0].runnerRegistrationTokens.getFirstRegistrationToken();
            peertubeRunner = new shared_1.PeerTubeRunnerProcess(servers[0]);
            yield peertubeRunner.runServer();
            yield peertubeRunner.registerPeerTubeInstance({ registrationToken, runnerName: 'runner' });
        });
    });
    describe('With videos on local filesystem storage', function () {
        describe('Web video only enabled', function () {
            before(function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield servers[0].config.enableTranscoding(true, false, true);
                });
            });
            runSuite({ webtorrentEnabled: true, hlsEnabled: false });
        });
        describe('HLS videos only enabled', function () {
            before(function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield servers[0].config.enableTranscoding(false, true, true);
                });
            });
            runSuite({ webtorrentEnabled: false, hlsEnabled: true });
        });
        describe('Web video & HLS enabled', function () {
            before(function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield servers[0].config.enableTranscoding(true, true, true);
                });
            });
            runSuite({ webtorrentEnabled: true, hlsEnabled: true });
        });
    });
    describe('With videos on object storage', function () {
        if ((0, core_utils_1.areMockObjectStorageTestsDisabled)())
            return;
        const objectStorage = new server_commands_1.ObjectStorageCommand();
        before(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield objectStorage.prepareDefaultMockBuckets();
                yield servers[0].kill();
                yield servers[0].run(objectStorage.getDefaultMockConfig());
                yield (0, core_utils_1.wait)(1500);
            });
        });
        describe('Web video only enabled', function () {
            before(function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield servers[0].config.enableTranscoding(true, false, true);
                });
            });
            runSuite({ webtorrentEnabled: true, hlsEnabled: false, objectStorage });
        });
        describe('HLS videos only enabled', function () {
            before(function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield servers[0].config.enableTranscoding(false, true, true);
                });
            });
            runSuite({ webtorrentEnabled: false, hlsEnabled: true, objectStorage });
        });
        describe('Web video & HLS enabled', function () {
            before(function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield servers[0].config.enableTranscoding(true, true, true);
                });
            });
            runSuite({ webtorrentEnabled: true, hlsEnabled: true, objectStorage });
        });
        after(function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield objectStorage.cleanupMock();
            });
        });
    });
    describe('Check cleanup', function () {
        it('Should have an empty cache directory', function () {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield (0, shared_1.checkPeerTubeRunnerCacheIsEmpty)(peertubeRunner);
            });
        });
    });
    after(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (peertubeRunner) {
                yield peertubeRunner.unregisterPeerTubeInstance({ runnerName: 'runner' });
                peertubeRunner.kill();
            }
            yield (0, server_commands_1.cleanupTests)(servers);
        });
    });
});
//# sourceMappingURL=vod-transcoding.js.map