"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SQLCommand = void 0;
const tslib_1 = require("tslib");
const sequelize_1 = require("sequelize");
const core_utils_1 = require("../../../shared/core-utils");
class SQLCommand {
    constructor(server) {
        this.server = server;
    }
    deleteAll(table) {
        const seq = this.getSequelize();
        const options = { type: sequelize_1.QueryTypes.DELETE };
        return seq.query(`DELETE FROM "${table}"`, options);
    }
    getVideoShareCount() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const [{ total }] = yield this.selectQuery(`SELECT COUNT(*) as total FROM "videoShare"`);
            if (total === null)
                return 0;
            return parseInt(total, 10);
        });
    }
    getInternalFileUrl(fileId) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.selectQuery(`SELECT "fileUrl" FROM "videoFile" WHERE id = :fileId`, { fileId })
                .then(rows => rows[0].fileUrl);
        });
    }
    setActorField(to, field, value) {
        return this.updateQuery(`UPDATE actor SET ${this.escapeColumnName(field)} = :value WHERE url = :to`, { value, to });
    }
    setVideoField(uuid, field, value) {
        return this.updateQuery(`UPDATE video SET ${this.escapeColumnName(field)} = :value WHERE uuid = :uuid`, { value, uuid });
    }
    setPlaylistField(uuid, field, value) {
        return this.updateQuery(`UPDATE "videoPlaylist" SET ${this.escapeColumnName(field)} = :value WHERE uuid = :uuid`, { value, uuid });
    }
    countVideoViewsOf(uuid) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const query = 'SELECT SUM("videoView"."views") AS "total" FROM "videoView" ' +
                `INNER JOIN "video" ON "video"."id" = "videoView"."videoId" WHERE "video"."uuid" = :uuid`;
            const [{ total }] = yield this.selectQuery(query, { uuid });
            if (!total)
                return 0;
            return (0, core_utils_1.forceNumber)(total);
        });
    }
    getActorImage(filename) {
        return this.selectQuery(`SELECT * FROM "actorImage" WHERE filename = :filename`, { filename })
            .then(rows => rows[0]);
    }
    setPluginVersion(pluginName, newVersion) {
        return this.setPluginField(pluginName, 'version', newVersion);
    }
    setPluginLatestVersion(pluginName, newVersion) {
        return this.setPluginField(pluginName, 'latestVersion', newVersion);
    }
    setPluginField(pluginName, field, value) {
        return this.updateQuery(`UPDATE "plugin" SET ${this.escapeColumnName(field)} = :value WHERE "name" = :pluginName`, { pluginName, value });
    }
    selectQuery(query, replacements = {}) {
        const seq = this.getSequelize();
        const options = {
            type: sequelize_1.QueryTypes.SELECT,
            replacements
        };
        return seq.query(query, options);
    }
    updateQuery(query, replacements = {}) {
        const seq = this.getSequelize();
        const options = { type: sequelize_1.QueryTypes.UPDATE, replacements };
        return seq.query(query, options);
    }
    getPlaylistInfohash(playlistId) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const query = 'SELECT "p2pMediaLoaderInfohashes" FROM "videoStreamingPlaylist" WHERE id = :playlistId';
            const result = yield this.selectQuery(query, { playlistId });
            if (!result || result.length === 0)
                return [];
            return result[0].p2pMediaLoaderInfohashes;
        });
    }
    setActorFollowScores(newScore) {
        return this.updateQuery(`UPDATE "actorFollow" SET "score" = :newScore`, { newScore });
    }
    setTokenField(accessToken, field, value) {
        return this.updateQuery(`UPDATE "oAuthToken" SET ${this.escapeColumnName(field)} = :value WHERE "accessToken" = :accessToken`, { value, accessToken });
    }
    cleanup() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!this.sequelize)
                return;
            yield this.sequelize.close();
            this.sequelize = undefined;
        });
    }
    getSequelize() {
        if (this.sequelize)
            return this.sequelize;
        const dbname = 'peertube_test' + this.server.internalServerNumber;
        const username = 'peertube';
        const password = 'peertube';
        const host = '127.0.0.1';
        const port = 5432;
        this.sequelize = new sequelize_1.Sequelize(dbname, username, password, {
            dialect: 'postgres',
            host,
            port,
            logging: false
        });
        return this.sequelize;
    }
    escapeColumnName(columnName) {
        return this.getSequelize().escape(columnName)
            .replace(/^'/, '"')
            .replace(/'$/, '"');
    }
}
exports.SQLCommand = SQLCommand;
//# sourceMappingURL=sql-command.js.map