"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FFmpegCommandWrapper = void 0;
const tslib_1 = require("tslib");
const fluent_ffmpeg_1 = tslib_1.__importStar(require("fluent-ffmpeg"));
const core_utils_1 = require("../core-utils");
class FFmpegCommandWrapper {
    constructor(options) {
        this.availableEncoders = options.availableEncoders;
        this.profile = options.profile;
        this.niceness = options.niceness;
        this.tmpDirectory = options.tmpDirectory;
        this.threads = options.threads;
        this.logger = options.logger;
        this.lTags = options.lTags || { tags: [] };
        this.updateJobProgress = options.updateJobProgress;
        this.onEnd = options.onEnd;
        this.onError = options.onError;
    }
    getAvailableEncoders() {
        return this.availableEncoders;
    }
    getProfile() {
        return this.profile;
    }
    getCommand() {
        return this.command;
    }
    debugLog(msg, meta) {
        this.logger.debug(msg, Object.assign(Object.assign({}, meta), this.lTags));
    }
    buildCommand(input) {
        if (this.command)
            throw new Error('Command is already built');
        this.command = (0, fluent_ffmpeg_1.default)(input, {
            niceness: this.niceness,
            cwd: this.tmpDirectory
        });
        if (this.threads > 0) {
            this.command.outputOption('-threads ' + this.threads);
        }
        return this.command;
    }
    runCommand(options = {}) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { silent = false } = options;
            return new Promise((res, rej) => {
                let shellCommand;
                this.command.on('start', cmdline => { shellCommand = cmdline; });
                this.command.on('error', (err, stdout, stderr) => {
                    if (silent !== true)
                        this.logger.error('Error in ffmpeg.', Object.assign({ stdout, stderr, shellCommand }, this.lTags));
                    if (this.onError)
                        this.onError(err);
                    rej(err);
                });
                this.command.on('end', (stdout, stderr) => {
                    this.logger.debug('FFmpeg command ended.', Object.assign({ stdout, stderr, shellCommand }, this.lTags));
                    if (this.onEnd)
                        this.onEnd();
                    res();
                });
                if (this.updateJobProgress) {
                    this.command.on('progress', progress => {
                        if (!progress.percent)
                            return;
                        let percent = Math.round(progress.percent);
                        if (percent < 0)
                            percent = 0;
                        if (percent > 100)
                            percent = 100;
                        this.updateJobProgress(percent);
                    });
                }
                this.command.run();
            });
        });
    }
    static resetSupportedEncoders() {
        FFmpegCommandWrapper.supportedEncoders = undefined;
    }
    getEncoderBuilderResult(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!this.availableEncoders) {
                throw new Error('There is no available encoders');
            }
            const { streamType, videoType } = options;
            const encodersToTry = this.availableEncoders.encodersToTry[videoType][streamType];
            const encoders = this.availableEncoders.available[videoType];
            for (const encoder of encodersToTry) {
                if (!(yield this.checkFFmpegEncoders(this.availableEncoders)).get(encoder)) {
                    this.logger.debug(`Encoder ${encoder} not available in ffmpeg, skipping.`, this.lTags);
                    continue;
                }
                if (!encoders[encoder]) {
                    this.logger.debug(`Encoder ${encoder} not available in peertube encoders, skipping.`, this.lTags);
                    continue;
                }
                const builderProfiles = encoders[encoder];
                let builder = builderProfiles[this.profile];
                if (!builder) {
                    this.logger.debug(`Profile ${this.profile} for encoder ${encoder} not available. Fallback to default.`, this.lTags);
                    builder = builderProfiles.default;
                    if (!builder) {
                        this.logger.debug(`Default profile for encoder ${encoder} not available. Try next available encoder.`, this.lTags);
                        continue;
                    }
                }
                const result = yield builder((0, core_utils_1.pick)(options, [
                    'input',
                    'canCopyAudio',
                    'canCopyVideo',
                    'resolution',
                    'inputBitrate',
                    'fps',
                    'inputRatio',
                    'streamNum'
                ]));
                return {
                    result,
                    encoder: result.copy === true
                        ? 'copy'
                        : encoder
                };
            }
            return null;
        });
    }
    checkFFmpegEncoders(peertubeAvailableEncoders) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (FFmpegCommandWrapper.supportedEncoders !== undefined) {
                return FFmpegCommandWrapper.supportedEncoders;
            }
            const getAvailableEncodersPromise = (0, core_utils_1.promisify0)(fluent_ffmpeg_1.getAvailableEncoders);
            const availableFFmpegEncoders = yield getAvailableEncodersPromise();
            const searchEncoders = new Set();
            for (const type of ['live', 'vod']) {
                for (const streamType of ['audio', 'video']) {
                    for (const encoder of peertubeAvailableEncoders.encodersToTry[type][streamType]) {
                        searchEncoders.add(encoder);
                    }
                }
            }
            const supportedEncoders = new Map();
            for (const searchEncoder of searchEncoders) {
                supportedEncoders.set(searchEncoder, availableFFmpegEncoders[searchEncoder] !== undefined);
            }
            this.logger.info('Built supported ffmpeg encoders.', Object.assign({ supportedEncoders, searchEncoders }, this.lTags));
            FFmpegCommandWrapper.supportedEncoders = supportedEncoders;
            return supportedEncoders;
        });
    }
}
exports.FFmpegCommandWrapper = FFmpegCommandWrapper;
//# sourceMappingURL=ffmpeg-command-wrapper.js.map