"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.canDoQuickVideoTranscode = exports.canDoQuickAudioTranscode = exports.getDefaultEncodersToTry = exports.getDefaultAvailableEncoders = void 0;
const tslib_1 = require("tslib");
const core_utils_1 = require("../core-utils");
const ffmpeg_1 = require("./");
const defaultX264VODOptionsBuilder = (options) => {
    const { fps, inputRatio, inputBitrate, resolution } = options;
    const targetBitrate = getTargetBitrate({ inputBitrate, ratio: inputRatio, fps, resolution });
    return {
        outputOptions: [
            ...getCommonOutputOptions(targetBitrate),
            `-r ${fps}`
        ]
    };
};
const defaultX264LiveOptionsBuilder = (options) => {
    const { streamNum, fps, inputBitrate, inputRatio, resolution } = options;
    const targetBitrate = getTargetBitrate({ inputBitrate, ratio: inputRatio, fps, resolution });
    return {
        outputOptions: [
            ...getCommonOutputOptions(targetBitrate, streamNum),
            `${(0, ffmpeg_1.buildStreamSuffix)('-r:v', streamNum)} ${fps}`,
            `${(0, ffmpeg_1.buildStreamSuffix)('-b:v', streamNum)} ${targetBitrate}`
        ]
    };
};
const defaultAACOptionsBuilder = ({ input, streamNum, canCopyAudio }) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const probe = yield (0, ffmpeg_1.ffprobePromise)(input);
    if (canCopyAudio && (yield canDoQuickAudioTranscode(input, probe))) {
        return { copy: true, outputOptions: [] };
    }
    const parsedAudio = yield (0, ffmpeg_1.getAudioStream)(input, probe);
    const audioCodecName = parsedAudio.audioStream['codec_name'];
    const bitrate = (0, ffmpeg_1.getMaxAudioBitrate)(audioCodecName, parsedAudio.bitrate);
    const base = ['-channel_layout', 'stereo'];
    if (bitrate !== -1) {
        return { outputOptions: base.concat([(0, ffmpeg_1.buildStreamSuffix)('-b:a', streamNum), bitrate + 'k']) };
    }
    return { outputOptions: base };
});
const defaultLibFDKAACVODOptionsBuilder = ({ streamNum }) => {
    return { outputOptions: [(0, ffmpeg_1.buildStreamSuffix)('-q:a', streamNum), '5'] };
};
function getDefaultAvailableEncoders() {
    return {
        vod: {
            libx264: {
                default: defaultX264VODOptionsBuilder
            },
            aac: {
                default: defaultAACOptionsBuilder
            },
            libfdk_aac: {
                default: defaultLibFDKAACVODOptionsBuilder
            }
        },
        live: {
            libx264: {
                default: defaultX264LiveOptionsBuilder
            },
            aac: {
                default: defaultAACOptionsBuilder
            }
        }
    };
}
exports.getDefaultAvailableEncoders = getDefaultAvailableEncoders;
function getDefaultEncodersToTry() {
    return {
        vod: {
            video: ['libx264'],
            audio: ['libfdk_aac', 'aac']
        },
        live: {
            video: ['libx264'],
            audio: ['libfdk_aac', 'aac']
        }
    };
}
exports.getDefaultEncodersToTry = getDefaultEncodersToTry;
function canDoQuickAudioTranscode(path, probe) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const parsedAudio = yield (0, ffmpeg_1.getAudioStream)(path, probe);
        if (!parsedAudio.audioStream)
            return true;
        if (parsedAudio.audioStream['codec_name'] !== 'aac')
            return false;
        const audioBitrate = parsedAudio.bitrate;
        if (!audioBitrate)
            return false;
        const maxAudioBitrate = (0, ffmpeg_1.getMaxAudioBitrate)('aac', audioBitrate);
        if (maxAudioBitrate !== -1 && audioBitrate > maxAudioBitrate)
            return false;
        const channelLayout = parsedAudio.audioStream['channel_layout'];
        if (!channelLayout || channelLayout === 'unknown' || channelLayout === 'quad')
            return false;
        return true;
    });
}
exports.canDoQuickAudioTranscode = canDoQuickAudioTranscode;
function canDoQuickVideoTranscode(path, probe) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const videoStream = yield (0, ffmpeg_1.getVideoStream)(path, probe);
        const fps = yield (0, ffmpeg_1.getVideoStreamFPS)(path, probe);
        const bitRate = yield (0, ffmpeg_1.getVideoStreamBitrate)(path, probe);
        const resolutionData = yield (0, ffmpeg_1.getVideoStreamDimensionsInfo)(path, probe);
        if (!bitRate)
            return false;
        if (!videoStream)
            return false;
        if (videoStream['codec_name'] !== 'h264')
            return false;
        if (videoStream['pix_fmt'] !== 'yuv420p')
            return false;
        if (fps < 2 || fps > 65)
            return false;
        if (bitRate > (0, core_utils_1.getMaxTheoreticalBitrate)(Object.assign(Object.assign({}, resolutionData), { fps })))
            return false;
        return true;
    });
}
exports.canDoQuickVideoTranscode = canDoQuickVideoTranscode;
function getTargetBitrate(options) {
    const { inputBitrate, resolution, ratio, fps } = options;
    const capped = capBitrate(inputBitrate, (0, core_utils_1.getAverageTheoreticalBitrate)({ resolution, fps, ratio }));
    const limit = (0, core_utils_1.getMinTheoreticalBitrate)({ resolution, fps, ratio });
    return Math.max(limit, capped);
}
function capBitrate(inputBitrate, targetBitrate) {
    if (!inputBitrate)
        return targetBitrate;
    const inputBitrateWithMargin = inputBitrate + (inputBitrate * 0.3);
    return Math.min(targetBitrate, inputBitrateWithMargin);
}
function getCommonOutputOptions(targetBitrate, streamNum) {
    return [
        `-preset veryfast`,
        `${(0, ffmpeg_1.buildStreamSuffix)('-maxrate:v', streamNum)} ${targetBitrate}`,
        `${(0, ffmpeg_1.buildStreamSuffix)('-bufsize:v', streamNum)} ${targetBitrate * 2}`,
        `-b_strategy 1`,
        `-bf 16`
    ];
}
//# sourceMappingURL=ffmpeg-default-transcoding-profile.js.map