"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FFmpegEdition = void 0;
const tslib_1 = require("tslib");
const ffmpeg_command_wrapper_1 = require("./ffmpeg-command-wrapper");
const presets_1 = require("./shared/presets");
const ffprobe_1 = require("./ffprobe");
class FFmpegEdition {
    constructor(options) {
        this.commandWrapper = new ffmpeg_command_wrapper_1.FFmpegCommandWrapper(options);
    }
    cutVideo(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { inputPath, outputPath } = options;
            const mainProbe = yield (0, ffprobe_1.ffprobePromise)(inputPath);
            const fps = yield (0, ffprobe_1.getVideoStreamFPS)(inputPath, mainProbe);
            const { resolution } = yield (0, ffprobe_1.getVideoStreamDimensionsInfo)(inputPath, mainProbe);
            const command = this.commandWrapper.buildCommand(inputPath)
                .output(outputPath);
            yield (0, presets_1.presetVOD)({
                commandWrapper: this.commandWrapper,
                input: inputPath,
                resolution,
                fps,
                canCopyAudio: false,
                canCopyVideo: false
            });
            if (options.start) {
                command.outputOption('-ss ' + options.start);
            }
            if (options.end) {
                command.outputOption('-to ' + options.end);
            }
            yield this.commandWrapper.runCommand();
        });
    }
    addWatermark(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { watermarkPath, inputPath, outputPath, videoFilters } = options;
            const videoProbe = yield (0, ffprobe_1.ffprobePromise)(inputPath);
            const fps = yield (0, ffprobe_1.getVideoStreamFPS)(inputPath, videoProbe);
            const { resolution } = yield (0, ffprobe_1.getVideoStreamDimensionsInfo)(inputPath, videoProbe);
            const command = this.commandWrapper.buildCommand(inputPath)
                .output(outputPath);
            command.input(watermarkPath);
            yield (0, presets_1.presetVOD)({
                commandWrapper: this.commandWrapper,
                input: inputPath,
                resolution,
                fps,
                canCopyAudio: true,
                canCopyVideo: false
            });
            const complexFilter = [
                {
                    inputs: ['[1]', '[0]'],
                    filter: 'scale2ref',
                    options: {
                        w: 'oh*mdar',
                        h: `ih*${videoFilters.watermarkSizeRatio}`
                    },
                    outputs: ['[watermark]', '[video]']
                },
                {
                    inputs: ['[video]', '[watermark]'],
                    filter: 'overlay',
                    options: {
                        x: `main_w - overlay_w - (main_h * ${videoFilters.horitonzalMarginRatio})`,
                        y: `main_h * ${videoFilters.verticalMarginRatio}`
                    }
                }
            ];
            command.complexFilter(complexFilter);
            yield this.commandWrapper.runCommand();
        });
    }
    addIntroOutro(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { introOutroPath, inputPath, outputPath, type } = options;
            const mainProbe = yield (0, ffprobe_1.ffprobePromise)(inputPath);
            const fps = yield (0, ffprobe_1.getVideoStreamFPS)(inputPath, mainProbe);
            const { resolution } = yield (0, ffprobe_1.getVideoStreamDimensionsInfo)(inputPath, mainProbe);
            const mainHasAudio = yield (0, ffprobe_1.hasAudioStream)(inputPath, mainProbe);
            const introOutroProbe = yield (0, ffprobe_1.ffprobePromise)(introOutroPath);
            const introOutroHasAudio = yield (0, ffprobe_1.hasAudioStream)(introOutroPath, introOutroProbe);
            const command = this.commandWrapper.buildCommand(inputPath)
                .output(outputPath);
            command.input(introOutroPath);
            if (!introOutroHasAudio && mainHasAudio) {
                const duration = yield (0, ffprobe_1.getVideoStreamDuration)(introOutroPath, introOutroProbe);
                command.input('anullsrc');
                command.withInputFormat('lavfi');
                command.withInputOption('-t ' + duration);
            }
            yield (0, presets_1.presetVOD)({
                commandWrapper: this.commandWrapper,
                input: inputPath,
                resolution,
                fps,
                canCopyAudio: false,
                canCopyVideo: false
            });
            const complexFilter = [
                {
                    inputs: ['1', '0'],
                    filter: 'scale2ref',
                    options: {
                        w: 'iw',
                        h: `ih`
                    },
                    outputs: ['intro-outro', 'main']
                },
                {
                    inputs: ['intro-outro', 'main'],
                    filter: 'scale2ref',
                    options: {
                        w: 'iw',
                        h: `ih`
                    },
                    outputs: ['to-scale', 'main']
                },
                {
                    inputs: 'to-scale',
                    filter: 'drawbox',
                    options: {
                        t: 'fill'
                    },
                    outputs: ['to-scale-bg']
                },
                {
                    inputs: ['1', 'to-scale-bg'],
                    filter: 'scale2ref',
                    options: {
                        w: 'iw',
                        h: 'ih',
                        force_original_aspect_ratio: 'decrease',
                        flags: 'spline'
                    },
                    outputs: ['to-scale', 'to-scale-bg']
                },
                {
                    inputs: ['to-scale-bg', 'to-scale'],
                    filter: 'overlay',
                    options: {
                        x: '(main_w - overlay_w)/2',
                        y: '(main_h - overlay_h)/2'
                    },
                    outputs: 'intro-outro-resized'
                }
            ];
            const concatFilter = {
                inputs: [],
                filter: 'concat',
                options: {
                    n: 2,
                    v: 1,
                    unsafe: 1
                },
                outputs: ['v']
            };
            const introOutroFilterInputs = ['intro-outro-resized'];
            const mainFilterInputs = ['main'];
            if (mainHasAudio) {
                mainFilterInputs.push('0:a');
                if (introOutroHasAudio) {
                    introOutroFilterInputs.push('1:a');
                }
                else {
                    introOutroFilterInputs.push('2:a');
                }
            }
            if (type === 'intro') {
                concatFilter.inputs = [...introOutroFilterInputs, ...mainFilterInputs];
            }
            else {
                concatFilter.inputs = [...mainFilterInputs, ...introOutroFilterInputs];
            }
            if (mainHasAudio) {
                concatFilter.options['a'] = 1;
                concatFilter.outputs.push('a');
                command.outputOption('-map [a]');
            }
            command.outputOption('-map [v]');
            complexFilter.push(concatFilter);
            command.complexFilter(complexFilter);
            yield this.commandWrapper.runCommand();
        });
    }
}
exports.FFmpegEdition = FFmpegEdition;
//# sourceMappingURL=ffmpeg-edition.js.map