"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FFmpegLive = void 0;
const tslib_1 = require("tslib");
const path_1 = require("path");
const core_utils_1 = require("../core-utils");
const ffmpeg_command_wrapper_1 = require("./ffmpeg-command-wrapper");
const ffmpeg_utils_1 = require("./ffmpeg-utils");
const shared_1 = require("./shared");
class FFmpegLive {
    constructor(options) {
        this.commandWrapper = new ffmpeg_command_wrapper_1.FFmpegCommandWrapper(options);
    }
    getLiveTranscodingCommand(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { inputUrl, outPath, toTranscode, bitrate, masterPlaylistName, ratio, hasAudio } = options;
            const command = this.commandWrapper.buildCommand(inputUrl);
            const varStreamMap = [];
            const complexFilter = [
                {
                    inputs: '[v:0]',
                    filter: 'split',
                    options: toTranscode.length,
                    outputs: toTranscode.map(t => `vtemp${t.resolution}`)
                }
            ];
            command.outputOption('-sc_threshold 0');
            (0, shared_1.addDefaultEncoderGlobalParams)(command);
            for (let i = 0; i < toTranscode.length; i++) {
                const streamMap = [];
                const { resolution, fps } = toTranscode[i];
                const baseEncoderBuilderParams = {
                    input: inputUrl,
                    canCopyAudio: true,
                    canCopyVideo: true,
                    inputBitrate: bitrate,
                    inputRatio: ratio,
                    resolution,
                    fps,
                    streamNum: i,
                    videoType: 'live'
                };
                {
                    const streamType = 'video';
                    const builderResult = yield this.commandWrapper.getEncoderBuilderResult(Object.assign(Object.assign({}, baseEncoderBuilderParams), { streamType }));
                    if (!builderResult) {
                        throw new Error('No available live video encoder found');
                    }
                    command.outputOption(`-map [vout${resolution}]`);
                    (0, shared_1.addDefaultEncoderParams)({ command, encoder: builderResult.encoder, fps, streamNum: i });
                    this.commandWrapper.debugLog(`Apply ffmpeg live video params from ${builderResult.encoder} using ${this.commandWrapper.getProfile()} profile.`, { builderResult, fps, toTranscode });
                    command.outputOption(`${(0, ffmpeg_utils_1.buildStreamSuffix)('-c:v', i)} ${builderResult.encoder}`);
                    (0, shared_1.applyEncoderOptions)(command, builderResult.result);
                    complexFilter.push({
                        inputs: `vtemp${resolution}`,
                        filter: (0, ffmpeg_utils_1.getScaleFilter)(builderResult.result),
                        options: `w=-2:h=${resolution}`,
                        outputs: `vout${resolution}`
                    });
                    streamMap.push(`v:${i}`);
                }
                if (hasAudio) {
                    const streamType = 'audio';
                    const builderResult = yield this.commandWrapper.getEncoderBuilderResult(Object.assign(Object.assign({}, baseEncoderBuilderParams), { streamType }));
                    if (!builderResult) {
                        throw new Error('No available live audio encoder found');
                    }
                    command.outputOption('-map a:0');
                    (0, shared_1.addDefaultEncoderParams)({ command, encoder: builderResult.encoder, fps, streamNum: i });
                    this.commandWrapper.debugLog(`Apply ffmpeg live audio params from ${builderResult.encoder} using ${this.commandWrapper.getProfile()} profile.`, { builderResult, fps, resolution });
                    command.outputOption(`${(0, ffmpeg_utils_1.buildStreamSuffix)('-c:a', i)} ${builderResult.encoder}`);
                    (0, shared_1.applyEncoderOptions)(command, builderResult.result);
                    streamMap.push(`a:${i}`);
                }
                varStreamMap.push(streamMap.join(','));
            }
            command.complexFilter(complexFilter);
            this.addDefaultLiveHLSParams(Object.assign(Object.assign({}, (0, core_utils_1.pick)(options, ['segmentDuration', 'segmentListSize'])), { outPath, masterPlaylistName }));
            command.outputOption('-var_stream_map', varStreamMap.join(' '));
            return command;
        });
    }
    getLiveMuxingCommand(options) {
        const { inputUrl, outPath, masterPlaylistName } = options;
        const command = this.commandWrapper.buildCommand(inputUrl);
        command.outputOption('-c:v copy');
        command.outputOption('-c:a copy');
        command.outputOption('-map 0:a?');
        command.outputOption('-map 0:v?');
        this.addDefaultLiveHLSParams(Object.assign(Object.assign({}, (0, core_utils_1.pick)(options, ['segmentDuration', 'segmentListSize'])), { outPath, masterPlaylistName }));
        return command;
    }
    addDefaultLiveHLSParams(options) {
        const { outPath, masterPlaylistName, segmentListSize, segmentDuration } = options;
        const command = this.commandWrapper.getCommand();
        command.outputOption('-hls_time ' + segmentDuration);
        command.outputOption('-hls_list_size ' + segmentListSize);
        command.outputOption('-hls_flags delete_segments+independent_segments+program_date_time');
        command.outputOption(`-hls_segment_filename ${(0, path_1.join)(outPath, '%v-%06d.ts')}`);
        command.outputOption('-master_pl_name ' + masterPlaylistName);
        command.outputOption(`-f hls`);
        command.output((0, path_1.join)(outPath, '%v.m3u8'));
    }
}
exports.FFmpegLive = FFmpegLive;
//# sourceMappingURL=ffmpeg-live.js.map