"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FFmpegVOD = void 0;
const tslib_1 = require("tslib");
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
const core_utils_1 = require("../core-utils");
const ffmpeg_command_wrapper_1 = require("./ffmpeg-command-wrapper");
const ffprobe_1 = require("./ffprobe");
const presets_1 = require("./shared/presets");
class FFmpegVOD {
    constructor(options) {
        this.ended = false;
        this.commandWrapper = new ffmpeg_command_wrapper_1.FFmpegCommandWrapper(options);
    }
    transcode(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const builders = {
                'quick-transcode': this.buildQuickTranscodeCommand.bind(this),
                'hls': this.buildHLSVODCommand.bind(this),
                'hls-from-ts': this.buildHLSVODFromTSCommand.bind(this),
                'merge-audio': this.buildAudioMergeCommand.bind(this),
                'only-audio': this.buildOnlyAudioCommand.bind(this),
                'video': this.buildWebVideoCommand.bind(this)
            };
            this.commandWrapper.debugLog('Will run transcode.', { options });
            const command = this.commandWrapper.buildCommand(options.inputPath)
                .output(options.outputPath);
            yield builders[options.type](options);
            command.on('start', () => {
                setTimeout(() => {
                    options.inputFileMutexReleaser();
                }, 1000);
            });
            yield this.commandWrapper.runCommand();
            yield this.fixHLSPlaylistIfNeeded(options);
            this.ended = true;
        });
    }
    isEnded() {
        return this.ended;
    }
    buildWebVideoCommand(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { resolution, fps, inputPath } = options;
            if (resolution === 0) {
                (0, presets_1.presetOnlyAudio)(this.commandWrapper);
                return;
            }
            let scaleFilterValue;
            if (resolution !== undefined) {
                const probe = yield (0, ffprobe_1.ffprobePromise)(inputPath);
                const videoStreamInfo = yield (0, ffprobe_1.getVideoStreamDimensionsInfo)(inputPath, probe);
                scaleFilterValue = (videoStreamInfo === null || videoStreamInfo === void 0 ? void 0 : videoStreamInfo.isPortraitMode) === true
                    ? `w=${resolution}:h=-2`
                    : `w=-2:h=${resolution}`;
            }
            yield (0, presets_1.presetVOD)({
                commandWrapper: this.commandWrapper,
                resolution,
                input: inputPath,
                canCopyAudio: true,
                canCopyVideo: true,
                fps,
                scaleFilterValue
            });
        });
    }
    buildQuickTranscodeCommand(_options) {
        const command = this.commandWrapper.getCommand();
        (0, presets_1.presetCopy)(this.commandWrapper);
        command.outputOption('-map_metadata -1')
            .outputOption('-movflags faststart');
    }
    buildAudioMergeCommand(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const command = this.commandWrapper.getCommand();
            command.loop(undefined);
            yield (0, presets_1.presetVOD)(Object.assign(Object.assign({}, (0, core_utils_1.pick)(options, ['resolution'])), { commandWrapper: this.commandWrapper, input: options.audioPath, canCopyAudio: true, canCopyVideo: true, fps: options.fps, scaleFilterValue: this.getMergeAudioScaleFilterValue() }));
            command.outputOption('-preset:v veryfast');
            command.input(options.audioPath)
                .outputOption('-tune stillimage')
                .outputOption('-shortest');
        });
    }
    buildOnlyAudioCommand(_options) {
        (0, presets_1.presetOnlyAudio)(this.commandWrapper);
    }
    getMergeAudioScaleFilterValue() {
        return 'trunc(iw/2)*2:trunc(ih/2)*2';
    }
    buildHLSVODCommand(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const command = this.commandWrapper.getCommand();
            const videoPath = this.getHLSVideoPath(options);
            if (options.copyCodecs)
                (0, presets_1.presetCopy)(this.commandWrapper);
            else if (options.resolution === 0)
                (0, presets_1.presetOnlyAudio)(this.commandWrapper);
            else
                yield this.buildWebVideoCommand(options);
            this.addCommonHLSVODCommandOptions(command, videoPath);
        });
    }
    buildHLSVODFromTSCommand(options) {
        const command = this.commandWrapper.getCommand();
        const videoPath = this.getHLSVideoPath(options);
        command.outputOption('-c copy');
        if (options.isAAC) {
            command.outputOption('-bsf:a aac_adtstoasc');
        }
        this.addCommonHLSVODCommandOptions(command, videoPath);
    }
    addCommonHLSVODCommandOptions(command, outputPath) {
        return command.outputOption('-hls_time 4')
            .outputOption('-hls_list_size 0')
            .outputOption('-hls_playlist_type vod')
            .outputOption('-hls_segment_filename ' + outputPath)
            .outputOption('-hls_segment_type fmp4')
            .outputOption('-f hls')
            .outputOption('-hls_flags single_file');
    }
    fixHLSPlaylistIfNeeded(options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (options.type !== 'hls' && options.type !== 'hls-from-ts')
                return;
            const fileContent = yield (0, fs_extra_1.readFile)(options.outputPath);
            const videoFileName = options.hlsPlaylist.videoFilename;
            const videoFilePath = this.getHLSVideoPath(options);
            const newContent = fileContent.toString()
                .replace(`#EXT-X-MAP:URI="${videoFilePath}",`, `#EXT-X-MAP:URI="${videoFileName}",`);
            yield (0, fs_extra_1.writeFile)(options.outputPath, newContent);
        });
    }
    getHLSVideoPath(options) {
        return `${(0, path_1.dirname)(options.outputPath)}/${options.hlsPlaylist.videoFilename}`;
    }
}
exports.FFmpegVOD = FFmpegVOD;
//# sourceMappingURL=ffmpeg-vod.js.map